package com.randomsoftware.dance;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Game
{
   public Game()
   {
      _playback = new Playback();

      _song = new Song("m j c b  mc  jb  m b c j () jb "); // TODO: temporary
      _playback.resume(); // TODO: temporary
   }

   public Playback playback()
   {
      return _playback;
   }

   public void onAppSuspend()
   {
      _playback.suspend();
   }

   public void onAppResume()
   {
      // TODO: show menu
   }

   public void update()
   {
      _playback.update();
   }

   public void paint (Pen pp)
   {
      ResourceManager res = Dance.resmgr();

      // The playback 'position' marker represents the exact time-index at which a
      // particular key is best struck; that is, when the position reaches ten
      // notes into the song, the center of the tenth note's picture should
      // visually be passing directly through the center of the hot zone.

      int cxWidth = Config.KEY_ICON_SIZE;       // tile size
      int cyHeight = Config.KEY_ICON_SIZE;      // tile size
      int cxHalfWidth = 18;   // half of tile size, rounded up
      int xHotZone = 40;      // X-position of center of hot zone
      long leadTime = 2000;   // amount of music shown ahead of hotzone
      long lossTime = 500;    // amount of music shown behind hotzone

      long timePerNote = _playback.getRate();
      long timeHotZone = _playback.getPosition();
      long timeLeftEdge = timeHotZone - lossTime;
      long timeRightEdge = timeHotZone + leadTime;

      for (int iNote = (int)(timeLeftEdge / timePerNote); iNote < _song.length; iNote++)
      {
         // See if this note should be drawn

         if (iNote < 0)
            continue;
         long timeAtNote = iNote * timePerNote;
         int ixCenter = (int)((timeAtNote - timeLeftEdge) * Config.SCREEN_WIDTH / (timeRightEdge - timeLeftEdge));
         if (ixCenter < 0-cxHalfWidth)
            continue;
         if (ixCenter > Config.SCREEN_WIDTH + cxHalfWidth)
            continue;

         // What note is it? It may have either one or two icons.

         int keysNote = _song.keys[ iNote ];

         int countAtTop = (((keysNote & Keys.KEY_MENU) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_CANCEL) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_LINDEX) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_HOOK) != 0) ? 1 : 0);
         int countAtMid = (((keysNote & Keys.KEY_UP) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_DOWN) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_LEFT) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_RIGHT) != 0) ? 1 : 0);
         int countAtBtm = (((keysNote & Keys.KEY_BACK) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_JUMP) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_RINDEX) != 0) ? 1 : 0) +
                          (((keysNote & Keys.KEY_SEND) != 0) ? 1 : 0);

         // Decide where to draw those icons.

         int iy1;
         int iy2;
         if ((countAtTop + countAtBtm + countAtMid) == 1)
         {
            if (countAtTop != 0)
               iy1 = iy2 = TOP_POSITIONS[ iNote % TOP_POSITIONS.length ];
            else if (countAtBtm != 0)
               iy1 = iy2 = BOTTOM_POSITIONS[ iNote % BOTTOM_POSITIONS.length ];
            else
               iy1 = iy2 = MID_POSITIONS[ iNote % MID_POSITIONS.length ];
         }
         else
         {
            iy1 = TOP_POSITIONS[ iNote % TOP_POSITIONS.length ];
            iy2 = BOTTOM_POSITIONS[ iNote % BOTTOM_POSITIONS.length ];
         }

         // Draw them

         int yy = iy1;
         for (int ii = 0; ii < 16; ++ii)
         {
            if ((keysNote & (1 << ii)) != 0)
            {
               pp.drawBitmap (ixCenter - cxHalfWidth, yy, res.bmpKeys, res.getKeyIconRect(1 << ii, false));
               yy = iy2;
            }
         }
      }
   }

   Playback _playback;
   Song _song;

   static int[] TOP_POSITIONS = new int[] { 8, 30, 19, 30, 8, 19, 8 };
   static int[] BOTTOM_POSITIONS = new int[] { 84, 73, 95, 73, 95, 84, 73 };
   static int[] MID_POSITIONS = new int[] { 73, 30, 84, 8, 52, 19, 95 };
}

