package com.randomsoftware.dance;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Playback extends Listener
{
   public Playback()
   {
      _keysDown = 0;
      _running = false;
      _timerMenu = null;
      _timerJump = null;
      _rate = SLOW_RATE;
      _position = 0;
      _startTime = 0;
   }

   public void suspend()
   {
      if (_running)
      {
         _keysDown = 0;
         _running = false;
         cancelMenuTimer();
         cancelJumpTimer();
      }
   }

   public void resume()
   {
      if (!_running)
      {
         _running = true;
         _startTime = System.currentTimeMillis() - _position;
      }
   }

   public void update()
   {
      if (_running)
      {
         _position = System.currentTimeMillis() - _startTime;
      }
   }

   public boolean isRunning()
   {
      return _running;
   }

   public long getPosition()
   {
      return _position;
   }

   public long getRate()
   {
      return _rate;
   }

   public void onKeyDown (int rawKey)
   {
      if (_running)
      {
         int trkey = Keys.translateKey(rawKey);
         _keysDown |= trkey;

         if (trkey == Keys.KEY_JUMP)
            startJumpTimer();
         else if (trkey == Keys.KEY_MENU)
            startMenuTimer();
      }
   }

   public void onKeyUp (int rawKey)
   {
      if (_running)
      {
         int trkey = Keys.translateKey(rawKey);
         _keysDown &= ~trkey;

         if (trkey == Keys.KEY_JUMP)
            cancelJumpTimer();
         else if (trkey == Keys.KEY_MENU)
            cancelMenuTimer();
      }
   }

   void startJumpTimer()
   {
      if (_timerJump == null)
      {
         _timerJump = new danger.app.Timer (TIMER_DELAY, false, this, JUMPTIMER, null);
         _timerJump.start();
      }
   }

   void startMenuTimer()
   {
      if (_timerMenu == null)
      {
         _timerMenu = new danger.app.Timer (TIMER_DELAY, false, this, MENUTIMER, null);
         _timerMenu.start();
      }
   }

   void cancelJumpTimer()
   {
      if (_timerJump != null)
      {
         _timerJump.stop();
         _timerJump = null;
      }
   }

   void cancelMenuTimer()
   {
      if (_timerMenu != null)
      {
         _timerMenu.stop();
         _timerMenu = null;
      }
   }

   public boolean receiveEvent (Event e)
   {
      if ((e.type == EventType.EVENT_TIMER) && (e.data == JUMPTIMER))
      {
         onJumpTimer();
      }
      else if ((e.type == EventType.EVENT_TIMER) && (e.data == MENUTIMER))
      {
         onMenuTimer();
      }
      return false;
   }

   void onJumpTimer()
   {
      suspend();
      Dance.app().returnToLauncher();
   }

   void onMenuTimer()
   {
      suspend();
      // TODO: show the menu
   }

   int _keysDown;
   boolean _running;
   danger.app.Timer _timerMenu;
   danger.app.Timer _timerJump;
   long _rate;
   long _position;
   long _startTime;

   static final int TIMER_DELAY = 1500;
   static final int JUMPTIMER = 1;
   static final int MENUTIMER = 2;

   static final long SLOW_RATE = 750;
   static final long FAST_RATE = 250;
}

