package com.randomsoftware.threed;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Controller implements InputListener
{
   public Controller()
   {
      _active = false;
   }

   public void suspend()
   {
      if (_active)
      {
         Meta.unmuteID(Meta.FEEDBACK_KEY_ONCE);
         Meta.unmuteID(Meta.FEEDBACK_KEY_REPEAT);
         Meta.unmuteID(Meta.FEEDBACK_BACK_BUTTON);
         Meta.unmuteID(Meta.FEEDBACK_CANCEL_BUTTON);
         Meta.unmuteID(Meta.FEEDBACK_MENU_BUTTON);
         Meta.unmuteID(Meta.FEEDBACK_WHEEL_BUTTON);
         Meta.unmuteID(Meta.SCROLL_WHEEL_DOWN);
         Meta.unmuteID(Meta.SCROLL_WHEEL_UP);
         _active = false;
      }
   }

   public void resume()
   {
      if (!_active)
      {
         Meta.muteID(Meta.FEEDBACK_KEY_ONCE);
         Meta.muteID(Meta.FEEDBACK_KEY_REPEAT);
         Meta.muteID(Meta.FEEDBACK_BACK_BUTTON);
         Meta.muteID(Meta.FEEDBACK_CANCEL_BUTTON);
         Meta.muteID(Meta.FEEDBACK_MENU_BUTTON);
         Meta.muteID(Meta.FEEDBACK_WHEEL_BUTTON);
         Meta.muteID(Meta.SCROLL_WHEEL_DOWN);
         Meta.muteID(Meta.SCROLL_WHEEL_UP);
         _active = true;
         _leftDown = false;
         _rightDown = false;
         _upDown = false;
         _downDown = false;
      }
   }

   public boolean consume (int c)
   {
      switch (c) {
         case Event.DEVICE_ARROW_DOWN:
         case Event.DEVICE_ARROW_LEFT:
         case Event.DEVICE_ARROW_RIGHT:
         case Event.DEVICE_ARROW_UP:
         case Event.DEVICE_ARROW_DOWN_LEFT:
         case Event.DEVICE_ARROW_DOWN_RIGHT:
         case Event.DEVICE_ARROW_UP_LEFT:
         case Event.DEVICE_ARROW_UP_RIGHT:
         case Event.DEVICE_WHEEL_BUTTON:
         case Event.DEVICE_WHEEL:
            return true;
      }
      return false;
   }

   public boolean eventKeyDown (char c, Event e)
   {
      switch (c) {
         case 'p':
            paintTrial(); break;
         case 't':
            timeTrial(); break;
         case 'x':
            miscTrial(); break;
      }
      return consume(c);
   }

   public boolean eventKeyUp (char c, Event e)
   {
      return consume(c);
   }

   public boolean eventKeyRepeat (char c, Event e)
   {
      return consume(c);
   }

   public boolean eventWidgetDown (int c, Event e)
   {
      switch (c) {
         case Event.DEVICE_ARROW_DOWN:
            moveCamera (DOWN_DOWN); break;
         case Event.DEVICE_ARROW_LEFT:
            moveCamera (LEFT_DOWN); break;
         case Event.DEVICE_ARROW_RIGHT:
            moveCamera (RIGHT_DOWN); break;
         case Event.DEVICE_ARROW_UP:
            moveCamera (UP_DOWN); break;
         case Event.DEVICE_ARROW_DOWN_LEFT:
            moveCamera (DOWN_DOWN | LEFT_DOWN); break;
         case Event.DEVICE_ARROW_DOWN_RIGHT:
            moveCamera (DOWN_DOWN | RIGHT_DOWN); break;
         case Event.DEVICE_ARROW_UP_LEFT:
            moveCamera (UP_DOWN | LEFT_DOWN); break;
         case Event.DEVICE_ARROW_UP_RIGHT:
            moveCamera (UP_DOWN | RIGHT_DOWN); break;
         case Event.DEVICE_WHEEL:
            zoomCamera (1);  break;
      }
      return consume(c);
   }

   public boolean eventWidgetUp (int c, Event e)
   {
      switch (c) {
         case Event.DEVICE_ARROW_DOWN:
            moveCamera (DOWN_UP); break;
         case Event.DEVICE_ARROW_LEFT:
            moveCamera (LEFT_UP); break;
         case Event.DEVICE_ARROW_RIGHT:
            moveCamera (RIGHT_UP); break;
         case Event.DEVICE_ARROW_UP:
            moveCamera (UP_UP); break;
         case Event.DEVICE_ARROW_DOWN_LEFT:
            moveCamera (DOWN_UP | LEFT_UP); break;
         case Event.DEVICE_ARROW_DOWN_RIGHT:
            moveCamera (DOWN_UP | RIGHT_UP); break;
         case Event.DEVICE_ARROW_UP_LEFT:
            moveCamera (UP_UP | LEFT_UP); break;
         case Event.DEVICE_ARROW_UP_RIGHT:
            moveCamera (UP_UP | RIGHT_UP); break;
         case Event.DEVICE_WHEEL:
            zoomCamera (-1);  break;
      }
      return consume(c);
   }

   public boolean eventWidgetHeld (int c, Event e)
   {
      return consume(c);
   }

   public void moveCamera (int dd)
   {
      if ((dd & DOWN_DOWN) != 0)
         _downDown = true;
      if ((dd & LEFT_DOWN) != 0)
         _leftDown = true;
      if ((dd & RIGHT_DOWN) != 0)
         _rightDown = true;
      if ((dd & UP_DOWN) != 0)
         _upDown = true;
      if ((dd & DOWN_UP) != 0)
         _downDown = false;
      if ((dd & LEFT_UP) != 0)
         _leftDown = false;
      if ((dd & RIGHT_UP) != 0)
         _rightDown = false;
      if ((dd & UP_UP) != 0)
         _upDown = false;

      if (ThreeD.window()._test != null) {
         ThreeD.window()._test.moveCamera (
               (_leftDown) ? -1 : (_rightDown) ? 1 : 0,
               (_upDown) ? 1 : (_downDown) ? -1 : 0
            );
      }
   }

   public void zoomCamera (int dz)
   {
      if (ThreeD.window()._test != null)
         ThreeD.window()._test.zoomCamera (dz);
   }

   public void paintTrial()
   {
      if (ThreeD.window()._test != null)
         ThreeD.window()._test.paintTrial();
   }

   public void timeTrial()
   {
      if (ThreeD.window()._test != null)
         ThreeD.window()._test.timeTrial();
   }

   public void miscTrial()
   {
      if (ThreeD.window()._test != null)
         ThreeD.window()._test.miscTrial();
   }

   static final int DOWN_DOWN = 1;
   static final int LEFT_DOWN = 2;
   static final int RIGHT_DOWN = 4;
   static final int UP_DOWN = 8;
   static final int DOWN_UP = 16;
   static final int LEFT_UP = 32;
   static final int RIGHT_UP = 64;
   static final int UP_UP = 128;

   boolean _active;
   boolean _leftDown;
   boolean _rightDown;
   boolean _upDown;
   boolean _downDown;
}

