package com.randomsoftware.threed;
import com.randomsoftware.common.animation.*;
import com.randomsoftware.common.resloader.*;
import com.randomsoftware.common.meshlib.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Test
{
   static final boolean SHOW_TRIANGLES = false;
   static final boolean SHOW_TEAPOT = true;
   static final boolean USE_LIGHTING = false;
   static final boolean USE_MOVING_LIGHTS = false;
   static final boolean USE_CULLING = false;
   static final boolean USE_FOG = true;
   static final boolean OUTLINE_POLYGONS = false;

   public Test()
   {
      _scene = new Scene();
      _cam = new Camera(_scene, ThreeD.window(), 0);
      _dxEachFrame = 0;
      _dyEachFrame = 0;
      _triCount = 0;

      Rect rr = new Rect();
      rr.left = rr.top = 0;
      rr.right = 240;
      rr.bottom = 160;
      _cam.setRect (rr);
      _cam.setProjection (FixedMath.HEXIAN_45, 1<<FixedMath.SHIFT, 1000<<FixedMath.SHIFT);

      if (SHOW_TRIANGLES)
      {
         Mesh tri = new Mesh (_scene);
         Texture red = tri.addTexture (new Color(Color.RED));
         Texture green = tri.addTexture (new Color(Color.GREEN));

         Vertex v1a = tri.addVertex (new Triplet (0, 0, 0));
         Vertex v1b = tri.addVertex (new Triplet (100 << FixedMath.SHIFT, 0, 0));
         Vertex v1c = tri.addVertex (new Triplet (0, 100 << FixedMath.SHIFT, 0));
         Face f1 = tri.addFace (v1a, v1b, v1c, red);
         f1.isDoubleSided = true;
         _triCount++;

         Vertex v2a = tri.addVertex (new Triplet (0, 0, -200 << FixedMath.SHIFT));
         Vertex v2b = tri.addVertex (new Triplet (100 << FixedMath.SHIFT, 0, -200 << FixedMath.SHIFT));
         Vertex v2c = tri.addVertex (new Triplet (100 << FixedMath.SHIFT, 100 << FixedMath.SHIFT, -200 << FixedMath.SHIFT));
         Face f2 = tri.addFace (v2a, v2b, v2c, green);
         f2.isDoubleSided = true;
         _triCount++;
      }

      if (SHOW_TEAPOT)
      {
         Mesh pot = new Mesh (_scene, ThreeD.resmgr().mrTeapot);
         _triCount += pot.countFaces;
      }

      if (USE_LIGHTING)
      {
         _cam.lighting = Camera.LIGHTING_FLAT;
         _l1 = new Light (_scene, new Color(255,255,255), new Triplet (0, 500<<FixedMath.SHIFT, -20<<FixedMath.SHIFT));
         _l2 = new Light (_scene, new Color(64,64,255), new Triplet (-500<<FixedMath.SHIFT, -100<<FixedMath.SHIFT, 20<<FixedMath.SHIFT));
         _l3 = new Light (_scene, new Color(255,64,64), new Triplet (500<<FixedMath.SHIFT, -100<<FixedMath.SHIFT, 20<<FixedMath.SHIFT));

         if (USE_MOVING_LIGHTS)
         {
            _mxLights = new FixedMatrix();
            _mxLights.addRotation (new Rotation (FixedMath.HEXIAN_90 / 40, FixedMath.HEXIAN_90 / 128));
         }
      }

      if (USE_CULLING)
      {
         _cam.cull = Camera.CULL_BACKFACE;
      }

      if (USE_FOG)
      {
         _cam.fog = Camera.FOG_BLACK;
         _cam.fogNearPlane = MIN_FOG;
         _cam.fogFarPlane = MAX_FOG;
      }

      if (OUTLINE_POLYGONS)
      {
         _cam.quality = Camera.QUALITY_SOLID_OUTLINED;
      }

      _dCamera = DEFAULT_ZOOM;
      _rotCamera = new Rotation (FixedMath.HEXIAN_270, 0);
      updateCamera();
   }

   public void destroy()
   {
      _scene.destroy();
      _cam.destroy();
   }

   public void moveCamera (int dx, int dy)
   {
      _dxEachFrame = dx;
      _dyEachFrame = dy;
   }

   public void zoomCamera (int dz)
   {
      _dCamera += ADJUST_ZOOM * dz;
      if (_dCamera > MAX_ZOOM)
         _dCamera = MAX_ZOOM;
      else if (_dCamera < MIN_ZOOM)
         _dCamera = MIN_ZOOM;
      updateCamera();
   }

   public void updateCamera()
   {
      long cosGamma = FixedMath.cosine(_rotCamera.gamma);

      long vx = FixedMath.multiply(_dCamera, FixedMath.cosine(_rotCamera.theta));
      long vy = FixedMath.multiply(_dCamera, FixedMath.sine(_rotCamera.gamma));
      long vz = 0-FixedMath.multiply(_dCamera, FixedMath.sine(_rotCamera.theta));
      vx = FixedMath.multiply(vx, cosGamma);
      vz = FixedMath.multiply(vz, cosGamma);

      _cam.setSource (new Triplet (vx, vy, vz));
      _cam.setTarget (new Triplet (0, 0, 0));
      _cam.touch();
   }

   public void update()
   {
      if ((_dxEachFrame != 0) || (_dyEachFrame != 0))
      {
         _rotCamera.theta = FixedMath.normalize (_rotCamera.theta + _dxEachFrame * ADJUST_THETA);
         _rotCamera.gamma += _dyEachFrame * ADJUST_GAMMA;
         if (_rotCamera.gamma > MAX_GAMMA)
            _rotCamera.gamma = MAX_GAMMA;
         else if (_rotCamera.gamma < 0-MAX_GAMMA)
            _rotCamera.gamma = 0-MAX_GAMMA;

         updateCamera();
      }

      if (_mxLights != null)
      {
         _temp.makeMultiply (_mxLights, _l2);
         _l2.setPos (_temp);

         _temp.makeMultiply (_mxLights, _l3);
         _l3.setPos (_temp);
      }
   }

   public void paintTrial()
   {
      Bitmap bmp = new Bitmap (240, 160);
      Pen pp = bmp.createPen();
      long timeStart = System.currentTimeMillis();
      int frameCount = 0;

      long timeStop;
      while ((timeStop = System.currentTimeMillis()) < timeStart+5000)
      {
         _cam.paint(pp);
         ++frameCount;
      }

      long elapsed = timeStop - timeStart;
      long triTotal = _triCount * frameCount;

      DEBUG.p("PAINT TRIAL FINISHED: rendered "+(triTotal*1000/elapsed)+" triangles per second");
   }

   public void timeTrial()
   {
      Bitmap bmp = new Bitmap (240, 160);
      Pen pp = bmp.createPen();
      long timeStart = System.currentTimeMillis();
      int frameCount = 0;

      long timeStop;
      while ((timeStop = System.currentTimeMillis()) < timeStart+5000)
      {
         _scene.invalidate();
         _cam.paint(pp);
         ++frameCount;
      }

      long elapsed = timeStop - timeStart;
      long triTotal = _triCount * frameCount;

      DEBUG.p("TIME TRIAL FINISHED: rendered "+(triTotal*1000/elapsed)+" triangles per second");
   }

   public void miscTrial()
   {
      {
         long timeStart = System.currentTimeMillis();
         long count = 0;

         long timeStop;
         while ((timeStop = System.currentTimeMillis()) < timeStart+1000)
         {
            long xx = FixedMath.squareroot_unshifted (123121982312098312L);
            ++count;
         }

         long elapsed = timeStop - timeStart;
         DEBUG.p("MISC TRIAL MY ALGO: did "+(count*1000/elapsed)+" ops per second");
      }
      {
         long timeStart = System.currentTimeMillis();
         long count = 0;

         long timeStop;
         while ((timeStop = System.currentTimeMillis()) < timeStart+1000)
         {
            double xx = Math.sqrt (123121982312098312.0);
            ++count;
         }

         long elapsed = timeStop - timeStart;
         DEBUG.p("MISC TRIAL DOUBLEMATH: did "+(count*1000/elapsed)+" ops per second");
      }
   }

   static final long MIN_ZOOM = 5 << FixedMath.SHIFT;
   static final long MAX_ZOOM = 500 << FixedMath.SHIFT;
   static final long ADJUST_ZOOM = 5 << FixedMath.SHIFT;
   static final long DEFAULT_ZOOM = 300 << FixedMath.SHIFT;

   static final long MIN_FOG = 200 << FixedMath.SHIFT;
   static final long MAX_FOG = 400 << FixedMath.SHIFT;

   static final long MAX_GAMMA = FixedMath.HEXIAN_90 * 9 / 10;
   static final long ADJUST_GAMMA = FixedMath.HEXIAN_90 / 20;

   static final long MAX_THETA = FixedMath.HEXIAN_90 * 9 / 10;
   static final long ADJUST_THETA = FixedMath.HEXIAN_360 / 60;

   Scene _scene;
   Camera _cam;
   long _dCamera;
   Rotation _rotCamera;
   long _dxEachFrame;
   long _dyEachFrame;

   long _triCount;
   Light _l1, _l2, _l3;
   FixedMatrix _mxLights = null;
   Triplet _temp = new Triplet();
}

