package com.randomsoftware.xaiden;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class MainWindow extends AnimWindow
{
   public MainWindow (InputListener input)
   {
      super (input, AnimWindow.LARGE_WINDOW);
      setFullCoverage (true);

      _bmp = Xaiden.res().getOpaqueBitmap (Resources.IDB_BACKGROUND);
      _loc = new ObjectLocation(Layout.BACKGROUND_PARALLAX);
      _rr = new Rect();
      _rr.left = _rr.top = 0;
      _rr.right = Layout.WIDTH_SCREEN;
      _rr.bottom = Layout.HEIGHT_SCREEN;
      _xCorner = 0;
      _yCorner = 0;
      _resmgr = null;
      _menu = null;
      _needMenu = true;
   }

   public void paintBackground (Pen pp)
   {
      if (!ready())
         return;
      if (Layout.BACKGROUND_PARALLAX != 0)
         PaintUtil.tileBitmap (pp, _rr, _bmp, _xCorner, _yCorner);
      else
         pp.drawBitmap (0, 0, _bmp);
   }

   public void update()
   {
      if (!ready())
         return;

      if (_needMenu)
      {
         _needMenu = false;
         showMenu();
      }

      Xaiden.game().onFrame();
   }

   public void adjust (ObjectMotion om)
   {
      if (!ready())
         return;

      if (Layout.BACKGROUND_PARALLAX != 0)
      {
         _loc.moveBackwards (om);

         int xx = (int)_loc.xx;
         int yy = (int)_loc.yy;
         if (xx != _xCorner || yy != _yCorner)
         {
            _xCorner = xx;
            _yCorner = yy;
            touch (_rr);
         }
      }
   }

   public void showMenu()
   {
      if (_menu != null)
         _menu.destroy();
      if (Xaiden.game().haveGame())
         _menu = new GameMenu (GameMenu.MENU_RESUME);
      else
         _menu = new GameMenu (GameMenu.MENU_MAIN);
   }

   public void hideMenu()
   {
      if (_menu != null)
         _menu.destroy();
      _menu = null;
   }

   public GameMenu menu()
   {
      return _menu;
   }

   public boolean ready()
   {
      return _resmgr.ready();
   }

   public ResourceManager resmgr()
   {
      return _resmgr;
   }

   public void show()
   {
      _resmgr = new ResourceManager();
      super.show();
   }

   public void hide()
   {
      _resmgr.destroy();
      _resmgr = null;
      super.hide();
   }

   Bitmap _bmp;
   ObjectLocation _loc;
   Rect _rr;
   int _xCorner;
   int _yCorner;
   GameMenu _menu;
   ResourceManager _resmgr;
   boolean _needMenu;
}

