package com.randomsoftware.xaiden;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class MapSprite extends AnimSprite
{
   public MapSprite (AnimWindow win, int depth)
   {
      super(win,depth);
      _parallax = 1.0f;
      _dxOutOfPlace = _dyOutOfPlace = 0;
      _rNew = new Rect();
      _mapIndex = -1;
   }

   public MapSprite (AnimWindow win, int depth, float plx)
   {
      super(win,depth);
      _parallax = plx;
      _dxOutOfPlace = _dyOutOfPlace = 0;
      _rNew = new Rect();
      _mapIndex = -1;
   }

   void setWorldPlacement (float xWorld, float yWorld, Rect size)
   {
      int cx = size.right - size.left;
      int cy = size.bottom - size.top;

      Rect rr = new Rect();
      rr.left = (int)(xWorld * Layout.WORLD_SCALE) - cx/2 + Layout.WIDTH_SCREEN/2;
      rr.top = (int)(yWorld * Layout.WORLD_SCALE) - cy/2 + Layout.HEIGHT_SCREEN/2;
      rr.right = rr.left + cx;
      rr.bottom = rr.top + cy;
      setRect(rr);
   }

   public void adjust (ObjectMotion shipMotion)
   {
      moveBackwards (shipMotion);
   }

   public void update()
   {
      adjustSprite();
   }

   public void move (ObjectMotion om)
   {
      _dxOutOfPlace += om.dx * _parallax;
      _dyOutOfPlace += om.dy * _parallax;
   }

   public void moveMultiplied (ObjectMotion om, float ff)
   {
      _dxOutOfPlace += om.dx * _parallax * ff;
      _dyOutOfPlace += om.dy * _parallax * ff;
   }

   public void moveBackwards (ObjectMotion om)
   {
      _dxOutOfPlace -= om.dx * _parallax;
      _dyOutOfPlace -= om.dy * _parallax;
   }

   public void adjustSprite()
   {
      int dx = (int)_dxOutOfPlace;
      int dy = (int)_dyOutOfPlace;

      if (dx != 0 || dy != 0)
      {
         _dxOutOfPlace -= dx;
         _dyOutOfPlace -= dy;

         _rNew.left = pos.left + dx;
         _rNew.right = pos.right + dx;
         _rNew.top = pos.top + dy;
         _rNew.bottom = pos.bottom + dy;

         setRect(_rNew);
      }
   }

   public void destroy()
   {
      Xaiden.game().map().removeSprite(this);
      super.destroy();
   }

   Rect _rNew;
   float _parallax;
   float _dxOutOfPlace;
   float _dyOutOfPlace;
   int _mapIndex;
}

