package com.randomsoftware.xaiden;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class PointerSprite extends AnimSprite
{
   static final int POINTER_BADDIE = 0;
   static final int POINTER_GOODIE = 1;
   static final int POINTER_BASE = 2;

   static final float DIST_FAR = 400;
   static final float DIST_MEDIUM = 200;
   static final float DIST_CLOSE = 0;

   static final int NUMBER_OF_DISTS = 3;
   static final int NUMBER_OF_POINTERS = 3;

   public PointerSprite (int type, MapSprite target)
   {
      super (Xaiden.window(), Layout.DEPTH_POINTER);

      _type = type;
      _target = target;
      _bmpUpDown = Xaiden.res().getTransparentBitmap (Resources.IDB_POINTER_UPDOWN);
      _bmpLeftRight = Xaiden.res().getTransparentBitmap (Resources.IDB_POINTER_LEFTRIGHT);
      _cel = new Rect();
      _rect = new Rect();
      _cxUpDown = _bmpUpDown.getWidth() / (NUMBER_OF_DISTS*2);
      _cyUpDown = _bmpUpDown.getHeight() / NUMBER_OF_POINTERS;
      _cxLeftRight = _bmpLeftRight.getWidth() / (NUMBER_OF_DISTS*2);
      _cyLeftRight = _bmpLeftRight.getHeight() / NUMBER_OF_POINTERS;

      reposition();
   }

   public void update()
   {
      reposition();
   }

   public void paint (Pen pp)
   {
      if (_isNeeded)
      {
         if (_isUpDown)
            pp.drawBitmap (0, 0, _bmpUpDown, _cel);
         else
            pp.drawBitmap (0, 0, _bmpLeftRight, _cel);
      }
   }

   public void reposition()
   {
      if ((_isNeeded = !_target.isOnScreen()) == true)
      {
         int dx = (_target.pos.left + _target.pos.right)/2 - Layout.WIDTH_SCREEN/2;
         int dy = (_target.pos.top + _target.pos.bottom)/2 - Layout.HEIGHT_SCREEN/2;
         float angle = (float)Math.atan2 (-dy, dx);
         float dist = (float)Math.sqrt(dx*dx + dy*dy);

         int sizeIndex;
         if (dist > DIST_FAR)
            sizeIndex = 2;
         else if (dist > DIST_MEDIUM)
            sizeIndex = 1;
         else // (dist > DIST_CLOSE)
            sizeIndex = 0;

         while (angle < 0)
            angle += TWOPI;
         while (angle >= TWOPI) 
            angle -= TWOPI;

         if ((angle >= ANGLE_UPPERRIGHT) && (angle < ANGLE_UPPERLEFT))  // top edge
         {
            _rect.left = (int)((Layout.WIDTH_SCREEN - _cxUpDown) * (ANGLE_UPPERLEFT - angle) / (ANGLE_UPPERLEFT - ANGLE_UPPERRIGHT));
            _rect.right = _rect.left + _cxUpDown;
            _rect.top = 0;
            _rect.bottom = _rect.top + _cyUpDown;
            _cel.left = _cxUpDown * sizeIndex;
            _cel.right = _cel.left + _cxUpDown;
            _cel.top = _cyUpDown * _type;
            _cel.bottom = _cel.top + _cyUpDown;
            _isUpDown = true;
         }
         else if ((angle >= ANGLE_UPPERLEFT) && (angle < ANGLE_LOWERLEFT))  // left edge
         {
            _rect.top = (int)((Layout.HEIGHT_SCREEN - _cyLeftRight) * (angle - ANGLE_UPPERLEFT) / (ANGLE_LOWERLEFT - ANGLE_UPPERLEFT));
            _rect.bottom = _rect.top + _cyUpDown;
            _rect.left = 0;
            _rect.right = _rect.left + _cxLeftRight;
            _cel.left = _cxLeftRight * sizeIndex;
            _cel.right = _cel.left + _cxLeftRight;
            _cel.top = _cyLeftRight * _type;
            _cel.bottom = _cel.top + _cyLeftRight;
            _isUpDown = false;
         }
         else if ((angle >= ANGLE_LOWERLEFT) && (angle < ANGLE_LOWERRIGHT))  // bottom edge
         {
            _rect.left = (int)((Layout.WIDTH_SCREEN - _cxUpDown) * (angle - ANGLE_LOWERLEFT) / (ANGLE_LOWERRIGHT - ANGLE_LOWERLEFT));
            _rect.right = _rect.left + _cxUpDown;
            _rect.bottom = Layout.HEIGHT_SCREEN;
            _rect.top = _rect.bottom - _cyUpDown;
            _cel.left = _cxUpDown * (sizeIndex + NUMBER_OF_DISTS);
            _cel.right = _cel.left + _cxUpDown;
            _cel.top = _cyUpDown * _type;
            _cel.bottom = _cel.top + _cyUpDown;
            _isUpDown = true;
         }
         else // right edge
         {
            if (angle < PI)
               angle = PI - angle;
            else
               angle = (PI+TWOPI) - angle;

            _rect.top = (int)((Layout.HEIGHT_SCREEN - _cyLeftRight) * (angle - ANGLE_UPPERLEFT) / (ANGLE_LOWERLEFT - ANGLE_UPPERLEFT));
            _rect.bottom = _rect.top + _cyUpDown;
            _rect.right = Layout.WIDTH_SCREEN;
            _rect.left = _rect.right - _cxLeftRight;
            _cel.left = _cxLeftRight * (sizeIndex + NUMBER_OF_DISTS);
            _cel.right = _cel.left + _cxLeftRight;
            _cel.top = _cyLeftRight * _type;
            _cel.bottom = _cel.top + _cyLeftRight;
            _isUpDown = false;
         }

         setRect(_rect);
      }
   }

   int _type;
   MapSprite _target;
   Bitmap _bmpUpDown;
   Bitmap _bmpLeftRight;
   boolean _isNeeded;
   boolean _isUpDown;
   Rect _rect;
   Rect _cel;
   int _cxUpDown;
   int _cyUpDown;
   int _cxLeftRight;
   int _cyLeftRight;

   static final float PI = 3.14159265358979323846264338f;
   static final float TWOPI = 6.28318530717958647692528676f;

   static final float ANGLE_UPPERRIGHT = (float)Math.atan2(Layout.HEIGHT_SCREEN/2,Layout.WIDTH_SCREEN/2);
   static final float ANGLE_UPPERLEFT = PI - ANGLE_UPPERRIGHT;
   static final float ANGLE_LOWERLEFT = PI + ANGLE_UPPERRIGHT;
   static final float ANGLE_LOWERRIGHT = TWOPI - ANGLE_UPPERRIGHT;
}

