package com.randomsoftware.xaiden;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class ShipSprite extends AnimSprite
{
   public static final int SIZE_SHIPSPRITE = 32;

   public ShipSprite()
   {
      super (Xaiden.window(), Layout.DEPTH_SHIPSPRITE);
      buildLookupTable();

      _bmp = Xaiden.res().getTransparentBitmap (Resources.IDB_SHIP);
      _xDir = 0;  // -1=left, 1=right
      _yDir = -1; // -1=up, 1=down
      _shields = false;
      _moving = false;
      selectCel();

      Rect rr = new Rect();
      rr.left = (Layout.WIDTH_SCREEN - SIZE_SHIPSPRITE) / 2;
      rr.top = (Layout.HEIGHT_SCREEN - SIZE_SHIPSPRITE) / 2;
      rr.right = rr.left + SIZE_SHIPSPRITE;
      rr.bottom = rr.top + SIZE_SHIPSPRITE;
      setRect (rr);
   }

   void setDirection (int dx, int dy)
   {
      if (dx==0 && dy==0)
      {
         _moving = false;
      }
      else
      {
         _xDir = dx;
         _yDir = dy;
         _moving = true;
      }
      touch();
   }

   void setShields (boolean on)
   {
      _shields = on;
      touch();
   }

   public void update ()
   {
      if (_moving)
         touch();
   }

   public void paint (Pen pp)
   {
      selectCel();
      pp.drawBitmap (0, 0, _bmp, _cel);
   }

   void buildLookupTable()
   {
      int cx = SIZE_SHIPSPRITE;
      _table[0][0] = 7*cx;  _table[0][1] = 0*cx;  _table[0][2] = 1*cx;
      _table[1][0] = 6*cx;                        _table[1][2] = 2*cx;
      _table[2][0] = 5*cx;  _table[2][1] = 4*cx;  _table[2][2] = 3*cx;
   }

   void selectCel()
   {
      _cel.left = _table[_yDir+1][_xDir+1];

      if (_shields)
         _cel.top = 4 * SIZE_SHIPSPRITE;
      else if (!_moving)
         _cel.top = 0;
      else
         _cel.top = ((_win.getNextFrameNumber() % 3)+1) * SIZE_SHIPSPRITE;

      _cel.right = _cel.left + SIZE_SHIPSPRITE;
      _cel.bottom = _cel.top + SIZE_SHIPSPRITE;
   }

   Bitmap _bmp;
   Rect _cel = new Rect();
   int[][] _table = new int[3][3];

   int _xDir;
   int _yDir;
   boolean _shields;
   boolean _moving;
}

