package com.randomsoftware.xaiden;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class StarSprite extends AnimSprite
{
   public static final int SIZE_SHORT = 0;
   public static final int SIZE_TALL = 6;
   public static final int SIZE_GRANDE = 13;
   public static final int SIZE_VENTI = 18;
   public static final int SIZE_MAX = 20;

   public static final int FLASH_DELAY = 0;
   public static final int FLASH_TIME = 1;

   public StarSprite (int size)
   {
      super (Xaiden.window(), Layout.DEPTH_STARS + SIZE_MAX - size);
      _size = size;
      _loc = new ObjectLocation (pickParallax());
      pickImage();
      pickLocation();
   }

   public StarSprite (int size, ObjectMotion om)
   {
      super (Xaiden.window(), Layout.DEPTH_STARS + SIZE_MAX - size);
      _size = size;
      _loc = new ObjectLocation (pickParallax());
      pickImage();
      pickLocation(om);
   }

   static public int pickSize()
   {
      return Math.abs(_randgen.nextInt()) % SIZE_MAX;
   }

   float pickParallax()
   {
      return Layout.STAR_PARALLAX_MIN + (Layout.STAR_PARALLAX_MAX - Layout.STAR_PARALLAX_MIN) * _size / SIZE_MAX;
   }

   void pickImage()
   {
      if (_size < SIZE_TALL)
         _bmp = Xaiden.res().getOpaqueBitmap (Resources.IDB_STAR_SHORT);
      else if (_size < SIZE_GRANDE)
         _bmp = Xaiden.res().getOpaqueBitmap (Resources.IDB_STAR_TALL);
      else if (_size < SIZE_VENTI)
         _bmp = Xaiden.res().getOpaqueBitmap (Resources.IDB_STAR_GRANDE);
      else
         _bmp = Xaiden.res().getOpaqueBitmap (Resources.IDB_STAR_VENTI);

      _celSize = _bmp.getHeight();
      _celCount = _bmp.getWidth() / _celSize;
      _celIndex = 0;
      _countdown = Math.abs(_randgen.nextInt()) % (MainWindow.framesPerSecond * FLASH_TIME);

      _cel = new Rect();
      _cel.left = _cel.top = 0;
      _cel.right = _cel.bottom = _celSize;
   }

   void pickLocation()
   {
      Rect rr = new Rect();
      rr.left = Math.abs(_randgen.nextInt()) % (Layout.WIDTH_SCREEN - _celSize);
      rr.top = Math.abs(_randgen.nextInt()) % (Layout.HEIGHT_SCREEN - _celSize);
      rr.right = rr.left + _celSize;
      rr.bottom = rr.top + _celSize;
      setRect(rr);
   }

   void pickLocation (ObjectMotion om)
   {
      Rect rr = new Rect();
      float dx = om.dx;
      float dy = om.dy;

      // If we have a choice, pick which wall to arrive in
      if (dx != 0 && dy != 0) {
         if ((_randgen.nextInt() & 1) == 1)
            dx = 0;
         else
            dy = 0;
      }

      // Pick a spot to appear
      if (dx > 0) {
         rr.left = Layout.WIDTH_SCREEN - _celSize;
         rr.top = Math.abs(_randgen.nextInt()) % (Layout.HEIGHT_SCREEN - _celSize);
      } else if (dx < 0) {
         rr.left = 0;
         rr.top = Math.abs(_randgen.nextInt()) % (Layout.HEIGHT_SCREEN - _celSize);
      } else if (dy > 0) {
         rr.left = Math.abs(_randgen.nextInt()) % (Layout.WIDTH_SCREEN - _celSize);
         rr.top = Layout.HEIGHT_SCREEN - _celSize;
      } else {
         rr.left = Math.abs(_randgen.nextInt()) % (Layout.WIDTH_SCREEN - _celSize);
         rr.top = 0;
      }

      rr.right = rr.left + _celSize;
      rr.bottom = rr.top + _celSize;
      setRect(rr);
   }

   public void adjust (ObjectMotion shipMotion)
   {
      _loc.moveBackwards (shipMotion);
   }

   public void update()
   {
      _loc.adjustSprite (this);

      if (_countdown > 0)
         --_countdown;

      if (_countdown == 0) {
         if ((++_celIndex) == _celCount) {
            _celIndex = 0;
            _countdown = (MainWindow.framesPerSecond * FLASH_DELAY) + Math.abs(_randgen.nextInt()) % (MainWindow.framesPerSecond * FLASH_TIME);
         }
         _cel.left = _celSize * _celIndex;
         _cel.right = _cel.left + _celSize;
         touch();
      }
   }

   public void paint (Pen pp)
   {
      pp.drawBitmap (0, 0, _bmp, _cel);
   }

   static Random _randgen = new Random();
   int _size;
   Bitmap _bmp;
   Rect _cel;
   int _celSize;
   int _celCount;
   int _celIndex;
   int _countdown;
   ObjectLocation _loc;
}

