#include <stdio.h>
#include <process.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <string>
#include <map>

using std::map;
using std::string;

static bool isInputFile (char *arg)
{
   size_t arglen = strlen(arg);
   return ((arglen > 5) && !strcmp (&arg[arglen-5],".rsrc"));
}

static bool isReplacementDefinition (char *arg)
{
   // Some replacements have multiple paths.  We don't want to dork
   // around with that, so we'll just skip those.
   if (strchr (arg, ','))
      return false;
   return (!strncmp (arg, "-D", 2)) && (strchr(arg,'=') != NULL);
}

static void addReplacementMapping (map<string,string> &replacements, char *arg)
{
   char *lhs = &arg[2];
   char *lhsEnd = strchr(arg,'=');
   char *rhs = lhsEnd+1;

   string left (lhs, lhsEnd-lhs);
   string right (rhs);

   // This is totally illegal, but I'm REALLY lazy right now.
   for (char *psz = (char*)right.c_str(); *psz; ++psz) {
      *psz = (*psz == '\\') ? '/' : *psz;
   }

   replacements[left] = right;
}

static const char * getReplacementReference (map<string,string> &replacements, char *psz)
{
   if (*psz != '{')
      return false;

   char *lhs = &psz[1];
   char *end = lhs;
   while (*end && *end != '}')
      ++end;

   if (replacements.count(string(lhs,end-lhs)) == 0)
      return NULL;

   return replacements[ string(lhs,end-lhs) ].c_str();
}

static void replaceArgs (map<string,string> &replacements, char *outData, char *inData)
{
   while (inData && *inData)
   {
      if (*inData != '{')
      {
         *outData++ = *inData++;
      }
      else
      {
         const char *found = getReplacementReference (replacements, inData); 
         if (found == NULL)
         {
            *outData++ = *inData++;
         }
         else
         {
            strcpy (outData, found);
            outData = &outData[ strlen(outData) ];
            inData = strchr(inData,'}');
            if (inData != NULL)
               ++inData;
         }
      }
   }
   *outData = 0;
}

static void combineStrings (char *data)
{
   int len = strlen(data);

   // We're doing :g/"[ ]*\\$^[ ]*"/s///g

   for (char *walk = data; *walk; ++walk) {
      if (*walk != '"')
         continue;

      char *probe = walk;
      for (++probe; *probe == ' '; ++probe)
         ;
      if (*probe != '\\')
         continue;
      bool foundEol = false;
      for (++probe; *probe == ' ' || *probe == '\r' || *probe == '\n'; ++probe)
         foundEol |= (*probe == '\r' || *probe == '\n');
      if (!foundEol)
         continue;
      if (*probe != '"')
         continue;

      // Found one!  Get rid of all this crap in between.

      memmove (walk, &probe[1], len - (walk-data) + 1);
      len -= probe - walk + 1;
      if (!*walk)
         break;
   }
}

static bool createOutputFile (char *outName, char *outData, char *inName)
{
   strcpy (outName, inName);
   strcat (outName, "-processed.rsrc");

   int fh = open(outName, O_CREAT|O_TRUNC|O_WRONLY|O_BINARY, 0666);
   if (fh <= 0) {
      fprintf (stderr, "drcwrapper: unable to create temporary file %s\n", outName);
      return false;
   }

   write (fh, outData, strlen(outData));
   close(fh);
   return true;
}

static bool processInputFile (map<string,string> &replacements, char *outName, char *inName)
{
   int fh = open(inName, O_RDONLY|O_BINARY);
   if (fh <= 0) {
      fprintf (stderr, "drcwrapper: unable to open input file %s\n", inName);
      return false;
   }

   size_t inLength = lseek(fh, 0, SEEK_END);
   size_t outLength = inLength + 16384;	   // okay, sloppy.  Sue me.
   char *inData = (char*)malloc(inLength+1);
   char *outData = (char*)malloc(outLength+1);

   lseek (fh, 0, SEEK_SET);
   read (fh, inData, inLength);
   inData[inLength] = 0;

   replaceArgs (replacements, outData, inData);
   combineStrings (outData);
   bool rc = createOutputFile (outName, outData, inName);

   free (inData);
   free (outData);

   return rc;
}

int main (int argc, char **argv)
{
   // Check the command-line

   if (argc < 3) {
      fprintf (stderr, "usage: call drcwrapper instead of drc.\n");
      fprintf (stderr, "(the first parameter must tell me where the real drc.exe is.)\n");
      return 1;
   }

   // The first parameter is a pointer to the real DRC.  Pop our app name,
   // and that parameter, off the input args.

   --argc,++argv;

   char *drc = *argv;
   --argc,++argv;

   // All remaining arguments are going to be passed to DRC.  Except for
   // the .rsrc file itself--we'll be preprocessing it, copying it to a
   // temporary file, and passing that filename instead.

   char **outArgs = new char*[argc];
   char outFile[512] = "";
   char *inFile = NULL;
   map<string,string> replacements;

   for (size_t ii = 0; ii < argc; ++ii,++argv) {
      if (isInputFile(*argv)) {
         inFile = *argv;
         outArgs[ii] = outFile;
      } else if (isReplacementDefinition(*argv)) {
         addReplacementMapping (replacements, *argv);
         outArgs[ii] = *argv;
      } else {
         outArgs[ii] = *argv;
      }
   }

   if (!inFile) {
      fprintf (stderr, "drcwrapper: you never specified an input file\n");
      delete[] outArgs;
      return 1;
   }

   if (!processInputFile (replacements, outFile, inFile)) {
      delete[] outArgs;
      return 1;
   }

   int rc = execvp (drc, outArgs);

   delete[] outArgs;
   unlink (outFile);

   return rc;
}

