package com.randomsoftware.vncclient;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;

class EditHostDialog extends Listener
{
   public EditHostDialog (HostSettings host)
   {
      _host = host;
      _dlg = VncClient.app().getDialog (Resources.ID_DIALOG_EDITHOST, this);
      initDialog();
   }

   public void show() { _dlg.show(); }
   public void hide() { _dlg.hide(); }

   public boolean receiveEvent (Event e)
   {
      switch (Compat.Event_getType(e))
      {
         case ResourceEvents.EVENT_EDITHOST_DONE:
            int idsError = readDialog();
            if (idsError != 0) {
               _dlg.show();
               ResUtil ru = VncClient.getResUtil();
               (new AlertWindow (ru.getString(Resources.IDS_EDITHOST_ERROR_TITLE), ru.getString(idsError))).show();
               return true;
            }
            break;
      }
      return super.receiveEvent(e);
   }

   void initDialog()
   {
      danger.ui.RadioGroup group;

      ((TextField)_dlg.getControlWithID(Resources.IDC_EDITHOST_DESCRIPTION)).setText (_host.friendlyName);
      ((TextField)_dlg.getControlWithID(Resources.IDC_EDITHOST_ADDRESS)).setText (_host.serverAddress);
      ((TextField)_dlg.getControlWithID(Resources.IDC_EDITHOST_PASSWORD)).setText (_host.password);

      group = (danger.ui.RadioGroup)_dlg.getControlWithID(Resources.IDC_EDITHOST_SHAREFLAG);
      group.selectItemWithID (_host.makeExclusive ? Resources.IDC_EDITHOST_EXCLUSIVE : Resources.IDC_EDITHOST_SHARED);

      group = (danger.ui.RadioGroup)_dlg.getControlWithID(Resources.IDC_EDITHOST_POINTER);
      group.selectItemWithID (_host.hideRemoteCursor ? Resources.IDC_EDITHOST_HIDECURSOR : Resources.IDC_EDITHOST_SHOWCURSOR);

      _dlg.getControlWithID(Resources.IDC_EDITHOST_FULLSCREEN).setValue (_host.fullScreen ? 1 : 0);
      _dlg.getControlWithID(Resources.IDC_EDITHOST_SHOWTYPING).setValue (_host.showTyping ? 1 : 0);

      PopupMenu pm = (PopupMenu)_dlg.getControlWithID(Resources.IDC_EDITHOST_MAGNIFY);
      if (_host.zoomPercent > 200)
         pm.setValueWithItemID (Resources.ID_MENU_EDITHOST_MAGNIFY_400);
      else if (_host.zoomPercent > 100)
         pm.setValueWithItemID (Resources.ID_MENU_EDITHOST_MAGNIFY_200);
      else if (_host.zoomPercent > 50)
         pm.setValueWithItemID (Resources.ID_MENU_EDITHOST_MAGNIFY_100);
      else
         pm.setValueWithItemID (Resources.ID_MENU_EDITHOST_MAGNIFY_50);

      // If this host is already connected or connecting, don't let the user change the address
      if (_host.friendlyName.length() != 0) {
         int state = VncClient.getSessionManager().getSessionState(_host.friendlyName);
         if (SessionState.isTrying(state)) {
            _dlg.getControlWithID(Resources.IDC_EDITHOST_ADDRESS).disable();
            _dlg.getControlWithID(Resources.IDC_EDITHOST_EXCLUSIVE).disable();
            _dlg.getControlWithID(Resources.IDC_EDITHOST_SHARED).disable();
         }
      }
   }

   int readDialog()
   {
      String friendlyName = ((TextField)_dlg.getControlWithID(Resources.IDC_EDITHOST_DESCRIPTION)).toString();
      String serverAddress = ((TextField)_dlg.getControlWithID(Resources.IDC_EDITHOST_ADDRESS)).toString();
      String password = ((TextField)_dlg.getControlWithID(Resources.IDC_EDITHOST_PASSWORD)).toString();
      boolean makeExclusive = (_dlg.getControlWithID(Resources.IDC_EDITHOST_EXCLUSIVE).getValue() != 0);
      boolean hideRemoteCursor = (_dlg.getControlWithID(Resources.IDC_EDITHOST_HIDECURSOR).getValue() != 0);
      boolean fullScreen = (_dlg.getControlWithID(Resources.IDC_EDITHOST_FULLSCREEN).getValue() != 0);
      boolean showTyping = (_dlg.getControlWithID(Resources.IDC_EDITHOST_SHOWTYPING).getValue() != 0);

      String oldFriendlyName = _host.friendlyName;
      boolean isNewHost = (_host.keyInDatabase == null || _host.keyInDatabase.length() == 0);
      int zoomPercent = 100;

      PopupMenu pm = (PopupMenu)_dlg.getControlWithID(Resources.IDC_EDITHOST_MAGNIFY);
      switch (pm.getValueAsItemID()) {
         case Resources.ID_MENU_EDITHOST_MAGNIFY_400: zoomPercent = 400; break;
         case Resources.ID_MENU_EDITHOST_MAGNIFY_200: zoomPercent = 200; break;
         case Resources.ID_MENU_EDITHOST_MAGNIFY_100: zoomPercent = 100; break;
         case Resources.ID_MENU_EDITHOST_MAGNIFY_50:  zoomPercent =  50; break;
      }

      // The user must specify friendlyName and serverAddress data
      if (friendlyName.length() == 0) {
         return Resources.IDS_EDITHOST_ERROR_NO_DESC;
      }
      if (serverAddress.length() == 0) {
         return Resources.IDS_EDITHOST_ERROR_NO_ADDR;
      }

      // If the user has changed the description from what it was
      // when we started, then the new name better be unique
      if (!friendlyName.equalsIgnoreCase(_host.friendlyName)) {
         if (VncClient.getHostDatabase().getHostIndexByName(friendlyName) != HostDatabase.HOST_NOT_FOUND) {
            return Resources.IDS_EDITHOST_ERROR_DUP_DESC;
         }
      }

      // Update the passed-in Host object
      _host.friendlyName = new String(friendlyName);
      _host.serverAddress = new String(serverAddress);
      _host.password = new String(password);
      _host.makeExclusive = makeExclusive;
      _host.hideRemoteCursor = hideRemoteCursor;
      _host.fullScreen = fullScreen;
      _host.showTyping = showTyping;
      _host.zoomPercent = zoomPercent;
      _host.tempPassword = null;

      // Update the host database
      if (isNewHost) {
         VncClient.getHostDatabase().addHost (_host);
      } else {
         VncClient.getHostDatabase().saveHost (_host);
      }

      // If we're renaming an existing host, alert the session
      // manager (since it keys its sessions by friendlyName).
      if (!isNewHost) {
         Session session = VncClient.getSessionManager().findSession(oldFriendlyName);
         if (session != null) {
            session.updateHostSettings(_host);
         }
      }

      // Update the main display
      HostListScreen screen = VncClient.getHostListScreen();
      if (screen != null) {
         screen.refresh();
         screen.selectByName(friendlyName);
      }

      return 0;
   }

   DialogWindow _dlg;
   HostSettings _host;
}

