package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import danger.util.*;
import java.util.*;

public class HostDatabase extends Listener
{
   public HostDatabase() 
   {

      _hosts = null;
      _database = new SettingsDB("hosts", true);
      _database.setAutoSyncNotifyee (this);

      if (false) {
         // TODO: don't add this dummy entry; it's just for my convenience
         HostSettings ha = new HostSettings();
         ha.keyInDatabase = new String("default");
         ha.friendlyName = new String("Whirlwind");
         ha.password = new String("system10");
         ha.serverAddress = new String("71.195.88.28:0");
         ha.keyInDatabase = new String ("host-1");
         _database.setBytes (ha.keyInDatabase, ha.encode());
      }

      reloadHosts();
   }

   public int getHostCount()
   {
      return (_hosts == null) ? 0 : _hosts.size();
   }

   public HostSettings getHostByIndex (int ii)
   {
      return (ii < getHostCount()) ? (HostSettings)(_hosts.elementAt(ii)) : null;
   }

   public static final int HOST_NOT_FOUND = -1;
   public int getHostIndexByName (String friendlyName)
   {
      for (int ii = 0; ii < getHostCount(); ++ii) {
         HostSettings host = getHostByIndex(ii);
         if (host.friendlyName.equalsIgnoreCase(friendlyName))
            return ii;
      }
      return HOST_NOT_FOUND;
   }

   public void saveHost (HostSettings host)
   {
      _database.setBytes (host.keyInDatabase, host.encode());
      reloadHosts();
   }

   public void addHost (HostSettings host)
   {
      host.keyInDatabase = new String ("host-" + (_nextKeyValue++));
      _database.setBytes (host.keyInDatabase, host.encode());
      reloadHosts();
   }

   public void deleteHost (HostSettings host)
   {
      _database.remove(host.keyInDatabase);
      reloadHosts();
   }

   protected void reloadHosts()
   {
      _nextKeyValue = 0;

      int nHosts = _database.getNumEntries();
      if (nHosts == 0) {
         _hosts = null;
      } else {
         _hosts = new Vector();
         for (int ii = 0; ii < nHosts; ++ii) {
            try {
               String key = _database.getKey(ii);
               byte[] data = _database.getBytes(key);
               if (key.substring(0,5).equals("host-")) {
                  int keyValue = Integer.parseInt(key.substring(5));
                  _nextKeyValue = (keyValue > _nextKeyValue) ? keyValue : _nextKeyValue;
               }
               _hosts.add (new HostSettings (key, data));
            } catch (SettingsDBException ee) {
               // well, we tried to load it anyway
            }
         }
         sortHostsByName();
      }

      HostListScreen screen = VncClient.getHostListScreen();
      if (screen != null) {
         screen.refresh();
      }

      _nextKeyValue ++;
   }

   protected void sortHostsByName()
   {
      if (_hosts != null) {
         for (int ii = 0; ii+1 < _hosts.size(); ++ii) {
            HostSettings lhs = getHostByIndex(ii);

            for (int jj = ii+1; jj < _hosts.size(); ++jj) {
               HostSettings rhs = getHostByIndex(jj);

               if (lhs.friendlyName.compareToIgnoreCase (rhs.friendlyName) > 0) {
                  _hosts.setElementAt(rhs, ii);
                  _hosts.setElementAt(lhs, jj);
                  lhs = rhs;
               }
            }
         }
      }
   }

   public boolean receiveEvent (Event ee)
   {
      if (Compat.Event_getType(ee) == Event.EVENT_DATASTORE_RESTORED) {
         reloadHosts();
         return true;
      }
      return super.receiveEvent(ee);
   }

   SettingsDB _database;
   Vector _hosts;
   int _nextKeyValue;
}

