package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import danger.util.*;
import danger.util.format.*;

public class HostListScreen extends ScreenWindow
{
   public HostListScreen()
   {
   }

   public void show()
   {
      refresh();
      super.show();
   }

   public final boolean eventWidgetUp (int widget, Event ee)
   {
      switch (widget)
      {
         case Event.DEVICE_BUTTON_CANCEL:
         case Event.DEVICE_BUTTON_BACK:
            getApplication().returnToLauncher();
            return true;
      }

      return super.eventWidgetUp(widget, ee);
   }

   public void refresh()
   {
      HostListView vv = getHostListView();
      if (vv != null) {
         vv.repopulate();
         updateMenu();
      }
   }

   public void selectByName (String friendlyName)
   {
      HostListView vv = getHostListView();
      if (vv != null) {
         vv.selectByName(friendlyName);
         updateMenu();
      }
   }

   public void updateMenu()
   {
      Menu menu = getActionMenu();
      HostListView vv = getHostListView();
      if (menu != null && vv != null) {
         HostSettings host = vv.getFocusedHost();

         if (host == null) {
            menu.getItemWithID(Resources.ID_MENUITEM_EDIT_HOST).setEnabled(false);
            menu.getItemWithID(Resources.ID_MENUITEM_DELETE_HOST).setEnabled(false);
            menu.getItemWithID(Resources.ID_MENUITEM_OPEN_SESSION).setEnabled(false);
            menu.getItemWithID(Resources.ID_MENUITEM_CLOSE_SESSION).setEnabled(false);
         } else {
            SessionManager mgr = VncClient.getSessionManager();
            int state = mgr.getSessionState (host.friendlyName);
            menu.getItemWithID(Resources.ID_MENUITEM_EDIT_HOST).setEnabled(true);
            menu.getItemWithID(Resources.ID_MENUITEM_DELETE_HOST).setEnabled(!SessionState.isTrying(state));
            menu.getItemWithID(Resources.ID_MENUITEM_OPEN_SESSION).setEnabled(!SessionState.isTrying(state));
            menu.getItemWithID(Resources.ID_MENUITEM_CLOSE_SESSION).setEnabled(SessionState.isTrying(state));
         }
      }
   }

   public boolean receiveEvent (Event ee)
   {
      switch (Compat.Event_getType(ee)) {
         case ResourceEvents.EVENT_MENUITEM_NEW_HOST: {
            HostSettings host = new HostSettings();
            (new EditHostDialog(host)).show();
            return true;
         }

         case ResourceEvents.EVENT_MENUITEM_EDIT_HOST: {
            HostSettings host = getHostListView().getFocusedHost();
            (new EditHostDialog(host)).show();
            return true;
         }

         case ResourceEvents.EVENT_MENUITEM_DELETE_HOST: {
            HostSettings host = getHostListView().getFocusedHost();
            AlertWindow alert = VncClient.app().getAlert(Resources.ID_DIALOG_DELETEHOST, this);
            alert.setEventForControlWithID(Resources.IDC_DELETEHOST_DISCARD, new Event(ResourceEvents.EVENT_DELETEHOST_DISCARD, 0, 0, host));
            alert.setMessage(StringFormat.withFormat(alert.getMessage(), host.friendlyName));
            alert.show();
            return true;
         }

        case ResourceEvents.EVENT_DELETEHOST_DISCARD: {
            HostSettings host = (HostSettings)Compat.Event_getArgument(ee);
            VncClient.getSessionManager().closeSession(host);
            VncClient.getHostDatabase().deleteHost(host);
            return true;
         }

         case ResourceEvents.EVENT_MENUITEM_OPEN_SESSION: {
            HostSettings host = getHostListView().getFocusedHost();
            VncClient.getSessionManager().activateSession(host);
            return true;
         }

         case ResourceEvents.EVENT_MENUITEM_CLOSE_SESSION: {
            HostSettings host = getHostListView().getFocusedHost();
            VncClient.getSessionManager().closeSession(host);
            return true;
         }

         case ResourceEvents.EVENT_MENUITEM_HELP:
            VncClient.app().getResources().getDialog(Resources.ID_DIALOG_HELP_TOPICS).show();
            break;
      }

      return super.receiveEvent(ee);
   }

   protected HostListView getHostListView()
   {
      return (HostListView)getDescendantWithID (Resources.ID_VIEW_HOST_LIST);
   }
}

