package com.randomsoftware.vncclient;

import danger.app.*;
import danger.ui.*;
import danger.util.*;
import java.util.*;


public class HostListView extends ActiveListView
{
   public HostListView()
   {
      _contents = new StdActiveList();
      setList(_contents);
      repopulate();
   }

   public HostSettings getFocusedHost()
   {
      HostListViewItem item = (HostListViewItem)getFocusedItem();
      if (item == null) {
         return null;
      }
      return item.getHostSettings();
   }

   public void selectByName (String friendlyName)
   {
      for (int ii = 0; ii < _contents.size(); ++ii) {
         HostListViewItem item = (HostListViewItem)_contents.getItem(ii);
         if (item.friendlyName.equalsIgnoreCase (friendlyName)) {
            setFocusedItem (item);
         }
      }
   }

   protected void repopulate()
   {
      HostSettings preferred = getFocusedHost();

      _contents.removeAllItems();

      HostDatabase db = VncClient.getHostDatabase();
      int nHosts = db.getHostCount();
      for (int ii = 0; ii < nHosts; ++ii) {
         HostSettings host = db.getHostByIndex(ii);
         HostListViewItem item = new HostListViewItem (host);
         _contents.addItem (item);
      }

      if (preferred != null) {
         selectByName(preferred.friendlyName);
      }
   }

   protected void updateMenu()
   {
      HostListScreen screen = VncClient.getHostListScreen();
      if (screen != null) {
         screen.updateMenu();
      }
   }

   protected void itemFocused (Object oo)
   {
      super.itemFocused (oo);
      updateMenu();
   }

   protected void itemUpdated (Object oo, boolean isFocused)
   {
      super.itemUpdated (oo, isFocused);
      updateMenu();
   }

   protected void itemActivated (Object oo)
   {
      HostListViewItem item = (HostListViewItem)oo;
      VncClient.getSessionManager().activateSession(item.getHostSettings());
   }

   protected void paintEmptyList (Pen pp, int width, int height)
   {
      super.paintEmptyList (pp, width, height);

      ResUtil ru = VncClient.getResUtil();
      String s1 = ru.getString (Resources.IDS_NOHOSTS_1);
      String s2 = ru.getString (Resources.IDS_NOHOSTS_2);

      Font ff = Font.findSystemFont();
      pp.setFont (ff);
      pp.setColor (128, 128, 128);

      int hh = Compat.Font_getHeight(ff);
      int w1 = ff.getWidth(s1);
      int w2 = ff.getWidth(s2);
      pp.drawText ((width - w1)/2, height/2 - hh - 2 + ff.getAscent(), s1);
      pp.drawText ((width - w2)/2, height/2 + 2 + ff.getAscent(), s2);
   }

   protected void paintItemLine (Pen pp, int left, int top, int right, int bottom, boolean isFocused, Object oo)
   {
      if (isFocused) {
         paintSelection (pp, left, top, right, bottom);
         pp.setColor (getStyle().getColor(Style.BACKGROUND_COLOR));
      } else {
         pp.setColor(getStyle().getColor(Style.FOREGROUND_COLOR));
      }

      HostListViewItem item = (HostListViewItem)oo;
      item.paint (pp, left, top, right-left, bottom-top, isFocused);
   }

   protected StdActiveList _contents;
}

