package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import java.io.*;

public class HostSettings
{
   public String keyInDatabase;
   public String friendlyName;
   public String serverAddress;
   public String password;
   public String tempPassword;
   public boolean makeExclusive;
   public boolean hideRemoteCursor;
   public boolean fullScreen;
   public boolean showTyping;
   public int zoomPercent;

   public HostSettings()
   {
      tempPassword = null;
      setDefaults();
   }

   public HostSettings (String key, byte[] data) 
   {
      tempPassword = null;
      keyInDatabase = key;
      decode (data);
   }

   public HostSettings (HostSettings src)
   {
      keyInDatabase = new String(src.keyInDatabase);
      friendlyName = new String(src.friendlyName);
      serverAddress = new String(src.serverAddress);
      password = new String(src.password);
      makeExclusive = src.makeExclusive;
      hideRemoteCursor = src.hideRemoteCursor;
      fullScreen = src.fullScreen;
      showTyping = src.showTyping;
      zoomPercent = src.zoomPercent;
      if (src.tempPassword != null) {
         tempPassword = new String (src.tempPassword);
      } else {
         tempPassword = null;
      }
   }

   public void setDefaults()
   {
      friendlyName = new String("");
      serverAddress = new String("");
      password = new String("");
      makeExclusive = false;
      hideRemoteCursor = true;
      fullScreen = true;
      showTyping = true;
      zoomPercent = 100;
   }

   public byte[] encode()
   {
      DataSerializer out = new DataSerializer();
      out.writeInteger (VERSION_CURRENT);
      out.writeString (friendlyName);
      out.writeString (serverAddress);
      out.writeString (password);
      out.writeBoolean (makeExclusive);
      out.writeBoolean (hideRemoteCursor);
      out.writeBoolean (fullScreen);
      out.writeBoolean (showTyping);
      out.writeInteger (zoomPercent);
      return out.getBytes();
   }

   public void decode (byte[] data)
   {
      try {
         DataUnserializer in = new DataUnserializer (data);

         int version = in.readInteger();
         if (version != VERSION_CURRENT) {
            throw new java.io.IOException();
         }

         friendlyName = in.readString();
         serverAddress = in.readString();
         password = in.readString();
         makeExclusive = in.readBoolean();
         hideRemoteCursor = in.readBoolean();
         fullScreen = in.readBoolean();
         showTyping = in.readBoolean();
         zoomPercent = in.readInteger();
      } catch (java.io.IOException e) {
         setDefaults();
      }
   }

   public static final int VERSION_CURRENT = 1;
}

