package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import danger.util.*;
import danger.system.*;

public class KeySym
{
   public static final int BACKSPACE    =  0xFF08;
   public static final int TAB          =  0xFF09;
   public static final int LINEFEED     =  0xFF0A;
   public static final int RETURN       =  0xFF0D;
   public static final int PAUSE        =  0xFF13;
   public static final int SCROLL_LOCK  =  0xFF14;
   public static final int SYS_REQ      =  0xFF15;
   public static final int ESCAPE       =  0xFF1B;
   public static final int DELETE       =  0xFFFF;
   public static final int HOME         =  0xFF50;
   public static final int LEFT         =  0xFF51;
   public static final int UP           =  0xFF52;
   public static final int RIGHT        =  0xFF53;
   public static final int DOWN         =  0xFF54;
   public static final int PAGE_UP      =  0xFF55;
   public static final int PAGE_DOWN    =  0xFF56;
   public static final int END          =  0xFF57;
   public static final int PRINT        =  0xFF61;
   public static final int INSERT       =  0xFF63;
   public static final int NUM_LOCK     =  0xFF7F;
   public static final int F1           =  0xFFBE;
   public static final int F2           =  0xFFBF;
   public static final int F3           =  0xFFC0;
   public static final int F4           =  0xFFC1;
   public static final int F5           =  0xFFC2;
   public static final int F6           =  0xFFC3;
   public static final int F7           =  0xFFC4;
   public static final int F8           =  0xFFC5;
   public static final int F9           =  0xFFC6;
   public static final int F10          =  0xFFC7;
   public static final int F11          =  0xFFC8;
   public static final int F12          =  0xFFC9;
   public static final int SHIFT_L      =  0xFFE1;
   public static final int SHIFT_R      =  0xFFE2;
   public static final int CONTROL_L    =  0xFFE3;
   public static final int CONTROL_R    =  0xFFE4;
   public static final int CAPS_LOCK    =  0xFFE5;
   public static final int META_L       =  0xFFE7;
   public static final int META_R       =  0xFFE8;
   public static final int ALT_L        =  0xFFE9;
   public static final int ALT_R        =  0xFFEA;

   static int cookedCharToKeySym (char ch, int modifiers)
   {
      if ((modifiers & EventType.EVENT_MODIFIER_MENU_BUTTON) != 0) {
         return 0;
      }

      if ((modifiers & EventType.EVENT_MODIFIER_ALT) != 0) {
         if (ch == ' ') {
            return TAB;
         }
      }

      if ((modifiers & EventType.EVENT_MODIFIER_SYM) != 0) {
         if (ch == '0') {
            return F10;
         } else if (ch >= '1' && ch <= '9') {
            return F1 + ch - '1';
         }
         return 0;
      }

      if ((modifiers & EventType.EVENT_MODIFIER_CANCEL_BUTTON) != 0) {
         return 0;
      }

      if (ch == CHAR_DELETE) {
         if ((modifiers & EventType.EVENT_MODIFIER_CAPS) != 0) {
            return DELETE;
         } else {
            return BACKSPACE;
         }
      }
      if (ch == CHAR_TAB) {
         return TAB;
      }
      if (ch == CHAR_LINEFEED) {
         return LINEFEED;
      }
      if (ch == CHAR_ENTER) {
         return RETURN;
      }
      if (isCharPrintable (ch)) {
         return ch;
      }

      return 0;
   }

   static int widgetToKeySym (int widget, int modifiers)
   {
      if ((modifiers & EventType.EVENT_MODIFIER_ALT) != 0) {
         if (widget == EventType.DEVICE_ARROW_LEFT) {
            return HOME;
         } else if (widget == EventType.DEVICE_ARROW_RIGHT) {
            return END;
         } else if (widget == EventType.DEVICE_ARROW_UP) {
            return PAGE_UP;
         } else if (widget == EventType.DEVICE_ARROW_DOWN) {
            return PAGE_DOWN;
         }
         return 0;
      }

      if (widget == EventType.DEVICE_ARROW_LEFT) {
         return LEFT;
      } else if (widget == EventType.DEVICE_ARROW_RIGHT) {
         return RIGHT;
      } else if (widget == EventType.DEVICE_ARROW_UP) {
         return UP;
      } else if (widget == EventType.DEVICE_ARROW_DOWN) {
         return DOWN;
      }

      return 0;
   }

   static final boolean isCharPrintable (char ch)
   {
      if (ch >= '0' && ch <= '9')
         return true;
      if (ch >= 'A' && ch <= 'Z')
         return true;
      if (ch >= 'a' && ch <= 'z')
         return true;
      return (_tidbits.indexOf(ch) != -1);
   }

   static final boolean isKeySymPrintable (int key)
   {
      return (key > 0 && key < 255 && isCharPrintable((char)key));
   }

   static final char CHAR_DELETE = (char)8;
   static final char CHAR_TAB = (char)9;
   static final char CHAR_LINEFEED = (char)10;
   static final char CHAR_ENTER = (char)13;
   static final String _tidbits = new String("`~!@#$%^&*()_+- ={}|[]\\;:'\",./<>?");
}

