package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import danger.util.*;
import danger.system.*;

// The RefreshOverlay object draws the progress bar at the bottom
// of the RemoteScreen, whenever we have inbound screen data

public class RefreshOverlay
{
   public RefreshOverlay (Session session, RemoteScreen screen)
   {
      _session = session;
      _screen = screen;
      _totalDownloaded = 0;
      _totalOutstanding = _session.getFrameBuffer().getUpdatingSize();
      _font = Compat.Font_findBoldOutlineSystemFont();
      _sRefreshing = VncClient.getResUtil().getString(Resources.IDS_REFRESHING);
      _cxText = _font.getWidth(_sRefreshing);
      _cyTotal = Compat.Font_getHeight (_font) + yGAP*2;
      _progressBar = new ProgressBar (0, 0, _totalOutstanding);
      _progressBar.setSize (Compat.SCREEN_WIDTH - _cxText - xGAP * 3, 18);
      _clrBackground = new Color (0, 28, 74);
      _rBackground = new Rect();
   }

   public void destroy()
   {
      _screen.invalidate (0, _screen.getHeight() - _cyTotal, _screen.getWidth(), _screen.getHeight());
      _progressBar = null;
   }

   public void update()
   {
      int previous = _totalOutstanding;
      _totalOutstanding = _session.getFrameBuffer().getUpdatingSize();

      if (_totalOutstanding < previous) {
         _totalDownloaded += previous - _totalOutstanding;
      }

      _progressBar.setMaxValue (_totalDownloaded + _totalOutstanding);
      _progressBar.setValue (_totalDownloaded);
      _screen.invalidate (0, _screen.getHeight() - _cyTotal, _screen.getWidth(), _screen.getHeight());
   }

   public void paint (Pen pp)
   {
      if (_totalOutstanding + _totalDownloaded != 0) {
         Pen p2 = pp.clonePen();

         // Draw the background field
         _rBackground.left = 0;
         _rBackground.right = _screen.getWidth();
         _rBackground.top = _screen.getHeight() - _cyTotal;
         _rBackground.bottom = _screen.getHeight() + ROUNDSIZE;
         p2.setColor(_clrBackground);
         p2.fillRoundRect (_rBackground, ROUNDSIZE, ROUNDSIZE);

         // Draw the "refreshing" text
         p2.setFont(_font);
         p2.setColor(Color.WHITE);
         p2.setTextOutlineColor(Color.BLACK);
         p2.drawText(xGAP, _screen.getHeight() - _cyTotal + yGAP + _font.getAscent(), _sRefreshing);

         // Draw the progress bar
         p2.offset (_cxText + xGAP * 2, _screen.getHeight() - _cyTotal + yGAP + yPOS_BAR);
         _progressBar.paint(p2);
      }
   }

   Session _session;
   RemoteScreen _screen;
   ProgressBar _progressBar;
   int _totalDownloaded;
   int _totalOutstanding;
   String _sRefreshing;
   int _cxText;
   int _cyTotal;
   Font _font;
   Rect _rBackground;
   Color _clrBackground;

   static final int xGAP = 10;
   static final int yGAP = 4;
   static final int yPOS_BAR = 3;
   static final int ROUNDSIZE = 8;
}

