package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import danger.util.*;
import java.util.*;

public class Region
{
   public Region() 
   {
      _rects = new RegionRectList();
      _subtracted = new RegionRectList();
      _freelist = new RegionRectList();
   }

   public void clear()
   {
      RegionRect rr;
      while ((rr = _rects.popFirst()) != null) {
         _freelist.insert(rr);
      }
      while ((rr = _subtracted.popFirst()) != null) {
         _freelist.insert(rr);
      }
      while (_freelist.size() > MAX_FREELIST_SIZE) {
         _freelist.popFirst();
      }
   }

   public void add (int xx, int yy, int cx, int cy)
   {
      // Ignore make-work calls
      if (cx <= 0 || cy <= 0) {
         return;
      }

      // As we look through the existing list, see if there's another
      // regionrect that we could theoretically merge with.
      RegionRect mergeWith = null;

      // Look for overlaps.  If we find any, recursively call this method
      // for each half of a break, then return.
      for (RegionRect rr = _rects.getFirst(); rr != null; rr = _rects.getNext(rr)) {

         // Is the new rect entirely enveloped by the existing rect?  Leave.
         if (xx >= rr.left && (xx+cx) <= rr.right && yy >= rr.top && (yy+cy) <= rr.bottom) {
            return;
         }

         // Is the new rect at all bisected by the existing rect's left edge?  Split.
         if (rr.left > xx && rr.left < xx+cx && yy < rr.bottom && yy+cy >= rr.top) {
            int bisect = rr.left;
            add (xx, yy, bisect-xx, cy);
            add (bisect, yy, xx+cx-bisect, cy);
            return;
         }

         // Is the new rect at all bisected by the existing rect's right edge?  Split.
         if (rr.right > xx && rr.right < xx+cx && yy < rr.bottom && yy+cy >= rr.top) {
            int bisect = rr.right;
            add (xx, yy, bisect-xx, cy);
            add (bisect, yy, xx+cx-bisect, cy);
            return;
         }

         // Is the new rect at all bisected by the existing rect's top edge?  Split.
         if (rr.top > yy && rr.top < yy+cy && xx < rr.right && xx+cx >= rr.left) {
            int bisect = rr.top;
            add (xx, yy, cx, bisect-yy);
            add (xx, bisect, cx, yy+cy-bisect);
            return;
         }

         // Is the new rect at all bisected by the existing rect's bottom edge?  Split.
         if (rr.bottom > yy && rr.bottom < yy+cy && xx < rr.right && xx+cx >= rr.left) {
            int bisect = rr.bottom;
            add (xx, yy, cx, bisect-yy);
            add (xx, bisect, cx, yy+cy-bisect);
            return;
         }

         // Okay, we don't overlap with this region.  Maybe we abut it, and
         // could merge with it?

         if (rr.canMerge) {
            if (yy == rr.top && (yy+cy) == rr.bottom) {
               if (xx == rr.right || (xx+cx) == rr.left) {
                  mergeWith = rr;
               }
            }
            if (xx == rr.left && (xx+cx) == rr.right) {
               if (yy == rr.bottom || (yy+cy) == rr.top) {
                  mergeWith = rr;
               }
            }
         }
      }

      // If we got here, there were no collisions.  But maybe we found
      // a rectangle we could merge with?
      if (mergeWith != null) {
         if (yy == mergeWith.top && (yy+cy) == mergeWith.bottom && xx == mergeWith.right) {
            mergeWith.right = xx+cx;
            return;
         } else if (yy == mergeWith.top && (yy+cy) == mergeWith.bottom && (xx+cx) == mergeWith.left) {
            mergeWith.left = xx;
            return;
         } else if (xx == mergeWith.left && (xx+cx) == mergeWith.right && yy == mergeWith.bottom) {
            mergeWith.bottom = yy+cy;
            return;
         } else if (xx == mergeWith.left && (xx+cx) == mergeWith.right && (yy+cy) == mergeWith.top) {
            mergeWith.top = yy;
            return;
         }
      }

      // No collisions, no merge options.  Just add this.
      _rects.insert (allocRegionRect (xx, yy, cx, cy));
   }

   public void subtract (int xx, int yy, int cx, int cy)
   {
      RegionRect rr = allocRegionRect (xx, yy, cx, cy);

      RegionRect rWalk = _rects.getFirst();
      RegionRect rNext;
      for ( ; rWalk != null; rWalk = rNext) {
         rNext = _rects.getNext(rWalk);
         if (rWalk.intersects(rr)) {
            _rects.remove(rWalk);
            _subtracted.insert(rWalk);
         }
      }

      if (!_subtracted.empty()) {
         rr.canMerge = false;
         _rects.insert(rr);

         while (!_subtracted.empty()) {
            RegionRect restore = _subtracted.popFirst();
            add (restore.left, restore.top, restore.right-restore.left, restore.bottom-restore.top);
            _freelist.insert(restore);
         }

         _rects.remove(rr);
         rr.canMerge = true;
      }

      freeRegionRect(rr);
   }


   public RegionRect walkAffected (Rect in, Rect out)
   {
      return walkAffectedNext (in, out, null);
   }

   public RegionRect walkAffectedNext (Rect in, Rect out, RegionRect rr)
   {
      if (rr == null) {
         rr = _rects.getFirst();
      } else {
         rr = _rects.getNext(rr);
      }

      for ( ; rr != null; rr = _rects.getNext(rr)) {
         if (rr.intersects(in)) {
            out.left = (in.left > rr.left) ? in.left : rr.left;
            out.right = (in.right < rr.right) ? in.right : rr.right;
            out.top = (in.top > rr.top) ? in.top : rr.top;
            out.bottom = (in.bottom < rr.bottom) ? in.bottom : rr.bottom;
            return rr;
         }
      }

      return rr;
   }

   public RegionRect getFirst()
   {
      return _rects.getFirst();
   }

   public RegionRect getNext (RegionRect rr)
   {
      return _rects.getNext(rr);
   }


   protected RegionRect allocRegionRect (int xx, int yy, int cx, int cy)
   {
      RegionRect rr;
      if ((rr = _freelist.popFirst()) == null) {
         rr = new RegionRect();
      }

      rr.left = xx;
      rr.top = yy;
      rr.right = xx + cx;
      rr.bottom = yy + cy;
      return rr;
   }

   protected void freeRegionRect (RegionRect rr)
   {
      if (_freelist.size() < MAX_FREELIST_SIZE) {
         _freelist.insert(rr);
      }
   }

   public int size()
   {
      return _rects.size();
   }

   RegionRectList _rects;
   RegionRectList _subtracted;
   RegionRectList _freelist;
   static final int MAX_FREELIST_SIZE = 32;
}

class RegionRect extends Rect
{
   public RegionRect _prev = null;
   public RegionRect _next = null;
   public boolean canMerge = true;
}

class RegionRectList
{
   public RegionRect getFirst()
   {
      return _first;
   }

   public RegionRect getNext (RegionRect rr)
   {
      return rr._next;
   }

   public RegionRect popFirst()
   {
      RegionRect rr = _first;
      if (rr != null) {
         remove (rr);
      }
      return rr;
   }

   public void insert (RegionRect rr)
   {
      rr._prev = _last;
      rr._next = null;
      if (_last != null)
         _last._next = rr;
      else
         _first = rr;
      _last = rr;
      _size++;
   }

   public void remove (RegionRect rr)
   {
      if (rr._prev == null)
         _first = rr._next;
      else
         rr._prev._next = rr._next;
      if (rr._next == null)
         _last = rr._prev;
      else
         rr._next._prev = rr._prev;
      _size--;
   }

   public boolean empty()
   {
      return (_size == 0);
   }

   public int size()
   {
      return _size;
   }

   protected RegionRect _first = null;
   protected RegionRect _last = null;
   protected int _size = 0;
}

