package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import danger.util.*;
import danger.system.*;
import danger.audio.*;

// The RemoteScreen is an interface between the FrameBuffer (that contains
// the content we want to draw) and the cell phone's UI layer.

public class RemoteScreen extends ScreenWindow
{
   public RemoteScreen()
   {
      _session = null;
      _view = null;
      _dlgStatePopup = null;
      _lastStatePopup = SessionState.DISCONNECTED;
      _refreshOverlay = null;
      _typingOverlay = null;
      _eatCancelUp = false;
      _isCancelDown = false;
      _refreshOnMenuUp = false;
      _mouseButtonsDown = 0;
      _rDesktop = new Rect();
      _incrementalTimer = Compat.new_Timer (MILLI_INCREMENTAL_UPDATE, true, this, TIMER_INCREMENTAL);
      _clearTypingTimer = Compat.new_Timer (MILLI_CLEAR_TYPING, false, this, TIMER_CLEAR_TYPING);
      _sendTypingTimer = null;
      _charsToSend = null;
      _charsToSendCount = 0;
      _controlState = CONTROL_NONE;
      _metaState = META_NONE;
      _remoteThinksControlDown = false;
      _remoteThinksMetaDown = false;
   }

   public void init (Session ss)
   {
      _session = ss;
      _view = _session.getView();
      setFullScreen (_view.fullScreen);
      setTitle (_session.getName());
   }

   public void show()
   {
      super.show();
      _incrementalTimer.start();

      Meta.muteID(Meta.FEEDBACK_DONE_BUTTON);
      Meta.muteID(Meta.FEEDBACK_CANCEL_BUTTON);
      Meta.muteID(Meta.FEEDBACK_MENU_BUTTON);
      Meta.muteID(Meta.FEEDBACK_LEFT_SHOULDER_BUTTON);
      Meta.muteID(Meta.FEEDBACK_RIGHT_SHOULDER_BUTTON);
      Meta.muteID(Meta.FEEDBACK_PAGE_UP);
      Meta.muteID(Meta.FEEDBACK_PAGE_DOWN);
      Meta.muteID(Meta.SCROLL_WHEEL_DOWN);
      Meta.muteID(Meta.SCROLL_WHEEL_UP);
      Meta.muteID(Meta.SCROLL_LIMIT);
      Meta.muteID(Meta.BEEP_COMMAND_REJECTED);
      Compat.VolumeManager_muteTrackball();

      _mouseButtonsDown = 0;
      _charsToSendCount = 0;
      _controlState = CONTROL_NONE;
      _metaState = META_NONE;

      createStatePopup();
      updateRefreshOverlay();
      addMenuShortcuts();
      Protocol.sendClipboard(_session.getConnection());
      touch();
   }

   public void hide()
   {
      _incrementalTimer.stop();
      clearRefreshOverlay();
      clearStatePopup();

      Meta.unmuteID(Meta.FEEDBACK_DONE_BUTTON);
      Meta.unmuteID(Meta.FEEDBACK_CANCEL_BUTTON);
      Meta.unmuteID(Meta.FEEDBACK_MENU_BUTTON);
      Meta.unmuteID(Meta.FEEDBACK_LEFT_SHOULDER_BUTTON);
      Meta.unmuteID(Meta.FEEDBACK_RIGHT_SHOULDER_BUTTON);
      Meta.unmuteID(Meta.FEEDBACK_PAGE_UP);
      Meta.unmuteID(Meta.FEEDBACK_PAGE_DOWN);
      Meta.unmuteID(Meta.SCROLL_WHEEL_DOWN);
      Meta.unmuteID(Meta.SCROLL_WHEEL_UP);
      Meta.unmuteID(Meta.SCROLL_LIMIT);
      Meta.unmuteID(Meta.BEEP_COMMAND_REJECTED);
      Compat.VolumeManager_unmuteTrackball();

      super.hide();
   }

   public Session getSession()
   {
      return _session;
   }

   protected void createStatePopup()
   {
      if ((_dlgStatePopup != null) &&
          SessionState.getsProgressPopup (_lastStatePopup) &&
          SessionState.getsProgressPopup (_session.getState())) {
         SessionState.updateProgressPopup(_session, _dlgStatePopup);
      } else {
         clearStatePopup();
         _dlgStatePopup = SessionState.createPopup(_session);
      }
      _lastStatePopup = _session.getState();
   }

   protected void clearStatePopup()
   {
      if (_dlgStatePopup != null) {
         _dlgStatePopup.hide();
         _dlgStatePopup = null;
         _lastStatePopup = SessionState.DISCONNECTED;
      }
   }

   public boolean eventWidgetDown (int code, Event ee)
   {
      switch (code) {
         case Compat.EVENTTYPE_DEVICE_NAVIGATION: {
            int cx = Compat.Event_getHorizontalCount(ee);
            int cy = Compat.Event_getVerticalCount(ee);
            int accel = SessionCursor.ACCEL_NORMAL;

            if (isMenuDown()) {
               markMenuUsed();
               accel = SessionCursor.ACCEL_HIGH | SessionCursor.ALLOW_SCREEN_MOVE;
            }

            queueMouseByWheel (cx, cy, accel);
            return true;
         }

         case EventType.DEVICE_WHEEL_PAGE_DOWN:
            markMenuUsed();
            queueMouseByWheel (0, 1, SessionCursor.ACCEL_HIGH | SessionCursor.ALLOW_SCREEN_MOVE);
            return true;

         default:
            if (KeySym.widgetToKeySym (code, Compat.Event_getModifiers(ee)) != 0) {
               return true;
            }
            break;
      }

      return super.eventWidgetDown (code, ee);
   }

   public boolean eventWidgetUp (int code, Event ee)
   {
      switch (code) {
         case EventType.DEVICE_WHEEL_PAGE_UP:
            markMenuUsed();
            queueMouseByWheel (0, -1, SessionCursor.ACCEL_HIGH | SessionCursor.ALLOW_SCREEN_MOVE);
            return true;

         default: {
            int keysym;
            if ((keysym = KeySym.widgetToKeySym (code, Compat.Event_getModifiers(ee))) != 0) {
               queueKeyForSend (keysym);
               return true;
            }
            break;
         }
      }

      return super.eventWidgetUp (code, ee);
   }

   public final boolean eventRawKeyDown (int code, Event ee)
   {
      switch (code) {

         // The Cancel button might be used in conjunction with other things
         // happening.  Pay attention when it goes down.
         case RawKeymap.BUTTON_CANCEL:
            _eatCancelUp = false;
            _isCancelDown = true;
            break;

         // Some keydown events should be eaten if they're part of a combination
         case RawKeymap.KEY_T:
         case RawKeymap.KEY_R:
         case RawKeymap.BUTTON_UP_ARROW:
         case RawKeymap.BUTTON_DOWN_ARROW:
         case RawKeymap.BUTTON_LEFT_ARROW:
         case RawKeymap.BUTTON_RIGHT_ARROW:
            if (_isCancelDown) {
               return true;
            }
            break;

         // CANCEL+LEFT SHOULDER: zoom out
         // MENU+LEFT SHOULDER: double-click left mouse button
         // LEFT SHOULDER: left mouse button
         case RawKeymap.BUTTON_LEFT_SHOULDER:
            if (!_isCancelDown && !isMenuDown()) {
               changeMouseButton (LEFT_MOUSE_BUTTON, BUTTON_DOWN);
               requestIncrementalSoon();
            }
            return true;

         // CANCEL+RIGHT SHOULDER: zoom in
         // MENU+RIGHT SHOULDER: double-click right mouse button
         // RIGHT SHOULDER: right mouse button
         case RawKeymap.BUTTON_RIGHT_SHOULDER:
            if (!_isCancelDown && !isMenuDown()) {
               changeMouseButton (RIGHT_MOUSE_BUTTON, BUTTON_DOWN);
               requestIncrementalSoon();
            }
            return true;

         // The Menu button might be used in conjunction with other things
         // happening.  Pay attention when it goes down.
         case RawKeymap.BUTTON_MENU:
            onControlDown();
            return true;

         // MENU+WHEEL CLICK: double-click left mouse button
         // WHEEL DOWN: meta
         case RawKeymap.BUTTON_WHEEL:
            if (!isMenuDown()) {
               onMetaDown();
            }
            return true;

         // SHIFT
         case RawKeymap.KEY_LEFT_SHIFT:
         case RawKeymap.KEY_RIGHT_SHIFT:
            onShiftDown();
            return false;
      }
      return super.eventRawKeyDown (code, ee);
   }

   public final boolean eventRawKeyUp (int code, Event ee)
   {
      switch (code) {

         // If the Menu button was hit all by itself, then let this
         // event pass through.  Otherwise, eat it.
         case RawKeymap.BUTTON_MENU:
            if (onControlUp()) {
               return true;
            }
            break;

         // CANCEL: kill any typingOverlay, or pass through if none
         case RawKeymap.BUTTON_CANCEL:
            _isCancelDown = false;
            if (_typingOverlay != null) {
               clearTypingOverlay();
               _eatCancelUp = true;
            }
            if (_eatCancelUp) {
               _eatCancelUp = false;
               return true;
            }
            break;

         // BACK: return to host list
         case RawKeymap.BUTTON_BACK:
            VncClient.app().showSession(null);
            return true;

         // CANCEL+LEFT SHOULDER: zoom out
         // MENU+LEFT SHOULDER: double-click left mouse button
         // LEFT SHOULDER: left mouse button
         case RawKeymap.BUTTON_LEFT_SHOULDER:
            if (_isCancelDown) {
               _eatCancelUp = true;
               _view.zoomOut();
            } else if (isMenuDown()) {
               markMenuUsed();
               queueMouseDoubleClick (LEFT_MOUSE_BUTTON);
               requestIncrementalSoon();
            } else {
               changeMouseButton (LEFT_MOUSE_BUTTON, BUTTON_UP);
               requestIncrementalSoon();
            }
            return true;

         // CANCEL+RIGHT SHOULDER: zoom in
         // MENU+RIGHT SHOULDER: double-click right mouse button
         // RIGHT SHOULDER: right mouse button
         case RawKeymap.BUTTON_RIGHT_SHOULDER:
            if (_isCancelDown) {
               _eatCancelUp = true;
               _view.zoomIn();
            } else if (isMenuDown()) {
               queueMouseDoubleClick (RIGHT_MOUSE_BUTTON);
               requestIncrementalSoon();
               markMenuUsed();
            } else {
               changeMouseButton (RIGHT_MOUSE_BUTTON, BUTTON_UP);
               requestIncrementalSoon();
            }
            return true;

         // CANCEL+T: full screen
         case RawKeymap.KEY_T:
            if (_isCancelDown) {
               _eatCancelUp = true;
               _view.setFullScreen (!_view.fullScreen);
               return true;
            }
            break;

         // CANCEL+R: refresh
         case RawKeymap.KEY_R:
            if (_isCancelDown) {
               _eatCancelUp = true;
               _session.getFrameBuffer().refreshVisible (FrameBuffer.DISCARD_AND_REFRESH);
               invalidate();
               return true;
            }
            break;

         // CANCEL+DPAD: move display area
         case RawKeymap.BUTTON_UP_ARROW:
         case RawKeymap.BUTTON_DOWN_ARROW:
         case RawKeymap.BUTTON_LEFT_ARROW:
         case RawKeymap.BUTTON_RIGHT_ARROW:
            if (_isCancelDown) {
               _eatCancelUp = true;
               _view.moveByKeymap (code);
               return true;
            }
            break;

         // MENU+WHEEL CLICK: double-click left mouse button
         // WHEEL CLICK: meta or left mouse button
         case RawKeymap.BUTTON_WHEEL:
            if (isMenuDown()) {
               markMenuUsed();
               queueMouseDoubleClick (LEFT_MOUSE_BUTTON);
               requestIncrementalSoon();
            } else {
               onMetaUp();
            }
            return true;

         // SHIFT
         case RawKeymap.KEY_LEFT_SHIFT:
         case RawKeymap.KEY_RIGHT_SHIFT:
            onShiftUp();
            return false;
      }

      return super.eventRawKeyUp(code, ee);
   }

   public final boolean eventKeyDown (char ch, Event ee)
   {
      int keysym;
      if ((keysym = KeySym.cookedCharToKeySym (ch, Compat.Event_getModifiers(ee))) != 0) {
         queueKeyForSend (keysym);
         return true;
      }
      return super.eventKeyDown(ch, ee);
   }

   public final boolean eventKeyUp (char ch, Event ee)
   {
      if (KeySym.cookedCharToKeySym (ch, Compat.Event_getModifiers(ee)) != 0) {
         return true;
      }
      return super.eventKeyUp(ch, ee);
   }

   public final boolean eventKeyRepeat (char ch, Event ee)
   {
      int keysym;
      if ((keysym = KeySym.cookedCharToKeySym (ch, Compat.Event_getModifiers(ee))) != 0) {
         queueKeyForSend (keysym);
         return true;
      }
      return super.eventKeyRepeat(ch, ee);
   }

   public void adjustActionMenuState (Menu menu)
   {
      menu.getItemWithID(Resources.ID_MENUITEM_FULLSCREEN).setChecked(_view.fullScreen);

      Menu zoomMenu = menu.getItemWithID(Resources.ID_MENUITEM_ZOOM).getSubMenu();
      zoomMenu.getItemWithID(Resources.ID_MENUITEM_MAGNIFY_50).setChecked(_view.zoomPercent == 50);
      zoomMenu.getItemWithID(Resources.ID_MENUITEM_MAGNIFY_100).setChecked(_view.zoomPercent == 100);
      zoomMenu.getItemWithID(Resources.ID_MENUITEM_MAGNIFY_200).setChecked(_view.zoomPercent == 200);
      zoomMenu.getItemWithID(Resources.ID_MENUITEM_MAGNIFY_400).setChecked(_view.zoomPercent == 400);
      zoomMenu.getItemWithID(Resources.ID_MENUITEM_ZOOM_IN).setEnabled(_view.zoomPercent < 400);
      zoomMenu.getItemWithID(Resources.ID_MENUITEM_ZOOM_OUT).setEnabled(_view.zoomPercent > 50);
   }

   public void aboutToShowMenu()
   {
      onControlUp();
   }

   public void clearTypingOverlay()
   {
      if (_typingOverlay != null) {
         _typingOverlay.destroy();
         _typingOverlay = null;
      }
   }

   public void clearRefreshOverlay()
   {
      if (_refreshOverlay != null) {
         _refreshOverlay.destroy();
         _refreshOverlay = null;
      }
   }

   public void updateTypingOverlay (int key)
   {
      if (_view.showTyping) {
         if (key == KeySym.RETURN) {
            clearTypingOverlay();
         } else {
            boolean isBackspace = false;
            boolean isNavigation = false;
            boolean isVisible = KeySym.isKeySymPrintable(key);

            if (_typingOverlay != null && _typingOverlay.size() != 0) {
               if (key == KeySym.BACKSPACE || key == KeySym.DELETE) {
                  isBackspace = true;
               } else if (key == KeySym.LEFT || key == KeySym.RIGHT || key == KeySym.HOME || key == KeySym.END) {
                  isNavigation = true;
               }
            }

            if (isBackspace || isNavigation || isVisible) {
               if (_typingOverlay == null) {
                  clearRefreshOverlay();
                  _typingOverlay = new TypingOverlay (_session, this);
               }
               _typingOverlay.append (key);
               _clearTypingTimer.resetDelay(MILLI_CLEAR_TYPING);
            }
         }
      }
   }

   public void updateRefreshOverlay()
   {
      if (_session.getFrameBuffer().getUpdatingSize() == 0) {
         if (_refreshOverlay != null) {
            clearRefreshOverlay();
            requestIncrementalSoon();
         }
      } else if (_refreshOverlay != null) {
         _refreshOverlay.update();
      } else {
         clearTypingOverlay();
         _refreshOverlay = new RefreshOverlay (_session, this);
      }
   }

   public void queueKeyForSend (int keysym)
   {
      sendControlMetaState(0);
      markKeyEvent();

      if ((_charsToSend == null) || (_charsToSendCount+1 >= _charsToSend.length)) {
         int[] newCharsToSend = new int[ _charsToSendCount + 32 ];
         for (int ii = 0; ii < _charsToSendCount; ++ii) {
            newCharsToSend[ii] = _charsToSend[ii];
         }
         _charsToSend = newCharsToSend;
      }
      _charsToSend[ _charsToSendCount++ ] = keysym;

      if (_sendTypingTimer == null) {
         _sendTypingTimer = Compat.new_Timer (MILLI_SEND_TYPING, false, this, TIMER_SEND_TYPING);
         _sendTypingTimer.start();
      }

      updateTypingOverlay (keysym);
   }

   public void sendQueuedTyping()
   {
      if (_sendTypingTimer != null) {
         _sendTypingTimer.stop();
         _sendTypingTimer = null;
      }
      if (_charsToSendCount != 0) {
         if (_session.getState() == SessionState.CONNECTED) {
            Protocol.sendKeySequence (_session.getConnection(), _charsToSend, _charsToSendCount);
            requestIncrementalSoon();
         }
         _charsToSendCount = 0;
      }
   }

   void queueMouseByWheel (int dx, int dy, int accel)
   {
      sendControlMetaState(IGNORE_CONTROL);
      markMouseEvent();
      _session.getCursor().moveByWheel (dx, dy, accel);
   }

   void queueMouseDoubleClick (int iButton)
   {
      sendControlMetaState(0);
      markMouseEvent();
      _session.getCursor().sendCursorDoubleClick (iButton);
   }

   public void requestIncrementalSoon()
   {
      _incrementalTimer.resetDelay(MILLI_INCREMENTAL_UPDATE_SOON);
   }

   public void requestIncremental()
   {
      if (_session.getState() == SessionState.CONNECTED) {
         if (_refreshOverlay == null) {
            _view.getVisibleBuffer (_rDesktop);
            Protocol.requestUpdate (_session.getConnection(), _rDesktop.left, _rDesktop.top, _rDesktop.right-_rDesktop.left, _rDesktop.bottom-_rDesktop.top, Protocol.INCREMENTAL_UPDATE);
         }
      }
      _incrementalTimer.resetDelay(MILLI_INCREMENTAL_UPDATE);
   }

   public boolean receiveEvent (Event ee)
   {
      int type = Compat.Event_getType(ee);

      if (SessionState.isSessionState(type)) {
         createStatePopup();
      } else {
         switch(type) {
            case EventType.EVENT_TIMER:
               switch (Compat.Event_getData(ee)) {
                  case TIMER_INCREMENTAL:
                     requestIncremental();
                     break;
                  case TIMER_CLEAR_TYPING:
                     clearTypingOverlay();
                     break;
                  case TIMER_SEND_TYPING:
                     sendQueuedTyping();
                     break;
               }
               break;

            case ResourceEvents.EVENT_STATE_HIDE:
            case ResourceEvents.EVENT_MENUITEM_GOTO_HOSTLIST:
               VncClient.app().showSession(null);
               return true;

            case ResourceEvents.EVENT_STATE_GIVEUP:
            case ResourceEvents.EVENT_MENUITEM_QUIT:
               HostSettings host = _session.getHostSettings();
               VncClient.getSessionManager().closeSession(host);
               VncClient.app().showSession(null);
               return true;

            case ResourceEvents.EVENT_STATE_RETRY:
               _session.retry();
               return true;

            case ResourceEvents.EVENT_MENUITEM_FULLSCREEN:
               _view.setFullScreen (!_view.fullScreen);
               break;

            case ResourceEvents.EVENT_MENUITEM_ZOOM_IN:
               _view.zoomIn();
               break;
            case ResourceEvents.EVENT_MENUITEM_ZOOM_OUT:
               _view.zoomOut();
               break;
            case ResourceEvents.EVENT_MENUITEM_MAGNIFY_400:
               _view.setZoomPercent(400);
               break;
            case ResourceEvents.EVENT_MENUITEM_MAGNIFY_200:
               _view.setZoomPercent(200);
               break;
            case ResourceEvents.EVENT_MENUITEM_MAGNIFY_100:
               _view.setZoomPercent(100);
               break;
            case ResourceEvents.EVENT_MENUITEM_MAGNIFY_50:
               _view.setZoomPercent(50);
               break;

            case ResourceEvents.EVENT_MENUITEM_REFRESH:
               _session.getFrameBuffer().refreshVisible (FrameBuffer.DISCARD_AND_REFRESH);
               invalidate (0, 0, getWidth(), getHeight());
               break;

            case ResourceEvents.EVENT_MENUITEM_HELP:
               VncClient.app().getResources().getDialog(Resources.ID_DIALOG_HELP_TOPICS).show();
               break;

            case ResourceEvents.EVENT_MENUITEM_SEND_F1:
               queueKeyForSend (KeySym.F1);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F2:
               queueKeyForSend (KeySym.F2);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F3:
               queueKeyForSend (KeySym.F3);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F4:
               queueKeyForSend (KeySym.F4);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F5:
               queueKeyForSend (KeySym.F5);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F6:
               queueKeyForSend (KeySym.F6);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F7:
               queueKeyForSend (KeySym.F7);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F8:
               queueKeyForSend (KeySym.F8);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F9:
               queueKeyForSend (KeySym.F9);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F10:
               queueKeyForSend (KeySym.F10);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F11:
               queueKeyForSend (KeySym.F11);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_F12:
               queueKeyForSend (KeySym.F12);
               break;

            case ResourceEvents.EVENT_MENUITEM_SEND_ML1:
               changeMouseButton (LEFT_MOUSE_BUTTON, BUTTON_DOWN);
               changeMouseButton (LEFT_MOUSE_BUTTON, BUTTON_UP);
               requestIncrementalSoon();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_MM1:
               changeMouseButton (MIDDLE_MOUSE_BUTTON, BUTTON_DOWN);
               changeMouseButton (MIDDLE_MOUSE_BUTTON, BUTTON_UP);
               requestIncrementalSoon();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_MR1:
               changeMouseButton (RIGHT_MOUSE_BUTTON, BUTTON_DOWN);
               changeMouseButton (RIGHT_MOUSE_BUTTON, BUTTON_UP);
               requestIncrementalSoon();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_ML2:
               queueMouseDoubleClick (LEFT_MOUSE_BUTTON);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_MM2:
               queueMouseDoubleClick (MIDDLE_MOUSE_BUTTON);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_MR2:
               queueMouseDoubleClick (RIGHT_MOUSE_BUTTON);
               break;

            case ResourceEvents.EVENT_MENUITEM_SEND_HOME:
               queueKeyForSend (KeySym.HOME);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_END:
               queueKeyForSend (KeySym.END);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_PAGEUP:
               queueKeyForSend (KeySym.PAGE_UP);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_PAGEDOWN:
               queueKeyForSend (KeySym.PAGE_DOWN);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_TAB:
               queueKeyForSend (KeySym.TAB);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_CONTROL:
               Protocol.sendKeyEvent (_session.getConnection(), KeySym.CONTROL_L, true);
               Protocol.sendKeyEvent (_session.getConnection(), KeySym.CONTROL_L, false);
               _remoteThinksControlDown = false;
               markMenuUsed();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_ALT:
               Protocol.sendKeyEvent (_session.getConnection(), KeySym.ALT_L, true);
               Protocol.sendKeyEvent (_session.getConnection(), KeySym.ALT_L, false);
               _remoteThinksControlDown = false;
               markMenuUsed();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_SHIFT:
               Protocol.sendKeyEvent (_session.getConnection(), KeySym.SHIFT_L, true);
               Protocol.sendKeyEvent (_session.getConnection(), KeySym.SHIFT_L, false);
               _remoteThinksControlDown = false;
               markMenuUsed();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_ESCAPE:
               queueKeyForSend (KeySym.ESCAPE);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_INSERT:
               queueKeyForSend (KeySym.INSERT);
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_CTRL_RBRACKET:
               onControlDown();
               queueKeyForSend (']');
               onControlUp();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_CTRL_BACKSLASH:
               onControlDown();
               queueKeyForSend ('\\');
               onControlUp();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_CTRL_CARET:
               onControlDown();
               queueKeyForSend ('^');
               onControlUp();
               break;
            case ResourceEvents.EVENT_MENUITEM_SEND_CTRL_UNDERSCORE:
               onControlDown();
               queueKeyForSend ('_');
               onControlUp();
               break;

            /*
             * Doesn't work--don't know why.  I've verified that I'm sending
             * exactly ControlDown, AltDown, DeleteDown, DeleteUp, AltUp, ControlUp
             * yet the sequence is ignored by all servers I've tested.  Tried using
             * the META_L instead of ALT_L as well--no change.
             *
            case ResourceEvents.EVENT_MENUITEM_SEND_CTRL_ALT_DEL:
               onControlDown();
               onMetaDown();
               queueKeyForSend (KeySym.DELETE);
               onMetaUp();
               onControlUp();
               break;
            *
            */

         }
      }

      return super.receiveEvent(ee);
   }

   static final int BUTTON_UP = 0;
   static final int BUTTON_DOWN = 1;

   static final int LEFT_MOUSE_BUTTON = 0;
   static final int MIDDLE_MOUSE_BUTTON = 1;
   static final int RIGHT_MOUSE_BUTTON = 2;

   public int buttonIndexToMask (int iButton)
   {
      return 1 << iButton;
   }

   public void changeMouseButton (int iButton, int upDown)
   {
      int mask = buttonIndexToMask(iButton);
      if (upDown == BUTTON_DOWN) {
         _mouseButtonsDown |= mask;
      } else if (upDown == BUTTON_UP) {
         _mouseButtonsDown &= ~mask;
      }
      _session.getCursor().sendCursorStatus(SessionCursor.SEND_IF_CHANGED);
   }

   public int getMouseButtonMask()
   {
      return _mouseButtonsDown;
   }

   public void setRefreshOnMenuUp()
   {
      if (isMenuDown()) {
         _refreshOnMenuUp = true;
      } else {
         touch();
      }
   }

   public void invalidate()
   {
      // Our position has changed; repaint the display
      invalidate (0, 0, getWidth(), getHeight());
   }

   public void touch()
   {
      // Our position has changed; repaint the display
      invalidate();

      if (_session.getState() == SessionState.CONNECTED) {
         // And advise the FrameBuffer that it needs to make sure it knows
         // what's under the now-visible regions
         _session.getFrameBuffer().refreshVisible (FrameBuffer.REFRESH_IF_UNKNOWN);

         // If we had to request data to show this region, then we'll request
         // incremental updates when the refresh overlay is done.  If we
         // already knew about this area, though, there'll be no refresh overlay
         // to wait on and we should request incrementals now.
         if (_refreshOverlay == null) {
            requestIncrementalSoon();
         }
      }
   }

   public void paint (Pen pp)
   {
      _session.paint(pp);

      if (_refreshOverlay != null) {
         _refreshOverlay.paint(pp);
      } else if (_typingOverlay != null) {
         _typingOverlay.paint(pp);
      }
   }

   public boolean wantsPagingEventsFromMenuTrackball()
   {
      return false;
   }

   public boolean isMenuDown()
   {
      return (_controlState != CONTROL_NONE);
   }

   public void markMenuUsed()
   {
      if (_controlState == CONTROL_ACTIVE) {
         _controlState = CONTROL_USED;
      }
   }

   public void markKeyEvent()
   {
      if (_controlState == CONTROL_ACTIVE) {
         _controlState = CONTROL_USED;
      }
      if (_metaState == CONTROL_ACTIVE) {
         _metaState = META_USED;
      }
   }

   public void markMouseEvent()
   {
      if (_controlState == CONTROL_ACTIVE) {
         _controlState = CONTROL_USED;
      }
      if (_metaState == META_ACTIVE) {
         _metaState = META_USED;
      }
   }

   public void onShiftDown()
   {
      sendQueuedTyping();
      Protocol.sendKeyEvent (_session.getConnection(), KeySym.SHIFT_L, true);
   }

   public void onShiftUp()
   {
      sendQueuedTyping();
      Protocol.sendKeyEvent (_session.getConnection(), KeySym.SHIFT_L, false);
   }

   public void onControlDown()
   {
      switch (_controlState)
      {
         case CONTROL_NONE:
            _controlState = CONTROL_ACTIVE;
            _refreshOnMenuUp = false;
            break;
      }
   }

   public boolean onControlUp()
   {
      switch (_controlState)
      {
         case CONTROL_ACTIVE:
            _controlState = CONTROL_NONE;
            return false;

         case CONTROL_USED:
            _controlState = CONTROL_NONE;
            if (_refreshOnMenuUp) {
               touch();
               _refreshOnMenuUp = false;
            }
            sendControlMetaState(0);
            break;
      }
      return true;
   }

   public void onMetaDown()
   {
      switch (_metaState)
      {
         case CONTROL_NONE:
            _metaState = CONTROL_ACTIVE;
            break;
      }
   }

   public void onMetaUp()
   {
      switch (_metaState)
      {
         case META_ACTIVE:
            _metaState = META_NONE;
            onMetaClick();
            break;
         case META_USED:
            _metaState = META_NONE;
            sendControlMetaState(0);
            break;
      }
   }

   public void onMetaClick()
   {
      changeMouseButton (LEFT_MOUSE_BUTTON, BUTTON_DOWN);
      changeMouseButton (LEFT_MOUSE_BUTTON, BUTTON_UP);
      requestIncrementalSoon();
   }

   public boolean eventShortcut (char ch, Event ee)
   {
      int keysym;
      if (ch == '0') {
         keysym = ']';
      } else if (ch == '1') {
         keysym = KeySym.ESCAPE;
      } else if (ch == '2') {
         keysym = '\\';
      } else if (ch == '6') {
         keysym = '^';
      } else if (ch == '9') {
         keysym = KeySym.ESCAPE;
      } else if (ch == '.') {
         keysym = '_';
      } else {
         keysym = KeySym.cookedCharToKeySym (ch, 0);
      }

      if (keysym == KeySym.ESCAPE) {
         // Not a control key!
         onControlUp();
      }

      if (keysym != 0) {
         queueKeyForSend (keysym);
      }

      markMenuUsed();
      return true;
   }

   static final int IGNORE_CONTROL = 1;

   public void sendControlMetaState (int flags)
   {
      boolean wantMeta = (_metaState != META_NONE);
      boolean wantControl = (_controlState != CONTROL_NONE);
      if ((flags & IGNORE_CONTROL) != 0) {
         wantControl = false;
      }

      if (_remoteThinksControlDown != wantControl) {
         sendQueuedTyping();
         _remoteThinksControlDown = wantControl;
         Protocol.sendKeyEvent (_session.getConnection(), KeySym.CONTROL_L, _remoteThinksControlDown);
      }

      if (_remoteThinksMetaDown != wantMeta) {
         sendQueuedTyping();
         _remoteThinksMetaDown = wantMeta;
         Protocol.sendKeyEvent (_session.getConnection(), KeySym.ALT_L, _remoteThinksMetaDown);
      }
   }

   static final String GLYPH_ALT = "\uF003";
   static final String GLYPH_UP_ARROW = "\uF004";
   static final String GLYPH_BACK = "\uF005";
   static final String GLYPH_MENU = "\uF007";
   static final String GLYPH_WHEEL = "\uF00A";
   static final String GLYPH_LEFT_ARROW = "\uF00B";
   static final String GLYPH_RIGHT_ARROW = "\uF00C";
   static final String GLYPH_DOWN_ARROW = "\uF00D";
   static final String GLYPH_CANCEL = "\uF00F";
   static final String GLYPH_LEFT_SHOULDER = "\uF022";
   static final String GLYPH_RIGHT_SHOULDER = "\uF023";
   static final String GLYPH_SYM = "\uF02C";

   void addMenuShortcuts()
   {
      Menu menu = getActionMenu();
      menu.getItemWithID(Resources.ID_MENUITEM_REFRESH).setShortcutLabel (GLYPH_CANCEL+" R");
      menu.getItemWithID(Resources.ID_MENUITEM_FULLSCREEN).setShortcutLabel (GLYPH_CANCEL+" T");
      menu.getItemWithID(Resources.ID_MENUITEM_GOTO_HOSTLIST).setShortcutLabel (GLYPH_BACK);

      Menu sub = menu.getItemWithID(Resources.ID_MENUITEM_SENDF).getSubMenu();
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F1).setShortcutLabel (GLYPH_SYM + " 1");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F2).setShortcutLabel (GLYPH_SYM + " 2");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F3).setShortcutLabel (GLYPH_SYM + " 3");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F4).setShortcutLabel (GLYPH_SYM + " 4");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F5).setShortcutLabel (GLYPH_SYM + " 5");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F6).setShortcutLabel (GLYPH_SYM + " 6");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F7).setShortcutLabel (GLYPH_SYM + " 7");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F8).setShortcutLabel (GLYPH_SYM + " 8");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F9).setShortcutLabel (GLYPH_SYM + " 9");
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_F10).setShortcutLabel (GLYPH_SYM + " 0");

      sub = menu.getItemWithID(Resources.ID_MENUITEM_SENDM).getSubMenu();
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_ML1).setShortcutLabel (GLYPH_LEFT_SHOULDER);
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_MR1).setShortcutLabel (GLYPH_RIGHT_SHOULDER);
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_ML2).setShortcutLabel (GLYPH_MENU + " " + GLYPH_LEFT_SHOULDER);
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_MR2).setShortcutLabel (GLYPH_MENU + " " + GLYPH_RIGHT_SHOULDER);

      sub = menu.getItemWithID(Resources.ID_MENUITEM_SENDS).getSubMenu();
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_END).setShortcutLabel (GLYPH_ALT+" "+GLYPH_RIGHT_ARROW);
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_HOME).setShortcutLabel (GLYPH_ALT+" "+GLYPH_LEFT_ARROW);
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_PAGEDOWN).setShortcutLabel (GLYPH_ALT+" "+GLYPH_DOWN_ARROW);
      sub.getItemWithID(Resources.ID_MENUITEM_SEND_PAGEUP).setShortcutLabel (GLYPH_ALT+" "+GLYPH_UP_ARROW);

      sub = menu.getItemWithID(Resources.ID_MENUITEM_ZOOM).getSubMenu();
      sub.getItemWithID(Resources.ID_MENUITEM_ZOOM_IN).setShortcutLabel (GLYPH_CANCEL + " " + GLYPH_RIGHT_SHOULDER);
      sub.getItemWithID(Resources.ID_MENUITEM_ZOOM_OUT).setShortcutLabel (GLYPH_CANCEL + " " + GLYPH_LEFT_SHOULDER);
   }

   Session _session;
   RemoteView _view;
   SessionStateDialog _dlgStatePopup;
   RefreshOverlay _refreshOverlay;
   TypingOverlay _typingOverlay;
   int _lastStatePopup;
   boolean _eatCancelUp;
   boolean _isCancelDown;
   boolean _refreshOnMenuUp;
   int _mouseButtonsDown;
   Rect _rDesktop;
   danger.app.Timer _incrementalTimer;
   danger.app.Timer _clearTypingTimer;
   danger.app.Timer _sendTypingTimer;
   int[] _charsToSend;
   int _charsToSendCount;
   static final int MILLI_INCREMENTAL_UPDATE = 5000;
   static final int MILLI_INCREMENTAL_UPDATE_SOON = 500;
   static final int MILLI_CLEAR_TYPING = 4000;
   static final int MILLI_SEND_TYPING = 500;
   static final int TIMER_INCREMENTAL = 1;
   static final int TIMER_CLEAR_TYPING = 2;
   static final int TIMER_SEND_TYPING = 3;

   static final int CONTROL_NONE = 0;
   static final int CONTROL_ACTIVE = 1;
   static final int CONTROL_USED = 2;
   int _controlState;

   static final int META_NONE = 0;
   static final int META_ACTIVE = 1;
   static final int META_USED = 2;
   int _metaState;

   boolean _remoteThinksControlDown;
   boolean _remoteThinksMetaDown;
}

