package com.randomsoftware.vncclient;

import danger.app.*;
import danger.ui.*;
import danger.util.*;
import java.util.*;
import java.net.*;
import java.io.*;

// The SessionConnection class maintains an open socket through which
// it communicates with the server.  It also keeps a buffer of received
// data, and notifies the listener that you supply whenever something
// changes.

public class SendThread implements Runnable
{
   // UI-Thread Interface

   public SendThread (SessionConnection conn)
   {
      _shouldRun = true;
      _connection = conn;
      _freeBuffers = new DataBufferList();
      _usedBuffers = new DataBufferList();
      _thread = new Thread (this, "VNC Client Send Thread");
      _thread.start();
   }

   public synchronized void destroy()
   {
      _shouldRun = false;
      notify();
   }

   public synchronized DataBuffer begin()
   {
      DataBuffer buf = _freeBuffers.popFirst();
      if (buf != null) {
         buf.clear();
      } else {
         buf = new DataBuffer (512);
      }
      return buf;
   }

   public synchronized void end (DataBuffer buf)
   {
      _usedBuffers.insertTail(buf);
      notify();
   }

   public synchronized void purge()
   {
      DataBuffer buf;
      while ((buf = _usedBuffers.popFirst()) != null) {
         release(buf);
      }
   }

   private synchronized void release (DataBuffer buf)
   {
      if (_freeBuffers.size() < MAX_RETAINED_BUFFERS) {
         _freeBuffers.insertTail (buf);
      }
   }

   // Background-Thread Interface

   public void run()
   {
      while (_shouldRun) {
         synchronized(this) {
            try { wait(); } catch (InterruptedException ee) { }
         }
         while (_shouldRun) {
            DataBuffer buf = null;
            synchronized(this) {
               buf = _usedBuffers.popFirst();
            }
            if (buf == null) {
               break;
            }

            try {
               Socket sock = _connection.getSocket();
               OutputStream io = sock.getOutputStream();
               io.write (buf.data, buf.ibFirst, buf.cbTotal);
            } catch (Exception e) {
            }

            synchronized(this) {
               release(buf);
            }
         }
      }
   }

   boolean _shouldRun;
   SessionConnection _connection;
   DataBufferList _freeBuffers;
   DataBufferList _usedBuffers;
   Thread _thread;

   static final int MAX_RETAINED_BUFFERS = 5;
}

