package com.randomsoftware.vncclient;

import danger.app.*;
import danger.ui.*;
import danger.util.*;
import java.util.*;
import java.net.*;
import java.io.*;

// The SessionCursor class is responsible for cursor management for this
// session.  Not an easy task, actually.

public class SessionCursor extends Listener
{
   static final int CONFIG_SHOW_REMOTE_CURSOR = 1;  // draw a square for a cursor
   static final int CONFIG_HIDE_REMOTE_CURSOR = 2;  // hide cursor by jumping to (cxDesktop,cyDesktop)
   static final int CONFIG_AUTOMATIC = 3;           // remote cursor already hidden for us

   public SessionCursor (Session ss)
   {
      ResUtil ru = VncClient.getResUtil();
      _session = ss;
      _config = (_session.getHostSettings().hideRemoteCursor) ? CONFIG_HIDE_REMOTE_CURSOR : CONFIG_SHOW_REMOTE_CURSOR;
      _ixHotspotCustom = 0;
      _iyHotspotCustom = 0;
      _ixbCursor = Compat.SCREEN_WIDTH/2;
      _iybCursor = Compat.SCREEN_HEIGHT/2;
      _bitmapCustom = null;
      _bitmapArrow = ru.getBitmap (Resources.IDB_CURSOR_ARROW);
      _bitmapSquare = ru.getBitmap (Resources.IDB_CURSOR_SQUARE);
      _history = new CursorEventList();
      _rDesktop = new Rect();
      _ixbLastSend = 0;
      _iybLastSend = 0;
      _buttonLastSend = 0;
      _ixbCursorVisual = _ixbCursor;
      _iybCursorVisual = _iybCursor;
      _cxbCursorLastChange = 0;
      _cybCursorLastChange = 0;
      _repaintTimer = Compat.new_Timer (MILLI_REPAINT, true, this, TIMER_REPAINT);
      _sendTimer = Compat.new_Timer (MILLI_CURSOR_UPDATE, true, this, TIMER_SEND);
      _sendTimer.start();
   }

   public int getConfig()
   {
      return _config;
   }

   public void setConfig (int config)
   {
      _config = config;
      sendCursorStatus(SEND_ALWAYS);
   }

   public Bitmap createBitmap (int ix, int iy, int cx, int cy)
   {
      _ixHotspotCustom = ix;
      _iyHotspotCustom = iy;
      _bitmapCustom = new Bitmap (cx, cy);
      return _bitmapCustom;
   }

   public Bitmap getBitmap()
   {
      if (_config == CONFIG_SHOW_REMOTE_CURSOR) {
         return _bitmapSquare;
      } else if (_config == CONFIG_AUTOMATIC && (_bitmapCustom != null)) {
         return _bitmapCustom;
      } else {
         return _bitmapArrow;
      }
   }

   public int getHotSpotX()
   {
      if (_config == CONFIG_SHOW_REMOTE_CURSOR) {
         return _bitmapSquare.getWidth()/2;
      } else if (_config == CONFIG_AUTOMATIC && (_bitmapCustom != null)) {
         return _ixHotspotCustom;
      } else {
         return DEFAULT_HOTSPOT_XY;
      }
   }

   public int getHotSpotY()
   {
      if (_config == CONFIG_SHOW_REMOTE_CURSOR) {
         return _bitmapSquare.getHeight()/2;
      } else if (_config == CONFIG_AUTOMATIC && (_bitmapCustom != null)) {
         return _iyHotspotCustom;
      } else {
         return DEFAULT_HOTSPOT_XY;
      }
   }

   public static final int LIMIT_TO_SCREEN = 0;
   public static final int ALLOW_SCREEN_MOVE = 2;
   public void railCursorToView (int mode)
   {
      RemoteView view = _session.getRemoteView();
      view.getVisibleBuffer (_rDesktop);

      if ((mode & ALLOW_SCREEN_MOVE) != 0) {

         // Let the current view move based on the cursor
         int ixbCenterOld = view.ixbScreenCenter;
         int iybCenterOld = view.iybScreenCenter;

         if (_ixbCursor < _rDesktop.left) {
            view.ixbScreenCenter -= _rDesktop.left - _ixbCursor;
         } else if (_ixbCursor >= _rDesktop.right) {
            view.ixbScreenCenter +=  _ixbCursor - _rDesktop.right;
         }

         if (_iybCursor < _rDesktop.top) {
            view.iybScreenCenter -= _rDesktop.top - _iybCursor;
         } else if (_iybCursor >= _rDesktop.bottom) {
            view.iybScreenCenter +=  _iybCursor - _rDesktop.bottom;
         }
         view.railViewToDesktop (RemoteView.NEVER_REFRESH);

         if ((ixbCenterOld != view.ixbScreenCenter) || (iybCenterOld != view.iybScreenCenter)) {
            _session.getMyRemoteScreen().setRefreshOnMenuUp();
         }

      } else {

         // Make sure the cursor stays inside the current view
         if (_ixbCursor < _rDesktop.left) {
            _ixbCursor = _rDesktop.left;
         } else if (_ixbCursor >= _rDesktop.right) {
            _ixbCursor = _rDesktop.right-1;
         }

         if (_iybCursor < _rDesktop.top) {
            _iybCursor = _rDesktop.top;
         } else if (_iybCursor >= _rDesktop.bottom) {
            _iybCursor = _rDesktop.bottom-1;
         }

      }
   }

   public static final int ACCEL_NORMAL = 0;
   public static final int ACCEL_HIGH = 1;

   public void moveByWheel (int dx, int dy, int accel)
   {
      int recent = _history.countRecent (dx, dy);

      int speed;
      if (recent >= 5) {
         speed = 20;
      } else if (recent >= 2) {
         speed = recent*4;
      } else if (recent == 1) {
         speed = 3;
      } else {
         speed = 1;
      }
      if ((accel & ACCEL_HIGH) != 0) {
         speed += 15;
      }

      // If you're zoomed out, move the cursor extra fast
      RemoteView view = _session.getRemoteView();
      if (view.zoomPercent < 100) {
         _ixbCursor += (dx * speed) * 100 / view.zoomPercent;
         _iybCursor += (dy * speed) * 100 / view.zoomPercent;
      } else {
         _ixbCursor += (dx * speed);
         _iybCursor += (dy * speed);
      }

      _repaintTimer.start();
      _history.append (dx, dy);
      _sendTimer.resetDelay(MILLI_CURSOR_UPDATE);

      railCursorToView(accel & ALLOW_SCREEN_MOVE);
      _session.getMyRemoteScreen().invalidate();
   }

   public void moveByPixel (int dx, int dy)
   {
      _ixbCursor += dx;
      _iybCursor += dy;
      _repaintTimer.start();
      _sendTimer.resetDelay(MILLI_CURSOR_UPDATE);
      railCursorToView(LIMIT_TO_SCREEN);
      _session.getMyRemoteScreen().invalidate();
   }

   static final int SEND_ALWAYS = 1;
   static final int SEND_IF_CHANGED = 2;
   public void sendCursorStatus (int send)
   {
      if (_session.getState() != SessionState.CONNECTED) {
         _ixbLastSend = 0;
         _iybLastSend = 0;
         _buttonLastSend = 0;
         return;
      }

      int buttonMask = 0;
      RemoteScreen screen;
      if ((screen = _session.getMyRemoteScreen()) != null) {
         buttonMask = screen.getMouseButtonMask();
      }

      int ixbTarget;
      int iybTarget;
      if (_config == CONFIG_HIDE_REMOTE_CURSOR) {
         if ((buttonMask != 0) || (_buttonLastSend != 0)) {
            ixbTarget = _ixbCursor;
            iybTarget = _iybCursor;
         } else {
            RemoteView view = _session.getRemoteView();
            ixbTarget = view.cxRemoteDesktop;
            iybTarget = view.cyRemoteDesktop;
         }
      } else {
         ixbTarget = _ixbCursor;
         iybTarget = _iybCursor;
      }

      if ((_ixbLastSend != ixbTarget) || (_iybLastSend != iybTarget) || (_buttonLastSend != buttonMask) || (send == SEND_ALWAYS)) {
         Protocol.sendPointerEvent (_session.getConnection(), ixbTarget, iybTarget, buttonMask);
         _ixbLastSend = ixbTarget;
         _iybLastSend = iybTarget;
         _buttonLastSend = buttonMask;
      }
   }

   public void sendCursorDoubleClick (int iButton)
   {
      int buttonMask = 0;
      RemoteScreen screen;
      if ((screen = _session.getMyRemoteScreen()) != null) {
         buttonMask = screen.getMouseButtonMask();
      }

      Protocol.sendPointerDoubleClick (_session.getConnection(), _ixbCursor, _iybCursor, buttonMask, iButton);
      _ixbLastSend = _ixbCursor;
      _iybLastSend = _iybCursor;
      _buttonLastSend = buttonMask;
   }

   public void paint (Pen pp)
   {
      RemoteView view = _session.getRemoteView();
      int xDraw = view.getLocalWidth()/2 + ((_ixbCursorVisual - view.ixbScreenCenter) * view.zoomPercent / 100) - getHotSpotX();
      int yDraw = view.getLocalHeight()/2 + ((_iybCursorVisual - view.iybScreenCenter) * view.zoomPercent / 100) - getHotSpotY();
      pp.drawBitmap (xDraw, yDraw, getBitmap());
   }

   public boolean receiveEvent (Event ee)
   {
      switch (Compat.Event_getType(ee))
      {
         case EventType.EVENT_TIMER:
            switch (Compat.Event_getData(ee)) {
               case TIMER_SEND:
                  sendCursorStatus(SEND_IF_CHANGED);
                  break;
               case TIMER_REPAINT:
                  easeCursorTowardTarget();
                  break;
            }
            break;
      }
      return false;
   }

   protected int getCursorVisualSpeed (int lastSpeed, int distLeft)
   {
      if (lastSpeed < 0) {
         lastSpeed = 1;
      } else if (lastSpeed < 4) {
         lastSpeed += 1;
      } else {
         lastSpeed += 2;
      }
      if (distLeft > lastSpeed * 6) {
         lastSpeed = distLeft/6;
      }
      if (lastSpeed > distLeft) {
         lastSpeed = distLeft;
      }
      return lastSpeed;
   }

   public void easeCursorTowardTarget()
   {
      boolean repaint = false;

      if (_ixbCursorVisual < _ixbCursor) {
         int maxMove = getCursorVisualSpeed(_cxbCursorLastChange, _ixbCursor - _ixbCursorVisual);
         _ixbCursorVisual += maxMove;
         _cxbCursorLastChange = maxMove;
         repaint = true;
      }

      if (_ixbCursorVisual > _ixbCursor) {
         int maxMove = 0-getCursorVisualSpeed(0-_cxbCursorLastChange, _ixbCursorVisual - _ixbCursor);
         _ixbCursorVisual += maxMove;
         _cxbCursorLastChange = maxMove;
         repaint = true;
      }

      if (_iybCursorVisual < _iybCursor) {
         int maxMove = getCursorVisualSpeed(_cybCursorLastChange, _iybCursor - _iybCursorVisual);
         _iybCursorVisual += maxMove;
         _cybCursorLastChange = maxMove;
         repaint = true;
      }

      if (_iybCursorVisual > _iybCursor) {
         int maxMove = 0-getCursorVisualSpeed(0-_cybCursorLastChange, _iybCursorVisual - _iybCursor);
         _iybCursorVisual += maxMove;
         _cybCursorLastChange = maxMove;
         repaint = true;
      }

      if (repaint) {
         RemoteScreen screen;
         if ((screen = _session.getMyRemoteScreen()) != null) {
            screen.invalidate();
         }
      } else {
         _repaintTimer.stop();
      }
   }

   Session _session;
   int _config;
   Rect _rDesktop;
   Bitmap _bitmapSquare;
   Bitmap _bitmapArrow;
   Bitmap _bitmapCustom;
   int _ixHotspotCustom;
   int _iyHotspotCustom;
   int _ixbCursor;
   int _iybCursor;
   int _ixbCursorVisual;
   int _iybCursorVisual;
   int _cxbCursorLastChange;
   int _cybCursorLastChange;
   int _ixbLastSend;
   int _iybLastSend;
   int _buttonLastSend;
   CursorEventList _history;
   danger.app.Timer _sendTimer;
   danger.app.Timer _repaintTimer;
   static final int DEFAULT_HOTSPOT_XY = 4;
   static final int MILLI_CURSOR_UPDATE = 1000;
   static final int TIMER_SEND = 1;
   static final int TIMER_REPAINT = 2;
   static final int MILLI_REPAINT = 1000 / 40;
}

class CursorEvent
{
   int dx = 0;
   int dy = 0;
   long time = 0;
}

class CursorEventList
{
   CursorEventList()
   {
      _events = new CursorEvent[ HISTORY_LENGTH ];
      for (int ii = 0; ii < HISTORY_LENGTH; ++ii) {
         _events[ii] = new CursorEvent();
      }
      _iOldest = 0;
   }

   void append (int dx, int dy)
   {
      _events[ _iOldest ].dx = dx;
      _events[ _iOldest ].dy = dy;
      _events[ _iOldest ].time = getCurrentTime();
      _iOldest = (_iOldest+1) % HISTORY_LENGTH;
   }

   int countRecent (int dx, int dy)
   {
      int count = 0;
      long timeMinimum = getCurrentTime() - MILLI_RECENT;
      for (int ii = 0; ii < HISTORY_LENGTH; ++ii) {
         if ((_events[ii].time >= timeMinimum) &&
             ((_events[ii].dx > 0) == (dx > 0)) &&
             ((_events[ii].dy > 0) == (dy > 0)) &&
             ((_events[ii].dx < 0) == (dx < 0)) &&
             ((_events[ii].dy < 0) == (dy < 0))) {
            if (dx != 0) {
               count += (_events[ii].dx > 0) ? _events[ii].dx : (0 - _events[ii].dx);
            }
            if (dy != 0) {
               count += (_events[ii].dy > 0) ? _events[ii].dy : (0 - _events[ii].dy);
            }
         }
      }
      return count;
   }

   static long getCurrentTime()
   {
      return System.currentTimeMillis();
   }

   CursorEvent[] _events;
   int _iOldest;
   static final int HISTORY_LENGTH = 15;
   static final int MILLI_RECENT = 500;
}

