package com.randomsoftware.vncclient;

import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.util.format.*;
import java.util.*;

public class SessionState
{
   static final int DISCONNECTED = 0;
   static final int CONNECTING = 1;
   static final int NEGOTIATING = 2;
   static final int CONNECTED = 3;
   static final int BAD_PASSWORD = 4;
   static final int CONNECT_FAILED = 5;
   static final int CONNECTION_LOST = 6;
   static final int BAD_VERSION = 7;
   static final int BAD_SECURITY = 8;
   static final int NO_PASSWORD = 9;
   static final int STOP_BUGGING_ME = 10;

   static public boolean isTrying (int state)
   {
      return (state == CONNECTING || state == NEGOTIATING || state == CONNECTED);
   }

   static public boolean isSessionState (int state)
   {
      switch (state) {
         case DISCONNECTED:
         case CONNECTING:
         case NEGOTIATING:
         case CONNECTED:
         case BAD_PASSWORD:
         case CONNECT_FAILED:
         case CONNECTION_LOST:
         case BAD_VERSION:
         case BAD_SECURITY:
         case NO_PASSWORD:
         case STOP_BUGGING_ME:
            return true;

         default:
            return false;
      }
   }

   static public int toString (int state)
   {
      switch (state) {
         case CONNECTING:
         case NEGOTIATING:
            return Resources.IDS_SESSIONSTATE_CONNECTING;

         case CONNECTED:
            return Resources.IDS_SESSIONSTATE_CONNECTED;

         case BAD_PASSWORD:
            return Resources.IDS_SESSIONSTATE_BAD_PASSWORD;

         case CONNECT_FAILED:
            return Resources.IDS_SESSIONSTATE_CONNECT_FAILED;

         case CONNECTION_LOST:
            return Resources.IDS_SESSIONSTATE_CONNECTION_LOST;

         case BAD_VERSION:
            return Resources.IDS_SESSIONSTATE_BAD_VERSION;

         case BAD_SECURITY:
            return Resources.IDS_SESSIONSTATE_BAD_SECURITY;

         case NO_PASSWORD:
            return Resources.IDS_SESSIONSTATE_NO_PASSWORD;

         case STOP_BUGGING_ME:
            return Resources.IDS_SESSIONSTATE_STOP_BUGGING_ME;

         default:
            return Resources.IDS_SESSIONSTATE_NOT_CONNECTED;
      }
   }

   static public int toIcon (int state)
   {
      if (isTrying(state)) {
         return Resources.IDB_SESSIONSTATE_GOOD;
      } else if (state == DISCONNECTED) {
         return Resources.IDB_SESSIONSTATE_NOTHING;
      } else {
         return Resources.IDB_SESSIONSTATE_BAD;
      }
   }

   static public boolean getsProgressPopup (int state)
   {
      return (state == CONNECTING || state == NEGOTIATING);
   }

   static public SessionStateDialog createPopup (Session session)
   {
      switch (session.getState()) {
         case CONNECTING:
         case NEGOTIATING:
            return createProgressPopup (session);

         case CONNECTED:
            // No need for a popup!
            return null;

         case NO_PASSWORD:
            return createPasswordPopup (session, Resources.IDS_SESSIONSTATEPOPUP_NO_PASSWORD);

         case BAD_PASSWORD:
            return createPasswordPopup (session, Resources.IDS_SESSIONSTATEPOPUP_BAD_PASSWORD);

         case CONNECT_FAILED:
            return createErrorPopup (session, Resources.IDS_SESSIONSTATEPOPUP_CONNECT_FAILED);

         case BAD_VERSION:
            return createErrorPopup (session, Resources.IDS_SESSIONSTATEPOPUP_BAD_VERSION);

         case BAD_SECURITY:
            return createErrorPopup (session, Resources.IDS_SESSIONSTATEPOPUP_BAD_SECURITY);

         case CONNECTION_LOST:
         case DISCONNECTED:
            return createErrorPopup (session, Resources.IDS_SESSIONSTATEPOPUP_CONNECTION_LOST);

         case STOP_BUGGING_ME:
            return createErrorPopup (session, Resources.IDS_SESSIONSTATEPOPUP_STOP_BUGGING_ME);
      }

      return null;
   }

   static protected SessionStateDialog createProgressPopup (Session session)
   {
      SessionStateDialog dlg;
      dlg = (SessionStateDialog)VncClient.app().getResources().getDialog(Resources.ID_DIALOG_STATE1);
      dlg.setTitle (session.getName());
      dlg.setBackEvent (new Event (ResourceEvents.EVENT_STATE_HIDE));
      updateProgressPopup (session, dlg);
      dlg.show();
      return dlg;
   }

   static protected void updateProgressPopup (Session session, SessionStateDialog dlg)
   {
      String ss;
      if (session.getState() == CONNECTING) {
         ss = VncClient.getResUtil().getString(Resources.IDS_CONNECTING_OPENSOCKET);
      } else {
         ss = VncClient.getResUtil().getString(HandshakeState.toString(session.getHandshakeState()));
      }

      StaticTextBox ctl = (StaticTextBox)dlg.getControlWithID(Resources.IDC_STATE_DESCRIPTION);
      ctl.setText (StringFormat.withFormat(ss, session.getName()));
      dlg.invalidate (0, 0, dlg.getWidth(), dlg.getHeight());
   }

   static protected SessionStateDialog createErrorPopup (Session session, int ids)
   {
      SessionStateDialog dlg;
      dlg = (SessionStateDialog)VncClient.app().getResources().getDialog(Resources.ID_DIALOG_STATE2);
      dlg.setTitle (session.getName());
      String ss = VncClient.getResUtil().getString(ids);
      ((StaticTextBox)dlg.getControlWithID(Resources.IDC_STATE_DESCRIPTION)).setText (StringFormat.withFormat(ss, session.getName()));
      dlg.setBackEvent (new Event (ResourceEvents.EVENT_STATE_HIDE));
      dlg.show();
      return dlg;
   }

   static protected SessionStateDialog createPasswordPopup (Session session, int ids)
   {
      SessionStateDialog dlg;
      dlg = (SessionStateDialog)VncClient.app().getResources().getDialog(Resources.ID_DIALOG_STATE3);
      dlg.setTitle (session.getName());
      String ss = VncClient.getResUtil().getString(ids);
      ((StaticTextBox)dlg.getControlWithID(Resources.IDC_STATE_DESCRIPTION)).setText (StringFormat.withFormat(ss, session.getName()));
      CheckBox ctl = (CheckBox)dlg.getControlWithID(Resources.IDC_STATE_REMEMBER);
      ctl.setValue (session.getHostSettings().password.length() != 0 ? 1 : 0);
      dlg.setBackEvent (new Event (ResourceEvents.EVENT_STATE_RETRY));
      dlg.show();
      return dlg;
   }
}

