package com.randomsoftware.vncclient;

import danger.ui.*;
import danger.app.*;
import danger.util.*;
import danger.system.*;

// The RefreshOverlay object draws the progress bar at the bottom
// of the RemoteScreen, whenever we have inbound screen data

public class TypingOverlay extends Listener
{
   public TypingOverlay (Session session, RemoteScreen screen)
   {
      _session = session;
      _screen = screen;
      _font = Compat.Font_findBoldSystemFont();
      _clrBackground = new Color (0, 28, 74);
      _clrBorder = new Color (140, 154, 231);
      _rBackground = new Rect();
      _rTextArea = new Rect();
      _cxTextArea = Compat.SCREEN_WIDTH - xGAP*2 -xyBORDER*2;
      _cyTotal = Compat.Font_getHeight (_font) + yGAP*2 + xyBORDER*2;
      _text = new StringBuffer();
      _cchLeftOfCursor = 0;
      _cursorTimer = new danger.app.Timer (MILLI_CURSOR_FLASH, true, this);
      _cursorTimer.start();
      _drawCursor = true;
   }

   public void destroy()
   {
      invalidate();
   }

   public void invalidate()
   {
      _screen.invalidate (0, _screen.getHeight() - _cyTotal, _screen.getWidth(), _screen.getHeight());
   }

   public void paint (Pen pp)
   {
      Pen p2 = pp.clonePen();

      // Draw the background field
      _rBackground.left = 0;
      _rBackground.right = _screen.getWidth();
      _rBackground.top = _screen.getHeight() - _cyTotal;
      _rBackground.bottom = _screen.getHeight() + ROUNDSIZE;
      pp.setColor(_clrBackground);
      pp.fillRoundRect (_rBackground, ROUNDSIZE, ROUNDSIZE);

      // Paint the white field behind the text
      _rTextArea.left = _rBackground.left + xGAP;
      _rTextArea.right = _rBackground.right - xGAP;
      _rTextArea.top = _rBackground.top + yGAP;
      _rTextArea.bottom = _screen.getHeight() - yGAP;
      pp.setColor(Color.WHITE);
      pp.fillRoundRect (_rTextArea, ROUNDSIZE, ROUNDSIZE);
      pp.setSize(2,2);
      pp.setColor(_clrBorder);
      pp.drawRoundRect (_rTextArea, ROUNDSIZE, ROUNDSIZE);

      // Draw the text itself
      p2.setFont (_font);
      p2.setColor (Color.BLACK);
      p2.setClip (xGAP +xyBORDER, _rBackground.top +yGAP +xyBORDER, _rBackground.right -xGAP -xyBORDER, _rBackground.bottom -yGAP -xyBORDER);

      String str = _text.toString();
      int cxLeftOfCursor = _font.getWidth(str, 0, _cchLeftOfCursor);
      int xLeftOfText = xGAP + xyBORDER;
      int yTopOfText = _rBackground.top +yGAP +xyBORDER;
      int yBottomOfText = _screen.getHeight() -yGAP -xyBORDER;

      if (cxLeftOfCursor <= _cxTextArea) {
         p2.drawText (xLeftOfText, yTopOfText + _font.getAscent(), str);
      } else {
         xLeftOfText += _cxTextArea - cxLeftOfCursor;
         p2.drawText (xLeftOfText, yTopOfText + _font.getAscent(), str);
      }

      if (_drawCursor) {
         int xCursor = xLeftOfText + cxLeftOfCursor;
         p2.setColor (Color.BLACK);
         p2.drawLine (xCursor, yTopOfText, xCursor, yBottomOfText);
      }
   }

   public void append (int keysym)
   {
      if (keysym == KeySym.BACKSPACE) {
         if (_cchLeftOfCursor > 0) {
            _text = _text.deleteCharAt (--_cchLeftOfCursor);
            invalidate();
         }
      } else if (keysym == KeySym.DELETE) {
         if (_cchLeftOfCursor < _text.length()) {
            _text = _text.deleteCharAt (_cchLeftOfCursor);
            invalidate();
         }
      } else if (keysym == KeySym.HOME) {
         _cchLeftOfCursor = 0;
         invalidate();
      } else if (keysym == KeySym.END) {
         _cchLeftOfCursor = _text.length();
         invalidate();
      } else if (keysym == KeySym.LEFT) {
         if (_cchLeftOfCursor > 0) {
            _cchLeftOfCursor--;
         }
         invalidate();
      } else if (keysym == KeySym.RIGHT) {
         if (_cchLeftOfCursor < _text.length()) {
            _cchLeftOfCursor++;
         }
         invalidate();
      } else if (KeySym.isKeySymPrintable(keysym)) {
         _text.insert (_cchLeftOfCursor++, (char)keysym);
         invalidate();
      }
   }

   public boolean receiveEvent (Event ee)
   {
      if (Compat.Event_getType(ee) == EventType.EVENT_TIMER) {
         _drawCursor = !_drawCursor;
         invalidate();
         return true;
      }
      return false;
   }

   public int size()
   {
      return _text.length();
   }

   Session _session;
   RemoteScreen _screen;
   int _cyTotal;
   int _cxTextArea;
   Font _font;
   Rect _rBackground;
   Rect _rTextArea;
   Color _clrBackground;
   Color _clrBorder;
   StringBuffer _text;
   int _cchLeftOfCursor;
   danger.app.Timer _cursorTimer;
   boolean _drawCursor;

   static final char CHAR_BACKSPACE = (char)8;

   static final int xGAP = 4;
   static final int yGAP = 4;
   static final int xyBORDER = 4;
   static final int ROUNDSIZE = 8;
   static final int MILLI_CURSOR_FLASH = 300;
}

