package com.randomsoftware.common.animation;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;

public class BufferedAnimWindow extends AnimWindow
{
   public BufferedAnimWindow (InputListener parent, int size)
   {
      super (parent, size);

      _rgnDirty = new DirtyRegion();
      _bmpBuffer = new Bitmap (_rBounds.getWidth(), _rBounds.getHeight());
      _bmpBackground = null;
   }

   public void setBackground (Bitmap bmp)
   {
      _bmpBackground = bmp;
      touch();
   }

   public void touch()
   {
      touch (_rBounds);
   }

   public void touch (Rect rr)
   {
      _rgnDirty.add (new Rect(rr));
      if (_timer == null)
         invalidate (rr);
   }

   public void invalidate()
   {
      touch();
      if (_timer == null)
         updateBuffer();
   }

   public void drawNow()
   {
      updateBuffer();
   }

   public void updateBuffer()
   {
      // Let the sequencer know we're running

      if (!_sequencer.isEmpty())
      {
         _sequencer.advanceFrame (++_framenumber);
         if (_sequencer.isEmpty())
            releaseTimer();
      }

      // Tell every AnimSprite we're doing a new frame

      for (AnimSprite obj = _sprites.getFurthest(); obj != null; obj = _sprites.getCloser(obj))
      {
         obj.update();
      }

      // Then, if anything is dirty, update the display

      if (!_rgnDirty.isEmpty())
      {
         Pen pp = _bmpBuffer.createPen();
         Enumeration ee;

         // Draw the background onto the buffer

         if (_bmpBackground != null)
         {
            if (showUpdateAreas)
            {
               Bitmap.copyBits (_bmpBuffer, _rBounds, _bmpBackground, _rBounds);
            }
            else
            {
               for (ee = _rgnDirty.elements(); ee.hasMoreElements(); )
               {
                  Rect rDirty = (Rect)ee.nextElement();
                  Bitmap.copyBits (_bmpBuffer, rDirty, _bmpBackground, rDirty);
               }
            }
         }

         // Draw sprites onto the buffer

         for (ee = _rgnDirty.elements(); ee.hasMoreElements(); )
         {
            Rect rDirty = (Rect)ee.nextElement();
            for (AnimSprite obj = _sprites.getFurthest(); obj != null; obj = _sprites.getCloser(obj))
            {
               if (rDirty.intersects (obj.pos))
               {
                  pp.push();
                  pp.offset (obj.pos.left, obj.pos.top);
                  try {
                     pp.setClip (0, 0, obj.pos.getWidth(), obj.pos.getHeight());
                  } catch (Exception e) {
                     // 1.1 simulator blows chunks here; fuck it
                  }
                  obj.paint (pp);
                  pp.pop();
               }
            }
         }

         // If we're debugging the dirty-rectangle code, show update areas

         if (showUpdateAreas)
         {
            pp.push();
            pp.setColor (Color.GREEN);

            for (ee = _rgnDirty.elements(); ee.hasMoreElements(); )
            {
               Rect rDirty = (Rect)ee.nextElement();
               pp.drawRect (rDirty);
            }

            pp.pop();
         }

         // Remind the parent window that there's stuff out of date on the screen

         if (showUpdateAreas)
         {
            invalidate (_rBounds);
         }
         else
         {
            for (ee = _rgnDirty.elements(); ee.hasMoreElements(); )
            {
               Rect rDirty = (Rect)ee.nextElement();
               invalidate (rDirty);
            }
         }

         _rgnDirty.removeAllElements();
      }
   }

   public void paint (Pen inPen)
   {
      if (_timer == null)
         updateBuffer();
      inPen.drawBitmap (0, 0, _bmpBuffer);
   }

   public boolean receiveEvent (Event e)
   {
      switch (AnimCompat.Event_getType(e))
      {
         case EventType.EVENT_TIMER:
            drawNow();
            break;
      }
      return (super.receiveEvent(e));
   }

   public void removeSprite (AnimSprite o)
   {
      if (!_sequencer.isEmpty())
      {
         _sequencer.removeSprite(o);
         if (_sequencer.isEmpty())
            releaseTimer();
      }
   }

   protected Bitmap _bmpBuffer;
   protected Bitmap _bmpBackground;
   protected DirtyRegion _rgnDirty;
}

