package com.randomsoftware.common.animation;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;

class DirtyRegion
{
   public void add (Rect rr)
   {
      // See if this rectangle is worth adding at all

      if (rr.getWidth()<0 || rr.getHeight()<0)
         return;
      if (rr.getWidth()==0 && rr.getHeight()==0)
         return;

      // Is the rectangle entirely contained in an already-dirty region?

      int cc = _vrDirty.size();
      for (int ii = 0; ii < cc; ++ii)
      {
         Rect rInArray = (Rect)_vrDirty.elementAt(ii);
         if (rInArray.contains (rr))
            return;
      }

      // Otherwise, tack this rectangle on the end of the list of rects

      int iNew = cc++;
      _vrDirty.addElement (rr);

      // Then iteratively look for rectangles to merge it with

      for (int ii = 0; ii < cc; ++ii)
      {
         if (ii == iNew)
            continue;

         Rect rInArray = (Rect)_vrDirty.elementAt(ii);
         if (!rInArray.intersects (rr))
            continue;

         // rr collides with this already-known dirty rect.  Decide if
         // we should merge the two rectangles or not: we'll merge them
         // if their combined area is less than the sum of their areas.

         Rect rUnion = new Rect (rInArray);
         rUnion.union (rr);

         int areaUnion = rUnion.getWidth() * rUnion.getHeight();
         int areaAdd = rr.getWidth() * rr.getHeight();
         int areaArray = rInArray.getWidth() * rInArray.getHeight();
         if (areaUnion > areaAdd + areaArray)
            continue;

         // Replace one rect with the union, remove the other rect
         // and restart the loop from the beginning

         int iChange = (ii <  iNew) ? ii : iNew;
         int iErase = (ii <  iNew) ? iNew : ii;

         _vrDirty.setElementAt (rUnion, iChange);
         _vrDirty.removeElementAt (iErase);
         rr = rUnion;

         iNew = iChange;
         ii = -1;
         cc--;
      }
   }

   public void removeAllElements()
   {
      _vrDirty.removeAllElements();
   }

   public Enumeration elements()
   {
      return _vrDirty.elements();
   }

   public boolean isEmpty()
   {
      return _vrDirty.isEmpty();
   }

   Vector _vrDirty = new Vector();
}

