package com.randomsoftware.common.webscore;

import java.util.*;
import danger.app.*;
import danger.net.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

class LocalScoreListListener extends Listener
{
   public LocalScoreListListener (LocalScoreList list)
   {
      _list = list;
   }

   public boolean receiveEvent (Event e)
   {
      if (e.getType() == Event.EVENT_DATASTORE_RESTORED)
         _list.reload();
      return false;
   }

   LocalScoreList _list;
};

public class LocalScoreList extends ScoreList
{
   public LocalScoreList (int maxCount)
   {
      _maxCount = maxCount;
      _listener = new LocalScoreListListener (this);
      _ds = DataStore.createDataStore ("WebScore", true);
      _ds.registerListenerForAutoSyncDone (_listener);
      _ds.setAutoSyncNotifyee (_listener);

      reload();
   }

   public void reload()
   {
      try {
         clear();
         for (int ii = 0; ii < _ds.getRecordCount(); ++ii)
            insertRecord (new ScoreRecord (_ds.getRecordData(ii)), _maxCount);
      } catch (Exception e) {
         clear();
         _ds.removeAllRecords();
      }
   }

   public void add (ScoreRecord sr)
   {
      if (insertRecord (sr, _maxCount))
      {
         _ds.removeAllRecords();

         for (int ii = 0; ii < size(); ++ii)
         {
            ScoreRecord rec = elementAt(ii);
            rec.rank = ii+1;
            _ds.addRecord (rec.toData());
         }
      }
   }

   LocalScoreListListener _listener;
   DataStore _ds;
   int _maxCount;
}

