package com.randomsoftware.common.webscore;

import java.util.*;
import danger.app.*;
import danger.net.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class WebScore extends Listener
{
   public WebScore (String game)
   {
      _game = game;
      _localScores = new LocalScoreList (MAX_LOCAL_SCORES);
      _globalScores = new GlobalScoreList (MAX_GLOBAL_SCORES);
      _timerRefresh = new danger.app.Timer (cmsecTIMER_REFRESH, true, this, TIMERID_REFRESH, null);
      _listener = null;
      _lastRefresh = 0;
      refreshGlobalScores();
   }

   public ScoreList getLocalScores()
   {
      return _localScores;
   }

   public ScoreList getGlobalScores()
   {
      return _globalScores;
   }

   public void setListener (WebScoreListener ll)
   {
      _listener = ll;
   }

   public boolean networkEvent (Object o)
   {
      if (o instanceof HTTPTransaction)
      {
         HTTPTransaction http = (HTTPTransaction)o;
         if (http.getSequenceID() == SEQUENCE_WEBSCORE_NEWSCORE)
         {
            _lastRefresh = 0;
            refreshGlobalScores();
            return true;
         }
         if (http.getSequenceID() == SEQUENCE_WEBSCORE_LISTSCORES)
         {
            _globalScores.update (http.getString());
            if (_listener != null)
               _listener.onGlobalScoresUpdated (_globalScores);
            return true;
         }
      }
      return false;
   }

   public void postLocalScore (String player, int score)
   {
      _localScores.add (new ScoreRecord (player, score));
   }

   public void postGlobalScore (String player, int score)
   {
      String safePlayer = getSafeString(player);
      HTTPConnection.get( WEBSCORE_URL + "?" + encodeString("<command action=newscore gamename="+_game+
                                                            " playername=\""+safePlayer+"\""+
                                                            " score="+score+">"),
                        "",
                        (short)0,
                        SEQUENCE_WEBSCORE_NEWSCORE );
   }

   public void refreshGlobalScores()
   {
      long currentTime = System.currentTimeMillis();
      if (currentTime > _lastRefresh + MIN_REFRESH_RATE_MSEC)
      {
         _lastRefresh = currentTime;

         HTTPConnection.get( WEBSCORE_URL + "?" + encodeString("<command action=listscores gamename="+_game+" scorecount="+MAX_GLOBAL_SCORES+">"),
                           "",
                           (short)0,
                           SEQUENCE_WEBSCORE_LISTSCORES );
      }
   }

   static char hexdigit (int val)
   {
      if (val < 10)
         return (char)(val + (int)'0');
      else if (val < 16)
         return (char)(val-10 + (int)'A');
      else
         return '-';
   }

   String encodeString (String in)
   {
      byte[] bytes = in.getBytes();

      StringBuffer out = new StringBuffer();
      for (int ii = 0; ii < bytes.length; ++ii)
      {
         if ( ((bytes[ii] >= (byte)'a') && (bytes[ii] <= (byte)'z')) ||
              ((bytes[ii] >= (byte)'A') && (bytes[ii] <= (byte)'Z')) ||
              ((bytes[ii] >= (byte)'0') && (bytes[ii] <= (byte)'9')) )
         {
            out.append ((char)bytes[ii]);
         }
         else
         {
            out.append ('%');
            out.append (hexdigit(bytes[ii]/16));
            out.append (hexdigit(bytes[ii]%16));
         }
      }

      return out.toString();
   }

   String getSafeString (String in)
   {
      byte[] bytes = in.getBytes();

      StringBuffer out = new StringBuffer();
      for (int ii = 0; ii < bytes.length; ++ii)
      {
         if ((bytes[ii] != '"') && (bytes[ii] != '<') && (bytes[ii] != '>') && (bytes[ii] != '%'))
         {
            out.append ((char)bytes[ii]);
         }
      }

      return out.toString();
   }

   public boolean receiveEvent (Event e)
   {
      if ((e.type == Event.EVENT_TIMER) && (e.data == TIMERID_REFRESH))
      {
         refreshGlobalScores();
         return true;
      }
      return false;
   }


   String _game;
   LocalScoreList _localScores;
   GlobalScoreList _globalScores;
   danger.app.Timer _timerRefresh;
   WebScoreListener _listener;
   long _lastRefresh;


   static final String WEBSCORE_URL = "http://www.randomly.com/cgi-bin/webscore.cgi";
   static final int MAX_LOCAL_SCORES = 20;
   static final int MAX_GLOBAL_SCORES = 50;
   static final int MIN_REFRESH_RATE_MSEC = 10000;

   static final int cmsecTIMER_REFRESH = 1000 * 60 * 30;
   static final int TIMERID_REFRESH = 1;

   static final int SEQUENCE_WEBSCORE_NEWSCORE = 999901;
   static final int SEQUENCE_WEBSCORE_LISTSCORES = 999902;
}

