package com.randomsoftware.common.meshlib;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Face extends BasicFace
{
   public int[] au;
   public int[] av;

   Face (Scene scene, Mesh mesh, TextureData[] td, VertexData[] vd, FaceData fd)
   {
      super (scene, mesh, td, vd, fd);
      if (texture.imageMapping != null)
      {
         // The Danger poly-filler requires clockwise rendering, but
         // our visible triangles are always counter-clockwise on the
         // screen so we have to flip a coordinate pair when drawing.
         // That means we also must flip texture coordinates to match.

         au = new int[3];
         au[0] = fd.u1;
         au[1] = fd.u3;
         au[2] = fd.u2;

         av = new int[3];
         av[0] = fd.v1;
         av[1] = fd.v3;
         av[2] = fd.v2;
      }
   }

   public Face (Scene scene, Mesh mesh, Vertex v1, Vertex v2, Vertex v3, Texture tx)
   {
      super(scene, mesh, v1, v2, v3, tx);
   }

   public void setTextureCoords (int u1, int v1, int u2, int v2, int u3, int v3)
   {
      if (au == null)
      {
         au = new int[3];
         av = new int[3];
      }

      // The Danger poly-filler requires clockwise rendering, but
      // our visible triangles are always counter-clockwise on the
      // screen so we have to flip a coordinate pair when drawing.
      // That means we also must flip texture coordinates to match.
      au[0] = u1; av[0] = v1;
      au[1] = u3; av[1] = v3;
      au[2] = u2; av[2] = v2;
   }

   public boolean isTextured()
   {
      if (texture == null || texture.imageMapping == null)
         return false;
      else if (au == null)
         return false;
      else
         return (texture.imageMapping.getBitmap(this) != null);
   }

   public void paintTextured (Pen pp, int[] ax, int[] ay)
   {
      Bitmap bmp = null;
      if (texture != null && texture.imageMapping != null && au != null)
         bmp = texture.imageMapping.getBitmap(this);
      if (bmp != null)
         TriangleClipper.fillTexturedTriangle (pp, bmp, ax, ay, au, av);
      else
         paintSolid (pp, ax, ay);
   }
}

