package com.randomsoftware.common.meshlib;

import java.util.*;
import java.io.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class FaceData
{
   int iTexture;
   int iVertex1;
   int iVertex2;
   int iVertex3;
   boolean isDoubleSided;
   int u1, v1;
   int u2, v2;
   int u3, v3;

   FaceData (DataInputStream ds, boolean haveTextures, TextureData[] textures) throws IOException
   {
      int vv = ds.readInt();
      iVertex1 = (vv >> 24) & 0xFF;
      iVertex2 = (vv >> 16) & 0xFF;
      iVertex3 =  (vv >> 8) & 0xFF;
      iTexture =         vv & 0x7F;
      isDoubleSided =  ((vv & MeshResource.JMC_FACEFLAG_DOUBLESIDED) != 0);

      if (!haveTextures)
      {
         u1 = v1 = 0;
         u2 = v2 = 0;
         u3 = v3 = 0;
      }
      else
      {
         int uv1 = ds.readInt();
         long u1per = ((uv1 >> 16) & 0xFFFF) << (FixedMath.SHIFT - 8);
         long v1per = (uv1 & 0xFFFF) << (FixedMath.SHIFT - 8);

         int uv2 = ds.readInt();
         long u2per = ((uv2 >> 16) & 0xFFFF) << (FixedMath.SHIFT - 8);
         long v2per = (uv2 & 0xFFFF) << (FixedMath.SHIFT - 8);

         int uv3 = ds.readInt();
         long u3per = ((uv3 >> 16) & 0xFFFF) << (FixedMath.SHIFT - 8);
         long v3per = (uv3 & 0xFFFF) << (FixedMath.SHIFT - 8);

         Bitmap bmp = null;
         if ((iTexture < textures.length) && (textures[iTexture].imageMapping != null))
            bmp = textures[iTexture].imageMapping.getBitmap(null);

         if (bmp == null)
         {
            u1 = v1 = 0;
            u2 = v2 = 0;
            u3 = v3 = 0;
         }
         else
         {
            int cx = bmp.getWidth();
            int cy = bmp.getHeight();

            u1 = (int)((cx * u1per) >> FixedMath.SHIFT);
            v1 = cy - (int)((cy * v1per) >> FixedMath.SHIFT);

            u2 = (int)((cx * u2per) >> FixedMath.SHIFT);
            v2 = cy - (int)((cy * v2per) >> FixedMath.SHIFT);

            u3 = (int)((cx * u3per) >> FixedMath.SHIFT);
            v3 = cy - (int)((cy * v3per) >> FixedMath.SHIFT);

            if (u1 >= cx) u1 = cx;
            if (u2 >= cx) u2 = cx;
            if (u3 >= cx) u3 = cx;
            if (v1 >= cy) v1 = cy;
            if (v2 >= cy) v2 = cy;
            if (v3 >= cy) v3 = cy;
         }
      }
   }
}

