package com.randomsoftware.common.meshlib;

import java.util.*;
import java.io.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class ShadedImageMapping extends ImageMapping
{
   public ShadedImageMapping (int idr)
   {
      _idr = idr;
   }

   public void load (ResourceDatabase rdb)
   {
      _bmp1 = Bitmap.convertToScreenColorSpace (rdb.getBitmap(_idr));

      Bitmap shadow = Bitmap.copy(_bmp1);
      Pen pp = shadow.createPen();
      pp.setColor (0, 0, 0);
      pp.fillRect (0, 0, shadow.getWidth(), shadow.getHeight());

      fadeColors(_bmp2 = Bitmap.copy (_bmp1), shadow);
      fadeColors(_bmp3 = Bitmap.copy (_bmp2), shadow);
      fadeColors(_bmp4 = Bitmap.copy (_bmp3), shadow);
      fadeColors(_bmp5 = Bitmap.copy (_bmp4), shadow);
   }

   public void unload()
   {
      _bmp1 = _bmp2 = _bmp3 = _bmp4 = _bmp5 = null;
   }

   public Bitmap getBitmap (Face ff)
   {
      long dCamera = (ff == null) ? 0 : ff._dCamera;
      if (dCamera > DEPTH_ALWAYS_SOLID)
         return null;
      else if ((_bmp5 != null) && (dCamera > DEPTH_IMAGE_5))
         return _bmp5;
      else if ((_bmp4 != null) && (dCamera > DEPTH_IMAGE_4))
         return _bmp4;
      else if ((_bmp3 != null) && (dCamera > DEPTH_IMAGE_3))
         return _bmp3;
      else if ((_bmp2 != null) && (dCamera > DEPTH_IMAGE_2))
         return _bmp2;
      else
         return _bmp1;
   }

   void fadeColors (Bitmap raw, Bitmap shadow)
   {
      Pen pp = raw.createPen();
      pp.drawBitmap (0, 0, shadow, 64);
   }

   int _idr;
   Bitmap _bmp1;
   Bitmap _bmp2;
   Bitmap _bmp3;
   Bitmap _bmp4;
   Bitmap _bmp5;

   static public final long DEPTH_ALWAYS_SOLID = (long)(2.9 * FixedMath.PREC);
   static public final long DEPTH_IMAGE_5 = (long)(2.83 * FixedMath.PREC);
   static public final long DEPTH_IMAGE_4 = (long)(2.77 * FixedMath.PREC);
   static public final long DEPTH_IMAGE_3 = (long)(2.7 * FixedMath.PREC);
   static public final long DEPTH_IMAGE_2 = (long)(2.5 * FixedMath.PREC);
}

