package com.randomsoftware.common.meshlib;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class BasicTexture
{
   public Color color;

   public Texture nextTextureInMesh;
   public Texture prevTextureInMesh;

   public BasicTexture (TextureData td)
   {
      color = new Color(td.r, td.g, td.b);
   }

   public BasicTexture (int r, int g, int b)
   {
      color = new Color(r, g, b);
   }

   public static int GETCOLOR_FOG = 0x01;
   public static int GETCOLOR_LIGHTING = 0x02;

   public void getColor (Camera cam, Face ff, int what, Color out)
   {
      if ( ((what & GETCOLOR_FOG) == 0) || (cam.fog == Camera.FOG_NONE) || (ff._mesh._dToCamera < cam.fogNearPlane) )
      {
         out.r = color.r;
         out.g = color.g;
         out.b = color.b;
      }
      else if (ff._mesh._dToCamera >= cam.fogFarPlane)
      {
         out.r = cam.fogColor.r;
         out.g = cam.fogColor.g;
         out.b = cam.fogColor.b;
      }
      else
      {
         long scale = ((cam.fogFarPlane - ff._mesh._dToCamera) << FixedMath.SHIFT) / (cam.fogFarPlane - cam.fogNearPlane);
         out.r = cam.fogColor.r + (int)(((color.r - cam.fogColor.r) * scale) >> FixedMath.SHIFT);
         out.g = cam.fogColor.g + (int)(((color.g - cam.fogColor.g) * scale) >> FixedMath.SHIFT);
         out.b = cam.fogColor.b + (int)(((color.b - cam.fogColor.b) * scale) >> FixedMath.SHIFT);
      }

      if ((what & GETCOLOR_LIGHTING) != 0)
      {
         int crLights = 0;
         int cgLights = 0;
         int cbLights = 0;

         for (FaceLight fl = ff._firstFaceLight; fl != null; fl = fl.nextFaceLightInFace)
         {
            crLights += fl.clrR;
            cgLights += fl.clrG;
            cbLights += fl.clrB;
         }

         if (crLights > 256)
            crLights = 256;
         else if (crLights < 0)
            crLights = 0;

         if (cgLights > 256)
            cgLights = 256;
         else if (cgLights < 0)
            cgLights = 0;

         if (cbLights > 256)
            cbLights = 256;
         else if (cbLights < 0)
            cbLights = 0;

         out.r = (out.r * crLights) >> 8;
         out.g = (out.g * cgLights) >> 8;
         out.b = (out.b * cbLights) >> 8;
      }
   }
}

