package com.randomsoftware.common.meshlib;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class FixedMatrix extends Object
{
   public long val00, val01, val02, val03;
   public long val10, val11, val12, val13;
   public long val20, val21, val22, val23;
   public long val30, val31, val32, val33;

   public FixedMatrix()
   {
      makeIdentity();
   }

   public FixedMatrix (FixedMatrix src)
   {
      val00 = src.val00; val01 = src.val01; val02 = src.val02; val03 = src.val03;
      val10 = src.val10; val11 = src.val11; val12 = src.val12; val13 = src.val13;
      val20 = src.val20; val21 = src.val21; val22 = src.val22; val23 = src.val23;
      val30 = src.val30; val31 = src.val31; val32 = src.val32; val33 = src.val33;
   }

   public void makeZero()
   {
      val00 = val01 = val02 = val03 =
      val10 = val11 = val12 = val13 =
      val20 = val21 = val22 = val23 =
      val30 = val31 = val32 = val33 = 0;
   }

   public void makeIdentity()
   {
      val00 = FixedMath.PREC;  val01 = 0;               val02 = 0;               val03 = 0;
      val10 = 0;               val11 = FixedMath.PREC;  val12 = 0;               val13 = 0;
      val20 = 0;               val21 = 0;               val22 = FixedMath.PREC;  val23 = 0;
      val30 = 0;               val31 = 0;               val32 = 0;               val33 = FixedMath.PREC;
   }

   public void addRotationX (long ang)
   {
      long vcos = FixedMath.cosine(ang);
      long vsin = FixedMath.sine(ang);

      temp.makeIdentity();
      temp.val11 = vcos;  temp.val12 = -vsin;
      temp.val21 = vsin;  temp.val22 = vcos;
      multiplyBy (temp);
   }

   public void addRotationY (long ang)
   {
      long vcos = FixedMath.cosine(ang);
      long vsin = FixedMath.sine(ang);

      temp.makeIdentity();
      temp.val00 = vcos;  temp.val02 = vsin;
      temp.val20 = -vsin; temp.val22 = vcos;
      multiplyBy (temp);
   }

   public void addRotationZ (long ang)
   {
      long vcos = FixedMath.cosine(ang);
      long vsin = FixedMath.sine(ang);

      temp.makeIdentity();
      temp.val00 = vcos;  temp.val01 = -vsin;
      temp.val10 = vsin;  temp.val11 = vcos;
      multiplyBy (temp);
   }

   public void addRotation (Rotation rot)
   {
      addRotationY (rot.theta);
      addRotationX (rot.gamma);
   }

   public void addRotation (Triplet axis, long ang)
   {
      long c = FixedMath.cosine(ang);
      long s = FixedMath.sine(ang);
      long t = -c;

      long tx2 = FixedMath.multiply (FixedMath.multiply(t, axis.dx), axis.dx);
      long ty2 = FixedMath.multiply (FixedMath.multiply(t, axis.dy), axis.dy);
      long tz2 = FixedMath.multiply (FixedMath.multiply(t, axis.dz), axis.dz);
      long txy = FixedMath.multiply (FixedMath.multiply(t, axis.dx), axis.dy);
      long txz = FixedMath.multiply (FixedMath.multiply(t, axis.dx), axis.dz);
      long tyz = FixedMath.multiply (FixedMath.multiply(t, axis.dy), axis.dz);
      long sx  = FixedMath.multiply (s, axis.dx);
      long sy  = FixedMath.multiply (s, axis.dy);
      long sz  = FixedMath.multiply (s, axis.dz);

      temp.val00 = tx2 + c;   temp.val01 = txy + sz;  temp.val02 = txz - sy;   temp.val03 = 0;
      temp.val10 = txy - sz;  temp.val11 = ty2 + c;   temp.val12 = tyz + sx;   temp.val13 = 0;
      temp.val20 = txy + sy;  temp.val21 = tyz - sx;  temp.val22 = tz2 + c;    temp.val23 = 0;
      temp.val30 = 0;         temp.val31 = 0;         temp.val32 = 0;          temp.val33 = FixedMath.PREC;
      multiplyBy (temp);
   }

   public void setLocation (long xx, long yy, long zz)
   {
      val03 = xx;
      val13 = yy;
      val23 = zz;
   }

   public void addTranslationX (long dist)
   {
      val03 += FixedMath.multiply (dist, val00);
      val13 += FixedMath.multiply (dist, val10);
      val23 += FixedMath.multiply (dist, val20);
   }

   public void addTranslationY (long dist)
   {
      val03 += FixedMath.multiply (dist, val01);
      val13 += FixedMath.multiply (dist, val11);
      val23 += FixedMath.multiply (dist, val21);
   }

   public void addTranslationZ (long dist)
   {
      val03 += FixedMath.multiply (dist, val02);
      val13 += FixedMath.multiply (dist, val12);
      val23 += FixedMath.multiply (dist, val22);
   }

   public void addTranslation (long dx, long dy, long dz)
   {
      val03 += ((dx * val00) + (dy * val01) + (dz * val02)) >> FixedMath.SHIFT;
      val13 += ((dx * val10) + (dy * val11) + (dz * val12)) >> FixedMath.SHIFT;
      val23 += ((dx * val20) + (dy * val21) + (dz * val22)) >> FixedMath.SHIFT;
   }

   public void addTranslation (Triplet tt)
   {
      val03 += ((tt.dx * val00) + (tt.dy * val01) + (tt.dz * val02)) >> FixedMath.SHIFT;
      val13 += ((tt.dx * val10) + (tt.dy * val11) + (tt.dz * val12)) >> FixedMath.SHIFT;
      val23 += ((tt.dx * val20) + (tt.dy * val21) + (tt.dz * val22)) >> FixedMath.SHIFT;
   }

   public void scale (long dx, long dy, long dz)
   {
      temp.makeIdentity();
      temp.val00 = dx;
      temp.val11 = dy;
      temp.val22 = dz;
      multiplyBy (temp);
   }

   public void multiplyBy (FixedMatrix mult)
   {
      long old0, old1, old2, old3;

      old0 = val00; old1 = val01; old2 = val02; old3 = val03;
      val00 = ((old0 * mult.val00) + (old1 * mult.val10) + (old2 * mult.val20) + (old3 * mult.val30)) >> FixedMath.SHIFT;
      val01 = ((old0 * mult.val01) + (old1 * mult.val11) + (old2 * mult.val21) + (old3 * mult.val31)) >> FixedMath.SHIFT;
      val02 = ((old0 * mult.val02) + (old1 * mult.val12) + (old2 * mult.val22) + (old3 * mult.val32)) >> FixedMath.SHIFT;
      val03 = ((old0 * mult.val03) + (old1 * mult.val13) + (old2 * mult.val23) + (old3 * mult.val33)) >> FixedMath.SHIFT;

      old0 = val10; old1 = val11; old2 = val12; old3 = val13;
      val10 = ((old0 * mult.val00) + (old1 * mult.val10) + (old2 * mult.val20) + (old3 * mult.val30)) >> FixedMath.SHIFT;
      val11 = ((old0 * mult.val01) + (old1 * mult.val11) + (old2 * mult.val21) + (old3 * mult.val31)) >> FixedMath.SHIFT;
      val12 = ((old0 * mult.val02) + (old1 * mult.val12) + (old2 * mult.val22) + (old3 * mult.val32)) >> FixedMath.SHIFT;
      val13 = ((old0 * mult.val03) + (old1 * mult.val13) + (old2 * mult.val23) + (old3 * mult.val33)) >> FixedMath.SHIFT;

      old0 = val20; old1 = val21; old2 = val22; old3 = val23;
      val20 = ((old0 * mult.val00) + (old1 * mult.val10) + (old2 * mult.val20) + (old3 * mult.val30)) >> FixedMath.SHIFT;
      val21 = ((old0 * mult.val01) + (old1 * mult.val11) + (old2 * mult.val21) + (old3 * mult.val31)) >> FixedMath.SHIFT;
      val22 = ((old0 * mult.val02) + (old1 * mult.val12) + (old2 * mult.val22) + (old3 * mult.val32)) >> FixedMath.SHIFT;
      val23 = ((old0 * mult.val03) + (old1 * mult.val13) + (old2 * mult.val23) + (old3 * mult.val33)) >> FixedMath.SHIFT;

      old0 = val30; old1 = val31; old2 = val32; old3 = val33;
      val30 = ((old0 * mult.val00) + (old1 * mult.val10) + (old2 * mult.val20) + (old3 * mult.val30)) >> FixedMath.SHIFT;
      val31 = ((old0 * mult.val01) + (old1 * mult.val11) + (old2 * mult.val21) + (old3 * mult.val31)) >> FixedMath.SHIFT;
      val32 = ((old0 * mult.val02) + (old1 * mult.val12) + (old2 * mult.val22) + (old3 * mult.val32)) >> FixedMath.SHIFT;
      val33 = ((old0 * mult.val03) + (old1 * mult.val13) + (old2 * mult.val23) + (old3 * mult.val33)) >> FixedMath.SHIFT;
   }

   public void makeMultiply (FixedMatrix matrixA, FixedMatrix matrixB)
   {
      val00 = ((matrixA.val00 * matrixB.val00) + (matrixA.val01 * matrixB.val10) + (matrixA.val02 * matrixB.val20) + (matrixA.val03 * matrixB.val30)) >> FixedMath.SHIFT;
      val01 = ((matrixA.val00 * matrixB.val01) + (matrixA.val01 * matrixB.val11) + (matrixA.val02 * matrixB.val21) + (matrixA.val03 * matrixB.val31)) >> FixedMath.SHIFT;
      val02 = ((matrixA.val00 * matrixB.val02) + (matrixA.val01 * matrixB.val12) + (matrixA.val02 * matrixB.val22) + (matrixA.val03 * matrixB.val32)) >> FixedMath.SHIFT;
      val03 = ((matrixA.val00 * matrixB.val03) + (matrixA.val01 * matrixB.val13) + (matrixA.val02 * matrixB.val23) + (matrixA.val03 * matrixB.val33)) >> FixedMath.SHIFT;

      val10 = ((matrixA.val10 * matrixB.val00) + (matrixA.val11 * matrixB.val10) + (matrixA.val12 * matrixB.val20) + (matrixA.val13 * matrixB.val30)) >> FixedMath.SHIFT;
      val11 = ((matrixA.val10 * matrixB.val01) + (matrixA.val11 * matrixB.val11) + (matrixA.val12 * matrixB.val21) + (matrixA.val13 * matrixB.val31)) >> FixedMath.SHIFT;
      val12 = ((matrixA.val10 * matrixB.val02) + (matrixA.val11 * matrixB.val12) + (matrixA.val12 * matrixB.val22) + (matrixA.val13 * matrixB.val32)) >> FixedMath.SHIFT;
      val13 = ((matrixA.val10 * matrixB.val03) + (matrixA.val11 * matrixB.val13) + (matrixA.val12 * matrixB.val23) + (matrixA.val13 * matrixB.val33)) >> FixedMath.SHIFT;

      val20 = ((matrixA.val20 * matrixB.val00) + (matrixA.val21 * matrixB.val10) + (matrixA.val22 * matrixB.val20) + (matrixA.val23 * matrixB.val30)) >> FixedMath.SHIFT;
      val21 = ((matrixA.val20 * matrixB.val01) + (matrixA.val21 * matrixB.val11) + (matrixA.val22 * matrixB.val21) + (matrixA.val23 * matrixB.val31)) >> FixedMath.SHIFT;
      val22 = ((matrixA.val20 * matrixB.val02) + (matrixA.val21 * matrixB.val12) + (matrixA.val22 * matrixB.val22) + (matrixA.val23 * matrixB.val32)) >> FixedMath.SHIFT;
      val23 = ((matrixA.val20 * matrixB.val03) + (matrixA.val21 * matrixB.val13) + (matrixA.val22 * matrixB.val23) + (matrixA.val23 * matrixB.val33)) >> FixedMath.SHIFT;

      val30 = ((matrixA.val30 * matrixB.val00) + (matrixA.val31 * matrixB.val10) + (matrixA.val32 * matrixB.val20) + (matrixA.val33 * matrixB.val30)) >> FixedMath.SHIFT;
      val31 = ((matrixA.val30 * matrixB.val01) + (matrixA.val31 * matrixB.val11) + (matrixA.val32 * matrixB.val21) + (matrixA.val33 * matrixB.val31)) >> FixedMath.SHIFT;
      val32 = ((matrixA.val30 * matrixB.val02) + (matrixA.val31 * matrixB.val12) + (matrixA.val32 * matrixB.val22) + (matrixA.val33 * matrixB.val32)) >> FixedMath.SHIFT;
      val33 = ((matrixA.val30 * matrixB.val03) + (matrixA.val31 * matrixB.val13) + (matrixA.val32 * matrixB.val23) + (matrixA.val33 * matrixB.val33)) >> FixedMath.SHIFT;
   }

   static FixedMatrix temp = new FixedMatrix();
}

