package com.randomsoftware.common.meshlib;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Light extends Triplet
{
   public static final int AMBIENT_LIGHT = 1;
   public static final int POINT_LIGHT = 2;

   public Light nextLightInScene;
   public Light prevLightInScene;

   public Light (Scene sc)
   {
      _color = new Color (255, 255, 255);
      _scene = sc;
      _type = POINT_LIGHT;

      // Insert(scene.lights)
      if ((prevLightInScene = _scene.lastLight) != null)
         prevLightInScene.nextLightInScene = this;
      else
         _scene.firstLight = this;
      nextLightInScene = null;
      _scene.lastLight = this;
   }

   public Light (Scene sc, Color cc, int tt)
   {
      _color = cc;
      _scene = sc;
      _type = tt;

      // Insert(scene.lights)
      if ((prevLightInScene = _scene.lastLight) != null)
         prevLightInScene.nextLightInScene = this;
      else
         _scene.firstLight = this;
      nextLightInScene = null;
      _scene.lastLight = this;
   }

   public Light (Scene sc, Color cc, Triplet pp)
   {
      super(pp);
      _color = cc;
      _scene = sc;
      _type = POINT_LIGHT;

      // Insert(scene.lights)
      if ((prevLightInScene = _scene.lastLight) != null)
         prevLightInScene.nextLightInScene = this;
      else
         _scene.firstLight = this;
      nextLightInScene = null;
      _scene.lastLight = this;
   }

   public void destroy()
   {
      clearLights();

      // Remove(scene.lights)
      if (nextLightInScene != null)
         nextLightInScene.prevLightInScene = prevLightInScene;
      else
         _scene.lastLight = prevLightInScene;
      if (prevLightInScene != null)
         prevLightInScene.nextLightInScene = nextLightInScene;
      else
         _scene.firstLight = nextLightInScene;
   }

   public void clearLights()
   {
      while (_firstFaceLight != null)
      {
         _firstFaceLight.removeFromFace();
         _scene._faceLightFactory.release(_firstFaceLight);
         _firstFaceLight = _firstFaceLight.nextFaceLightInLight;
      }

      _firstFaceLight = null;
      _lastFaceLight = null;
   }

   public Triplet getPos()
   {
      return this;
   }

   public void setPos (Triplet src)
   {
      dx = src.dx;
      dy = src.dy;
      dz = src.dz;
      dw = src.dw;
      _fNeedsRelight = true;
   }

   public Color getColor()
   {
      return _color;
   }

   public void setColor (Color clr)
   {
      _color = clr;
      _fNeedsRelight = true;
   }

   void relightMeshFaces (Mesh mm, Camera cam)
   {
      for (Face face = mm.firstFace; face != null; face = face.nextFaceInMesh)
      {
         long diffuse = calculateLighting (face.vertex1._vPosInWorld, face._vNormalInWorld, cam);
         if (diffuse == 0)
            continue;

         FaceLight fl = _scene._faceLightFactory.alloc();
         fl.clrR = (int)((diffuse * _color.r) >> FixedMath.SHIFT);
         fl.clrG = (int)((diffuse * _color.g) >> FixedMath.SHIFT);
         fl.clrB = (int)((diffuse * _color.b) >> FixedMath.SHIFT);
         fl.light = this;
         fl.face = face;

         fl.insertIntoFace();
         fl.insertIntoLight();
      }
   }

   long calculateLighting (Triplet posSurface, Triplet normSurface, Camera cam)
   {
      if (_type == AMBIENT_LIGHT)
         return FixedMath.PREC;

      // Calculate diffuse component--does surface face the light?

      _tSurfaceToLight.dx = dx-posSurface.dx;
      _tSurfaceToLight.dy = dy-posSurface.dy;
      _tSurfaceToLight.dz = dz-posSurface.dz;

      long dist = _tSurfaceToLight.getMagnitude();
      if (dist < 100)  // arbitrary value
         return 0;

      long incidence = ( (_tSurfaceToLight.dx*normSurface.dx) + 
                         (_tSurfaceToLight.dy*normSurface.dy) + 
                         (_tSurfaceToLight.dz*normSurface.dz) ) / dist;

      if (incidence <= 0)
         return 0;
      else if (incidence >= FixedMath.PREC)
         return FixedMath.PREC;
      else
         return incidence;
   }

   int _type;
   Scene _scene;
   Color _color;
   boolean _fNeedsRelight = true;
   static Triplet _tSurfaceToLight = new Triplet();

   public FaceLight _firstFaceLight = null;
   public FaceLight _lastFaceLight = null;
}

