package com.randomsoftware.common.meshlib;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Rotation
{
   public long theta;
   public long gamma;

   public Rotation()
   {
      theta = gamma = 0;
   }

   public Rotation (Rotation src)
   {
      theta = src.theta;
      gamma = src.gamma;
   }

   public Rotation (long th, long ga)
   {
      theta = th;
      gamma = ga;
   }

   public Rotation (long vx, long vy, long vz)
   {
      theta = FixedMath.arctangent (vx, vz);
      gamma = FixedMath.arctangent (0-vy, FixedMath.distance (vx, vz));
   }

   public Rotation (Triplet vv)
   {
      theta = FixedMath.arctangent (vv.dx, vv.dz);
      gamma = FixedMath.arctangent (0-vv.dy, FixedMath.distance (vv.dx, vv.dz));
   }

   public Rotation (Triplet src, Triplet dst)
   {
      long vx = dst.dx - src.dx;
      long vy = dst.dy - src.dy;
      long vz = dst.dz - src.dz;
      theta = FixedMath.arctangent (vx, vz);
      gamma = FixedMath.arctangent (0-vy, FixedMath.distance (vx, vz));
   }

   public void normalize()
   {
      theta = FixedMath.normalize(theta);
      gamma = FixedMath.normalize(gamma);
   }

   public void add (Rotation rot)
   {
      theta += rot.theta;
      gamma += rot.gamma;
   }

   public void subtract (Rotation rot)
   {
      theta -= rot.theta;
      gamma -= rot.gamma;
   }

   public boolean equals (Rotation rot)
   {
      return (theta == rot.theta) && (gamma == rot.gamma);
   }
}

