package com.randomsoftware.common.meshlib;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public abstract class Trigger extends FaceSortable
{
   public Trigger nextTriggerInScene;
   public Trigger prevTriggerInScene;

   public Trigger nextTriggerInActor;
   public Trigger prevTriggerInActor;

   public Trigger (Scene sc)
   {
      _scene = sc;
      _actor = null;
      _vPosInParent = new Triplet();

      // Insert(scene.triggers)
      if ((prevTriggerInScene = _scene.lastTrigger) != null)
         prevTriggerInScene.nextTriggerInScene = this;
      else
         _scene.firstTrigger = this;
      nextTriggerInScene = null;
      _scene.lastTrigger = this;
      _scene.countTriggers++;
   }

   public Trigger (Scene sc, long x, long y, long z)
   {
      _vPosInParent = new Triplet (x, y, z);
      _scene = sc;
      _actor = null;

      // Insert(scene.triggers)
      if ((prevTriggerInScene = _scene.lastTrigger) != null)
         prevTriggerInScene.nextTriggerInScene = this;
      else
         _scene.firstTrigger = this;
      nextTriggerInScene = null;
      _scene.lastTrigger = this;
      _scene.countTriggers++;
   }

   public Trigger (Actor ac)
   {
      _scene = ac._scene;
      _actor = ac;
      _vPosInParent = new Triplet();
      _vPosInWorld = new Triplet();

      // Insert(scene.triggers)
      if ((prevTriggerInScene = _scene.lastTrigger) != null)
         prevTriggerInScene.nextTriggerInScene = this;
      else
         _scene.firstTrigger = this;
      nextTriggerInScene = null;
      _scene.lastTrigger = this;
      _scene.countTriggers++;

      // Insert(actor.triggers)
      if ((prevTriggerInActor = _actor.lastTrigger) != null)
         prevTriggerInActor.nextTriggerInActor = this;
      else
         _actor.firstTrigger = this;
      nextTriggerInActor = null;
      _actor.lastTrigger = this;
   }

   public Trigger (Actor ac, Triplet tt)
   {
      _scene = ac._scene;
      _actor = ac;
      _vPosInWorld = new Triplet();
      _vPosInParent = new Triplet(tt);

      // Insert(scene.triggers)
      if ((prevTriggerInScene = _scene.lastTrigger) != null)
         prevTriggerInScene.nextTriggerInScene = this;
      else
         _scene.firstTrigger = this;
      nextTriggerInScene = null;
      _scene.lastTrigger = this;
      _scene.countTriggers++;

      // Insert(actor.triggers)
      if ((prevTriggerInActor = _actor.lastTrigger) != null)
         prevTriggerInActor.nextTriggerInActor = this;
      else
         _actor.firstTrigger = this;
      nextTriggerInActor = null;
      _actor.lastTrigger = this;
   }

   public void destroy()
   {
      // Remove(scene.triggers)
      if (nextTriggerInScene != null)
         nextTriggerInScene.prevTriggerInScene = prevTriggerInScene;
      else
         _scene.lastTrigger = prevTriggerInScene;
      if (prevTriggerInScene != null)
         prevTriggerInScene.nextTriggerInScene = nextTriggerInScene;
      else
         _scene.firstTrigger = nextTriggerInScene;
      _scene.countTriggers--;

      if (_actor != null)
      {
         // Remove(actor.triggers)
         if (nextTriggerInActor != null)
            nextTriggerInActor.prevTriggerInActor = prevTriggerInActor;
         else
            _actor.lastTrigger = prevTriggerInActor;
         if (prevTriggerInActor != null)
            prevTriggerInActor.nextTriggerInActor = nextTriggerInActor;
         else
            _actor.firstTrigger = nextTriggerInActor;
      }
   }

   public void setPosition (long x, long y, long z)
   {
      _vPosInParent.dx = x;
      _vPosInParent.dy = y;
      _vPosInParent.dz = z;
      invalidate();
   }

   public void invalidate()
   {
      _isChanged = true;
   }

   public void recalculate (FixedMatrix mxWorldToCamera)
   {
      if (_actor != null)
      {
         _vPosInWorld.makeMultiply (_actor, _vPosInParent);
         _vPosInCamera.makeHomogenizedMultiply (mxWorldToCamera, _vPosInWorld);
      }
      else
      {
         _vPosInCamera.makeHomogenizedMultiply (mxWorldToCamera, _vPosInParent);
      }

      _dCamera = 0 - _vPosInCamera.dz;
      _isChanged = false;
   }

   abstract public void update ();
   abstract public void paint (Pen pp, int xx, int yy);

   int _dxScreen;
   int _dyScreen;
   Scene _scene;
   Actor _actor;
   Triplet _vPosInCamera = new Triplet();
   Triplet _vPosInParent = null;
   Triplet _vPosInWorld = null;
   boolean _isChanged = true;
}

