package com.randomsoftware.common.meshlib;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class Triplet
{
   public long dx;
   public long dy;
   public long dz;
   public long dw;

   public Triplet()
   {
      dx = dy = dz = 0;
      dw = FixedMath.PREC;
   }

   public Triplet (Triplet src)
   {
      dx = src.dx;
      dy = src.dy;
      dz = src.dz;
      dw = src.dw;
   }

   public Triplet (long x, long y, long z)
   {
      dx = x;
      dy = y;
      dz = z;
      dw = FixedMath.PREC;
   }

   public Triplet (long x, long y, long z, long w)
   {
      dx = x;
      dy = y;
      dz = z;
      dw = w;
   }

   public long getMagnitude()
   {
      return FixedMath.distance (dx, dy, dz);
   }

   public long getMagnitudeSquared()
   {
      return FixedMath.multiply(dx,dx) + FixedMath.multiply(dy,dy) + FixedMath.multiply(dz,dz);
   }

   public void normalize()
   {
      long ss = getMagnitude();
      if (ss != 0)
      {
         dx = FixedMath.divide(dx, ss);
         dy = FixedMath.divide(dy, ss);
         dz = FixedMath.divide(dz, ss);
      }
   }

   public void homogenize()
   {
      dx = FixedMath.divide(dx, dw);
      dy = FixedMath.divide(dy, dw);
      dz = FixedMath.divide(dz, dw);
      dw = FixedMath.PREC;
   }

   public void multiplyBy (long vv)
   {
      dx = FixedMath.multiply(dx, vv);
      dy = FixedMath.multiply(dy, vv);
      dz = FixedMath.multiply(dz, vv);
   }

   public void subtract (Triplet tt)
   {
      dx -= tt.dx;
      dy -= tt.dy;
      dz -= tt.dz;
   }

   public void add (Triplet tt)
   {
      dx += tt.dx;
      dy += tt.dy;
      dz += tt.dz;
   }

   public boolean equals (Triplet tt)
   {
      return (dx == tt.dx) && (dy == tt.dy) && (dz == tt.dz);
   }

   public void makeCross (Triplet va, Triplet vb)
   {
      dx = (va.dy*vb.dz - va.dz*vb.dy) >> FixedMath.SHIFT;
      dy = (va.dz*vb.dx - va.dx*vb.dz) >> FixedMath.SHIFT;
      dz = (va.dx*vb.dy - va.dy*vb.dx) >> FixedMath.SHIFT;
   }

   public void makeNormalizedCross (Triplet va, Triplet vb)
   {
      dx = (va.dy*vb.dz - va.dz*vb.dy) >> FixedMath.SHIFT;
      dy = (va.dz*vb.dx - va.dx*vb.dz) >> FixedMath.SHIFT;
      dz = (va.dx*vb.dy - va.dy*vb.dx) >> FixedMath.SHIFT;
      normalize();
   }

   public void makeMultiply (FixedMatrix mx, Triplet vv)
   {
      dx = ((vv.dx*mx.val00) + (vv.dy*mx.val01) + (vv.dz*mx.val02) + (vv.dw*mx.val03)) >> FixedMath.SHIFT;
      dy = ((vv.dx*mx.val10) + (vv.dy*mx.val11) + (vv.dz*mx.val12) + (vv.dw*mx.val13)) >> FixedMath.SHIFT;
      dz = ((vv.dx*mx.val20) + (vv.dy*mx.val21) + (vv.dz*mx.val22) + (vv.dw*mx.val23)) >> FixedMath.SHIFT;
      dw = ((vv.dx*mx.val30) + (vv.dy*mx.val31) + (vv.dz*mx.val32) + (vv.dw*mx.val33)) >> FixedMath.SHIFT;
   }

   public void makeHomogenizedMultiply (FixedMatrix mx, Triplet vv)
   {
      dw = ((vv.dx*mx.val30) + (vv.dy*mx.val31) + (vv.dz*mx.val32) + (vv.dw*mx.val33)) >> FixedMath.SHIFT;
      if (dw < FixedMath.PREC)
         dw = FixedMath.PREC;
      dx = ((vv.dx*mx.val00) + (vv.dy*mx.val01) + (vv.dz*mx.val02) + (vv.dw*mx.val03)) / dw;
      dy = ((vv.dx*mx.val10) + (vv.dy*mx.val11) + (vv.dz*mx.val12) + (vv.dw*mx.val13)) / dw;
      dz = ((vv.dx*mx.val20) + (vv.dy*mx.val21) + (vv.dz*mx.val22) + (vv.dw*mx.val23)) / dw;
      dw = FixedMath.PREC;
   }

   public static long dot (Triplet va, Triplet vb)
   {
      return ((va.dx*vb.dx) + (va.dy*vb.dy) + (va.dz*vb.dz)) >> FixedMath.SHIFT;
   }
}

