package com.randomsoftware.common.resloader;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;
import danger.system.*;

abstract public class ResLoader extends AnimSprite implements Runnable, ResLoaderThread
{
   public ResLoader (AnimWindow win, ResLoaderShow show)
   {
      super (win, DEPTH_RESLOADER);
      _show = show;

      Rect rr = new Rect();
      rr.left = rr.top = 0;
      rr.right = ResLibCompat.SCREEN_WIDTH;
      rr.bottom = ResLibCompat.SCREEN_HEIGHT;
      setRect(rr);

      _discard = false;
      _ready = false;
   }

   public void start()
   {
      _thread = new Thread (this, "ResLoaderThread");
      _thread.start();
   }

   public void destroy()
   {
      synchronized(this)
      {
         if (_thread != null)
         {
            _discard = true;
         }
         else if (_ready)
         {
            _ready = false;
            unload();
         }
      }

      if (_show != null)
      {
         _show.destroy();
         _show = null;
      }

      super.destroy();
   }

   public void run()
   {
      load();

      synchronized(this)
      {
         _thread = null;
         if (_discard)
            unload();
         else
            _ready = true;
      }
   }

   public boolean ready()
   {
      return (_ready) && (_show == null);
   }

   public void update()
   {
      if (_show != null)
      {
         int rc = _show.update(_ready);
         if (rc == ResLoaderShow.SHOW_IS_FINISHED)
         {
            _show.destroy();
            _show = null;
            setRect (new Rect());
         }
         else if (rc == ResLoaderShow.SHOW_STILL_RUNNING_SO_REPAINT)
         {
            touch();
         }
      }
   }

   public void paint (Pen pp)
   {
      if (_show != null)
         _show.paint(pp);
   }

   protected int _type;
   protected Thread _thread;
   protected boolean _discard;
   protected boolean _ready;
   protected ResLoaderShow _show;

   static final int DEPTH_RESLOADER = -9999;
}

