package com.randomsoftware.common.resloader;
import com.randomsoftware.common.animation.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class ResLoaderFullLogo implements ResLoaderShow
{
   public ResLoaderFullLogo (Window win, int idq)
   {
      _win = win;
      _frame = 0;
      _flooded = false;
      _sounded = false;
      _fadedSound = false;
      _cchSkipLast = 0;
      _switchedAway = false;
      _collected = false;

      _lines = new ResLoaderLine[ NUM_LINES ];
      for (int ii = 0; ii < NUM_LINES; ++ii)
         _lines[ii] = new ResLoaderLine();

      _font = ResLibCompat.getFont();

      _cxText = (int)(_font.getWidth(BANNER_TEXT, 0, BANNER_LENGTH) * 1.5);
      _cyText = _font.getAscent() + _font.getDescent() + 1;
      _yyText = _font.getAscent();
      _cxLetterMax = 0;

      int xLeft = (ResLibCompat.SCREEN_WIDTH - (_cxText * (BANNER_LENGTH-1) / BANNER_LENGTH)) / 2;
      _ixLetter = new int[ BANNER_LENGTH ];
      for (int ii = 0; ii < BANNER_LENGTH; ++ii) {
         int cxLetter = _font.getWidth(BANNER_TEXT, ii, 1);
         _cxLetterMax = (cxLetter > _cxLetterMax) ? cxLetter : _cxLetterMax;
         _ixLetter[ii] = xLeft + _cxText * ii / BANNER_LENGTH;
         _ixLetter[ii] -= cxLetter/2;
      }

      try {
         _seq = AudioManager.createSequence();
         _seq.loadFromResource(idq);
         _seq.setVolume(MAX_VOLUME);
      } catch (AudioException e) {
         if (_seq != null)
            _seq.destroy();
         _seq = null;
      }
   }

   public void destroy()
   {
      if (_seq != null)
         _seq.destroy();
      _seq = null;
      _lines = null;
      _font = null;
      _ixLetter = null;
   }

   public int update (boolean ready)
   {
      if (_lines == null)
      {
         if (_seq != null)
            _seq.stop();
         return SHOW_IS_FINISHED;
      }

      if (!Screen.isWindowActive (_win))
      {
         _switchedAway = true;
      }
      else if (_switchedAway)  // switched back now?
      {
         _switchedAway = false;
         _flooded = false;
      }
      if (_flooded && !_collected)
      {
         _collected = true;
         System.gc();
      }

      for (int ii = 0; ii < NUM_LINES; ++ii)
         _lines[ii].update();

      if (_switchedAway && _sounded)
      {
         if (_seq != null)
            _seq.stop();
         _sounded = false;
      }

      if ((_frame >= FRAME_SOUND_START) && (!_sounded) && (!_switchedAway))
      {
         _sounded = true;
         if (_seq != null)
            _seq.start();
      }

      if ((_frame >= FRAME_FADEOUT_START) && (!ready))
         return SHOW_STILL_RUNNING_SO_REPAINT;

      if ((_frame >= FRAME_FADEOUT_START) && (!_fadedSound) && (!_switchedAway))
      {
         _fadedSound = true;
         if (_seq != null)
            _seq.fadeFromToInTime (35, 0, (FRAME_FADEOUT_DONE - _frame) * 1000 / 20);
      }

      _frame++;

      if (_frame < FRAME_COLORCHANGE_START)
      {
         _textBright = COLOR_DIM;
      }
      if (_frame >= FRAME_COLORCHANGE_START && _frame <= FRAME_COLORCHANGE_DONE)
      {
         _textBright = COLOR_DIM + (255 - COLOR_DIM) * (_frame - FRAME_COLORCHANGE_START) / (FRAME_COLORCHANGE_DONE - FRAME_COLORCHANGE_START);
      }

      if (_frame < FRAME_FADEIN_START)
      {
         _lineBright = LINEBRIGHT_MAX;
         _cchSolidIn = 0;
         _cchFadeIn = 0;
         _cchSkip = 0;
         _cchFadeOut = 0;
         _cchSolidOut = 0;
      }
      else if (_frame < FRAME_FADEIN_DONE)
      {
         int cchAdvance = (_frame - FRAME_FADEIN_START) * (CCH_FADE + BANNER_LENGTH) / (FRAME_FADEIN_DONE - FRAME_FADEIN_START);
         _cchSolidIn = (cchAdvance > CCH_FADE) ? (cchAdvance - CCH_FADE) : 0;
         _cchFadeIn = cchAdvance - _cchSolidIn;
         if (_cchSolidIn + _cchFadeIn > BANNER_LENGTH)
            _cchFadeIn = BANNER_LENGTH - _cchSolidIn;
      }
      else if (_frame < FRAME_FADEOUT_START)
      {
         _cchSolidIn = BANNER_LENGTH;
         _cchFadeIn = 0;
      }
      else if (_frame < FRAME_FADEOUT_DONE)
      {
         if (_frame < FRAME_FADEOUT_TEXTDONE)
         {
            _cchSolidIn = 0;
            _cchFadeIn = 0;

            int cchAdvance = (_frame - FRAME_FADEOUT_START) * (CCH_FADE + BANNER_LENGTH) / (FRAME_FADEOUT_TEXTDONE - FRAME_FADEOUT_START);
            _cchSolidOut = BANNER_LENGTH - cchAdvance;
            if (_cchSolidOut < 0)
               _cchSolidOut = 0;

            _cchSkip = (cchAdvance > CCH_FADE) ? cchAdvance - CCH_FADE : 0;
            _cchFadeOut = BANNER_LENGTH - _cchSolidOut - _cchSkip;
         }
         else
         {
            _cchSolidIn = _cchSolidOut = _cchFadeIn = _cchFadeOut = 0;
            _cchSkip = BANNER_LENGTH;
         }

         _lineBright = (FRAME_FADEOUT_DONE - _frame) * LINEBRIGHT_MAX / (FRAME_FADEOUT_DONE - FRAME_FADEOUT_START);
      }
      else
      {
         _cchSolidIn = _cchSolidOut = _cchFadeIn = _cchFadeOut = 0;
         _cchSkip = BANNER_LENGTH;
         if (ready)
            return SHOW_IS_FINISHED;
      }

      return SHOW_STILL_RUNNING_SO_REPAINT;
   }

   public void paint (Pen pp)
   {
      if (_lines == null)
         return;

      pp.setColor (Color.BLACK);
      if (!_flooded)
      {
         pp.fillRect (0, 0, ResLibCompat.SCREEN_WIDTH, ResLibCompat.SCREEN_HEIGHT);
         _flooded = true;
      }
      else
      {
         for (int ii = 0; ii < NUM_LINES; ++ii)
         {
            int clr = _lines[ii].clr * _lineBright / LINEBRIGHT_MAX;
            pp.drawLine (0, (int)_lines[ii].iyOld, ResLibCompat.SCREEN_WIDTH, (int)_lines[ii].iyOld);
         }
      }

      for (int ii = 0; ii < NUM_LINES; ++ii)
      {
         int clr = _lines[ii].clr * _lineBright / LINEBRIGHT_MAX;
         pp.setColor (clr, clr, clr);
         pp.drawLine (0, (int)_lines[ii].iy, ResLibCompat.SCREEN_WIDTH, (int)_lines[ii].iy);
      }

      int off = 0;
      pp.setFont (_font);

      if (_cchFadeIn != 0 || _cchSolidIn != 0)
      {
         if (_cchSolidIn != 0)
         {
            pp.setColor (_textBright, _textBright, _textBright);
            drawText (pp, off, _cchSolidIn, DONT_ERASE_BEHIND);
            off += _cchSolidIn;
         }
         if (_cchFadeIn != 0)
         {
            for (int ii = 0; ii < _cchFadeIn; ++ii)
            {
               int clr;
               if (_cchSolidIn == 0)
                  clr = _textBright * (_cchFadeIn - ii) / (CCH_FADE+1);
               else
                  clr = _textBright * (CCH_FADE - ii) / (CCH_FADE+1);
               pp.setColor (clr, clr, clr);
               drawText (pp, off, 1, DONT_ERASE_BEHIND);
               off += 1;
            }
         }
      }
      else // (_cchSkip != 0)
      {
         off += _cchSkipLast;

         if (_cchSkipLast < _cchSkip)
         {
            pp.setColor (Color.BLACK);
            int cchDraw = _cchSkip - _cchSkipLast;
            drawText (pp, off, cchDraw, ERASE_BEHIND);
            off += cchDraw;
            _cchSkipLast = _cchSkip;
         }

         if (_cchFadeOut != 0)
         {
            for (int ii = 0; ii < _cchFadeOut; ++ii)
            {
               int clr;
               if (_cchSolidOut == 0)
                  clr = _textBright * (ii+1) / (CCH_FADE+1);
               else
                  clr = _textBright * (ii + CCH_FADE - _cchFadeOut + 1) / (CCH_FADE+1);
               pp.setColor (clr, clr, clr);
               drawText (pp, off, 1, ERASE_BEHIND);
               off += 1;
            }
         }
         if (_cchSolidOut != 0)
         {
            pp.setColor (_textBright, _textBright, _textBright);
            drawText (pp, off, _cchSolidOut, DONT_ERASE_BEHIND);
         }
      }
   }

   protected void drawText (Pen pp, int ich, int cch, int erase)
   {
      int yy = (ResLibCompat.SCREEN_HEIGHT - _cyText)/2 + _yyText;
      for ( ; cch > 0; ++ich, --cch) {
         if (erase == ERASE_BEHIND) {
            Pen p2 = pp.clonePen();
            p2.setColor (Color.BLACK);
            p2.fillRect (_ixLetter[ich], yy - _yyText, _ixLetter[ich] + _cxLetterMax, yy - _yyText + _cyText);
         }
         pp.drawText (_ixLetter[ich], yy, BANNER_TEXT, ich, 1);
      }
   }

   static protected final int DONT_ERASE_BEHIND = 0;
   static protected final int ERASE_BEHIND = 1;

   static protected final int NUM_LINES = 10;
   static protected final int LINEBRIGHT_MAX = 100;

   static protected final int FRAME_FADEIN_START = 10;
   static protected final int FRAME_FADEIN_DONE = 60;
   static protected final int FRAME_FADEOUT_START = 110;
   static protected final int FRAME_FADEOUT_TEXTDONE = 125;
   static protected final int FRAME_FADEOUT_DONE = 128;
   static protected final int FRAME_COLORCHANGE_START = 70;
   static protected final int FRAME_COLORCHANGE_DONE = 90;
   static protected final int FRAME_SOUND_START = 5;


   protected Window _win;
   protected Sequence _seq;
   protected ResLoaderLine[] _lines;
   protected int _lineBright;
   protected int _textBright;
   protected int _frame;
   protected Font _font;
   protected int _cxText;
   protected int _cyText;
   protected int _yyText;
   protected int[] _ixLetter;
   protected int _cxLetterMax;
   protected int _cchSolidIn;
   protected int _cchFadeIn;
   protected int _cchSkip;
   protected int _cchFadeOut;
   protected int _cchSolidOut;
   protected int _cchSkipLast;
   protected boolean _flooded;
   protected boolean _sounded;
   protected boolean _fadedSound;
   protected boolean _switchedAway;
   protected boolean _collected;

   static final int MAX_VOLUME = 35;

   static protected Random _randgen = new Random();
   static protected final String BANNER_TEXT = "RANDOM SOFTWARE";
   static protected final int BANNER_LENGTH = BANNER_TEXT.length();
   static protected final int COLOR_DIM = 96;
   static protected final int CCH_FADE = 6;
}

