package com.randomsoftware.common.terrain;
import com.randomsoftware.common.animation.*;
import com.randomsoftware.common.meshlib.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class GroundMeshGrid
{
   public GroundMeshGrid (GroundMesh gm)
   {
      _gm = gm;
      _config = gm._config;
      _xLeft = 0;
      _xRight = 0;
      _zTop = 0;
      _zBottom = 0;
      _xMeshCenter = 0;
      _zMeshCenter = 0;
      _majorQuads = new GroundQuad[_config.MAJOR_GROUP_COUNT][_config.MAJOR_GROUP_COUNT];
      _lists = new GroundMeshListManager(this, _config.MAX_VERTEX_PER_GRID_SIDE);
   }

   public void destroy()
   {
      for (int zz = 0; zz < _config.MAJOR_GROUP_COUNT; ++zz)
      {
         for (int xx = 0; xx < _config.MAJOR_GROUP_COUNT; ++xx)
         {
            if (_majorQuads[zz][xx] != null)
            {
               _lists.releaseQuad (_majorQuads[zz][xx]);
               _majorQuads[zz][xx] = null;
            }
         }
      }

      _lists.destroy();
   }

   public void reset (long xMeshCenter, long zMeshCenter, long xLeft, long zTop, long xRight, long zBottom)
   {
      for (int zz = 0; zz < _config.MAJOR_GROUP_COUNT; ++zz)
      {
         for (int xx = 0; xx < _config.MAJOR_GROUP_COUNT; ++xx)
         {
            if (_majorQuads[zz][xx] != null)
            {
               _lists.releaseQuad (_majorQuads[zz][xx]);
               _majorQuads[zz][xx] = null;
            }
         }
      }

      _xLeft = xLeft;
      _xRight = xRight;
      _zTop = zTop;
      _zBottom = zBottom;
      _xMeshCenter = xMeshCenter;
      _zMeshCenter = zMeshCenter;
   }

   public boolean update (long xTarget, long zTarget)
   {
      boolean updated = false;

      for (int zz = 0; zz < _config.MAJOR_GROUP_COUNT; ++zz)
      {
         for (int xx = 0; xx < _config.MAJOR_GROUP_COUNT; ++xx)
         {
            if (_majorQuads[zz][xx] == null)
            {
               _majorQuads[zz][xx] = _lists.allocQuad();

               _majorQuads[zz][xx].assign (
                           _config.SCALE_MULT, xx * _config.SCALE_MULT, zz * _config.SCALE_MULT,
                           xTarget, zTarget,
                           _xLeft + xx * _config.SCALE, _xLeft + (xx+1) * _config.SCALE,
                           _zTop + zz * _config.SCALE, _zTop + (zz+1) * _config.SCALE,
                           0, _config.TEXTURE_WIDTH-1, 0, _config.TEXTURE_HEIGHT-1);

               updated = true;
            }

            if (_majorQuads[zz][xx].buildQuads (xTarget, zTarget))
            {
               updated = true;
            }
         }
      }

      if (updated)
      {
         for (int zz = 0; zz < _config.MAJOR_GROUP_COUNT; ++zz)
         {
            for (int xx = 0; xx < _config.MAJOR_GROUP_COUNT; ++xx)
            {
               _majorQuads[zz][xx].buildFaces();
            }
         }
      }

      return updated;
   }

   GroundMesh _gm;
   TerrainConfig _config;
   long _xLeft;
   long _xRight;
   long _zTop;
   long _zBottom;
   long _xMeshCenter;
   long _zMeshCenter;
   GroundQuad[][] _majorQuads;
   GroundMeshListManager _lists;
}

