package com.randomsoftware.common.terrain;
import com.randomsoftware.common.animation.*;
import com.randomsoftware.common.meshlib.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class GroundMeshListManager
{
   public GroundMeshListManager (GroundMeshGrid grid, int mapsize)
   {
      _grid = grid;
      _firstFreeQuad = null;
      _firstFreeFace = null;
      _firstFreeVertex = null;
      _vertexMap = new VertexMap(mapsize);
   }

   public void destroy()
   {
      GroundQuad gq;
      while ((gq = popFreeQuad()) != null)
         gq.destroy();

      GroundFace gf;
      while ((gf = popFreeFace()) != null)
         gf.destroy();

      GroundVertex gv;
      while ((gv = popFreeVertex()) != null)
         ;
   }


   public GroundQuad allocQuad()
   {
      GroundQuad gq;
      if ((gq = popFreeQuad()) == null)
         gq = new GroundQuad (_grid);
      return gq;
   }

   public void releaseQuad (GroundQuad gq)
   {
      gq.destroy();
      pushFreeQuad (gq);
   }


   public GroundFace allocFace (GroundVertex gv1, int u1, int v1, GroundVertex gv2, int u2, int v2, GroundVertex gv3, int u3, int v3, Texture tx)
   {
      GroundFace gf;
      if ((gf = popFreeFace()) == null)
         gf = new GroundFace (_grid, gv1, u1, v1, gv2, u2, v2, gv3, u3, v3, tx);
      else
         gf.assign (gv1, u1, v1, gv2, u2, v2, gv3, u3, v3, tx);
      gf.connect();
      return gf;
   }

   public void releaseFace (GroundFace gf)
   {
      gf.disconnect();
      gf.destroy();
      pushFreeFace (gf);
   }


   public GroundVertex findVertex (int ix, int iz)
   {
      return _vertexMap.get (ix, iz);
   }

   public GroundVertex allocVertex (long dx, long dy, long dz)
   {
      GroundVertex gv;
      if ((gv = popFreeVertex()) == null)
         gv = new GroundVertex (_grid, dx, dy, dz);
      else
         gv.assign (dx, dy, dz);
      gv._ix = NOT_IN_VERTEX_MAP;
      gv._iz = NOT_IN_VERTEX_MAP;
      gv.connect();
      return gv;
   }

   public GroundVertex allocVertex (int ix, int iz, long dx, long dy, long dz)
   {
      GroundVertex gv;
      if ((gv = _vertexMap.get (ix, iz)) == null)
      {
         if ((gv = popFreeVertex()) == null)
            gv = new GroundVertex (_grid, dx, dy, dz);
         else
            gv.assign (dx, dy, dz);
         gv._ix = ix;
         gv._iz = iz;
         _vertexMap.set (ix, iz, gv);
      }
      gv.connect();
      return gv;
   }

   public void releaseVertex (GroundVertex gv)
   {
      if (gv.disconnect() == GroundVertex.NO_LONGER_NEEDED)
      {
         if (gv._ix != NOT_IN_VERTEX_MAP)
            _vertexMap.clear (gv._ix, gv._iz);
         pushFreeVertex (gv);
      }
   }


   GroundQuad popFreeQuad()
   {
      if (_firstFreeQuad == null)
         return null;
      GroundQuad gq = _firstFreeQuad;
      _firstFreeQuad = gq._nextInFreeList;
      gq._nextInFreeList = null;
      return gq;
   }

   void pushFreeQuad (GroundQuad gq)
   {
      gq._nextInFreeList = _firstFreeQuad;
      _firstFreeQuad = gq;
   }


   GroundFace popFreeFace()
   {
      if (_firstFreeFace == null)
         return null;
      GroundFace gf = _firstFreeFace;
      _firstFreeFace = gf._nextInFreeList;
      gf._nextInFreeList = null;
      return gf;
   }

   void pushFreeFace (GroundFace gf)
   {
      gf._nextInFreeList = _firstFreeFace;
      _firstFreeFace = gf;
   }


   GroundVertex popFreeVertex()
   {
      if (_firstFreeVertex == null)
         return null;
      GroundVertex gv = _firstFreeVertex;
      _firstFreeVertex = gv._nextInFreeList;
      gv._nextInFreeList = null;
      return gv;
   }

   void pushFreeVertex (GroundVertex gv)
   {
      gv._nextInFreeList = _firstFreeVertex;
      _firstFreeVertex = gv;
   }

   static int NOT_IN_VERTEX_MAP = 0x3FFFFFFF;
   GroundMeshGrid _grid;
   VertexMap _vertexMap;
   GroundQuad _firstFreeQuad;
   GroundFace _firstFreeFace;
   GroundVertex _firstFreeVertex;
}

