package com.randomsoftware.common.terrain;
import com.randomsoftware.common.animation.*;
import com.randomsoftware.common.meshlib.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class GroundQuad
{
   GroundQuad (GroundMeshGrid grid)
   {
      _grid = grid;
      _faces = new GroundFace[_grid._config.MAX_FACE_PER_QUAD];
      _subQuads = new GroundQuad[4];
      _cornerVertices = new GroundVertex[4];
      _needFaces = true;
      _childNeedsFaces = true;
      _numFacesNow = 0;
      _numSubQuadsNow = 0;
   }

   public void assign (int size, int iLeft, int iTop, long xTarget, long zTarget, long xLeft, long xRight, long zTop, long zBottom, int u1, int u2, int v1, int v2)
   {
      _iLeft = iLeft;
      _iTop = iTop;
      _size = size;
      _xTarget = xTarget;
      _zTarget = zTarget;
      _xLeft = xLeft;
      _xRight = xRight;
      _zTop = zTop;
      _zBottom = zBottom;
      _u1 = u1;
      _u2 = u2;
      _v1 = v1;
      _v2 = v2;
      _needFaces = true;
      _childNeedsFaces = true;

      long xo = _grid._xMeshCenter;
      long zo = _grid._zMeshCenter;

      _cornerVertices[0] = _grid._lists.allocVertex (iLeft,      iTop,      xLeft-xo,  _grid._gm._heightMap.getTerrainHeight(xLeft,zTop),     zTop-zo);
      _cornerVertices[1] = _grid._lists.allocVertex (iLeft+size, iTop,      xRight-xo, _grid._gm._heightMap.getTerrainHeight(xRight,zTop),    zTop-zo);
      _cornerVertices[2] = _grid._lists.allocVertex (iLeft,      iTop+size, xLeft-xo,  _grid._gm._heightMap.getTerrainHeight(xLeft,zBottom),  zBottom-zo);
      _cornerVertices[3] = _grid._lists.allocVertex (iLeft+size, iTop+size, xRight-xo, _grid._gm._heightMap.getTerrainHeight(xRight,zBottom), zBottom-zo);
   }

   void destroy()
   {
      for (int ii = 0; ii < 4; ++ii)
      {
         if (_subQuads[ii] != null)
         {
            _grid._lists.releaseQuad (_subQuads[ii]);
            _subQuads[ii] = null;
         }
      }
      _numSubQuadsNow = 0;

      for (int ii = 0; ii < _numFacesNow; ++ii)
      {
         _grid._lists.releaseFace (_faces[ii]);
         _faces[ii] = null;
      }
      _numFacesNow = 0;

      if (_centerVertexA != null)
      {
         _grid._lists.releaseVertex (_centerVertexA);
         _centerVertexA = null;
      }

      if (_centerVertexB != null)
      {
         _grid._lists.releaseVertex (_centerVertexB);
         _centerVertexB = null;
      }

      for (int ii = 0; ii < 4; ++ii)
      {
         if (_cornerVertices[ii] != null)
         {
            _grid._lists.releaseVertex (_cornerVertices[ii]);
            _cornerVertices[ii] = null;
         }
      }
   }

   public boolean buildQuads (long xTarget, long zTarget)
   {
      _xTarget = xTarget;
      _zTarget = zTarget;
      _numSubQuadsNow = 0;

      long xCenter = (_xLeft+_xRight)/2;
      long zCenter = (_zTop+_zBottom)/2;

      if (_size != 1)
      {
         for (int ii = 0; ii < 4; ++ii)
         {
            buildOneQuad (ii);

            if (_subQuads[ii] != null)
               ++_numSubQuadsNow;
         }
      }

      return _needFaces || _childNeedsFaces;
   }

   public void buildOneQuad (int iQuad)
   {
      long xSubLeft = ((iQuad & 1) == 0) ? _xLeft : (_xLeft+_xRight)/2;
      long xSubRight = xSubLeft + (_xRight-_xLeft)/2;
      long zSubTop = ((iQuad & 2) == 0) ? _zTop : (_zTop+_zBottom)/2;
      long zSubBottom = zSubTop + (_zBottom-_zTop)/2;
      long xSubCenter = (xSubLeft + xSubRight) / 2;
      long zSubCenter = (zSubTop + zSubBottom) / 2;

      boolean wantSub = false;
      if (_size > 1)
      {
         if (FixedMath.roughDistance(xSubCenter - _xTarget, zSubCenter - _zTarget) < (_size * _grid._config.PRECISION))
            wantSub = true;
      }

      if ((wantSub) && (_subQuads[iQuad] == null))
      {
         int iSubLeft = ((iQuad & 1) == 0) ? _iLeft : (_iLeft+_size/2);
         int iSubTop = ((iQuad & 2) == 0) ? _iTop : (_iTop+_size/2);

         int uSub1 = ((iQuad & 1) == 0) ? _u1 : ((_u1+_u2)/2);
         int uSub2 = ((iQuad & 1) == 0) ? ((_u1+_u2)/2) : _u2;
         int vSub1 = ((iQuad & 2) == 0) ? _v1 : ((_v1+_v2)/2);
         int vSub2 = ((iQuad & 2) == 0) ? ((_v1+_v2)/2) : _v2;

         _subQuads[iQuad] = _grid._lists.allocQuad();
         _subQuads[iQuad].assign (_size/2,
                                 iSubLeft,
                                 iSubTop,
                                 _xTarget,
                                 _zTarget,
                                 xSubLeft,
                                 xSubRight,
                                 zSubTop,
                                 zSubBottom,
                                 uSub1,
                                 uSub2,
                                 vSub1,
                                 vSub2);

         _needFaces = true;
      }
      else if ((!wantSub) && (_subQuads[iQuad] != null))
      {
         _grid._lists.releaseQuad (_subQuads[iQuad]);
         _subQuads[iQuad] = null;
         _needFaces = true;
      }

      if (_subQuads[iQuad] != null)
      {
         if (_subQuads[iQuad].buildQuads (_xTarget, _zTarget))
            _needFaces = _childNeedsFaces = true;
      }
   }

   public void buildFaces()
   {
      int _iRight = _iLeft + _size;
      int _iBottom = _iTop + _size;
      Texture tx = _grid._gm._terrainMap.getTerrainTexture ((_xLeft+_xRight)/2, (_zTop+_zBottom)/2);


      for (int ii = 0; ii < _numFacesNow; ++ii)
      {
         _grid._lists.releaseFace (_faces[ii]);
         _faces[ii] = null;
      }
      _numFacesNow = 0;

      if (_centerVertexA != null)
      {
         _grid._lists.releaseVertex (_centerVertexA);
         _centerVertexA = null;
      }

      if (_centerVertexB != null)
      {
         _grid._lists.releaseVertex (_centerVertexB);
         _centerVertexB = null;
      }


      int ixMid = _iLeft+_size/2;
      int izMid = _iTop+_size/2;
      int uMid = (_u1+_u2)/2;
      int vMid = (_v1+_v2)/2;
      boolean isolatedSquares = false;

      if ((_size == 1) || (_numSubQuadsNow == 0) && (!hasSubdividedNeighbors()))
      {
         _faces[ _numFacesNow++ ] = _grid._lists.allocFace (
                                          _grid._lists.findVertex (_iLeft, _iBottom), _u1, _v2,
                                          _grid._lists.findVertex (_iRight, _iTop), _u2, _v1,
                                          _grid._lists.findVertex (_iLeft, _iTop), _u1, _v1,
                                          tx);

         _faces[ _numFacesNow++ ] = _grid._lists.allocFace (
                                          _grid._lists.findVertex (_iLeft, _iBottom), _u1, _v2,
                                          _grid._lists.findVertex (_iRight, _iBottom), _u2, _v2,
                                          _grid._lists.findVertex (_iRight, _iTop), _u2, _v1,
                                          tx);
      }
      else if (_numSubQuadsNow == 0)
      {
         int uCenter = (_u1+_u2)/2;
         int vCenter = (_v1+_v2)/2;
         _centerVertexA = allocCenterVertex (2, 2);
         fanSquare (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, _iRight, _u2, _iTop, _v1, _iBottom, _v2);
      }
      else if (_numSubQuadsNow == 1)
      {
         if (_subQuads[0] != null)
         {
            int uCenter = _u1+(_u2-_u1)*3/4;
            int vCenter = _v1+(_v2-_v1)*3/4;
            _centerVertexA = allocCenterVertex (3, 3);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iTop, _v1, _iRight, ixMid, -1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, ixMid, uMid, _iTop, izMid, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, izMid, vMid, ixMid, _iLeft, -1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, izMid, _iBottom, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iBottom, _v2, _iLeft, _iRight, 1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iRight, _u2, _iBottom, _iTop, -1);
         }
         else if (_subQuads[1] != null)
         {
            int uCenter = _u1+(_u2-_u1)*1/4;
            int vCenter = _v1+(_v2-_v1)*3/4;
            _centerVertexA = allocCenterVertex (1, 3);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iTop, _v1, ixMid, _iLeft, -1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, _iTop, _iBottom, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iBottom, _v2, _iLeft, _iRight, 1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iRight, _u2, _iBottom, izMid, -1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, izMid, vMid, _iRight, ixMid, -1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, ixMid, uMid, izMid, _iTop, -1);
         }
         else if (_subQuads[2] != null)
         {
            int uCenter = _u1+(_u2-_u1)*3/4;
            int vCenter = _v1+(_v2-_v1)*1/4;
            _centerVertexA = allocCenterVertex (3, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iTop, _v1, _iRight, _iLeft, -1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, _iTop, izMid, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, izMid, vMid, _iLeft, ixMid, 1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, ixMid, uMid, izMid, _iBottom, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iBottom, _v2, ixMid, _iRight, 1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iRight, _u2, _iBottom, _iTop, -1);
         }
         else // (_subQuads[3] != null)
         {
            int uCenter = _u1+(_u2-_u1)*1/4;
            int vCenter = _v1+(_v2-_v1)*1/4;
            _centerVertexA = allocCenterVertex (1, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iTop, _v1, _iRight, _iLeft, -1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, _iTop, _iBottom, 1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, _iBottom, _v2, _iLeft, ixMid, 1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, ixMid, uMid, _iBottom, izMid, -1);
            fanHorizontalEdge (_centerVertexA, uCenter, vCenter, tx, izMid, vMid, ixMid, _iRight, 1);
            fanVerticalEdge (_centerVertexA, uCenter, vCenter, tx, _iRight, _u2, izMid, _iTop, -1);
         }
      }
      else if (_numSubQuadsNow == 2)
      {
         if ((_subQuads[0] == null) && (_subQuads[1] == null))
         {
            int uCenter = (_u1+_u2)*2/4;
            int vCenter = (_v1+_v2)*1/4;
            _centerVertexA = allocCenterVertex (2, 1);
            fanSquare (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, _iRight, _u2, _iTop, _v1, izMid, vMid);
         }
         else if ((_subQuads[1] == null) && (_subQuads[3] == null))
         {
            int uCenter = (_u1+_u2)*3/4;
            int vCenter = (_v1+_v2)*2/4;
            _centerVertexA = allocCenterVertex (3, 2);
            fanSquare (_centerVertexA, uCenter, vCenter, tx, ixMid, uMid, _iRight, _u2, _iTop, _v1, _iBottom, _v2);
         }
         else if ((_subQuads[0] == null) && (_subQuads[2] == null))
         {
            int uCenter = (_u1+_u2)*1/4;
            int vCenter = (_v1+_v2)*2/4;
            _centerVertexA = allocCenterVertex (1, 2);
            fanSquare (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, ixMid, uMid, _iTop, _v1, _iBottom, _v2);
         }
         else if ((_subQuads[2] == null) && (_subQuads[3] == null))
         {
            int uCenter = (_u1+_u2)*2/4;
            int vCenter = (_v1+_v2)*3/4;
            _centerVertexA = allocCenterVertex (2, 3);
            fanSquare (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, _iRight, _u2, izMid, vMid, _iBottom, _v2);
         }
         else
         {
            isolatedSquares = true;
         }
      }
      else if (_numSubQuadsNow == 3)
      {
         isolatedSquares = true;
      }

      if (isolatedSquares)
      {
         if (_subQuads[0] == null)
         {
            int uCenter = (_u1+_u2)*1/4;
            int vCenter = (_v1+_v2)*1/4;
            _centerVertexA = allocCenterVertex (1, 1);
            fanSquare (_centerVertexA, uCenter, vCenter, tx, _iLeft, _u1, ixMid, uMid, _iTop, _v1, izMid, vMid);
         }
         if (_subQuads[1] == null)
         {
            int uCenter = (_u1+_u2)*3/4;
            int vCenter = (_v1+_v2)*1/4;
            GroundVertex gv = allocCenterVertex (3, 1);
            if (_centerVertexA == null) _centerVertexA = gv; else _centerVertexB = gv;
            fanSquare (gv, uCenter, vCenter, tx, ixMid, uMid, _iRight, _u2, _iTop, _v1, izMid, vMid);
         }
         if (_subQuads[2] == null)
         {
            int uCenter = (_u1+_u2)*1/4;
            int vCenter = (_v1+_v2)*3/4;
            GroundVertex gv = allocCenterVertex (1, 3);
            if (_centerVertexA == null) _centerVertexA = gv; else _centerVertexB = gv;
            fanSquare (gv, uCenter, vCenter, tx, _iLeft, _u1, ixMid, uMid, izMid, vMid, _iBottom, _v2);
         }
         if (_subQuads[3] == null)
         {
            int uCenter = (_u1+_u2)*3/4;
            int vCenter = (_v1+_v2)*3/4;
            GroundVertex gv = allocCenterVertex (3, 3);
            if (_centerVertexA == null) _centerVertexA = gv; else _centerVertexB = gv;
            fanSquare (gv, uCenter, vCenter, tx, ixMid, uMid, _iRight, _u2, izMid, vMid, _iBottom, _v2);
         }
      }

      for (int ii = 0; ii < 4; ++ii)
      {
         if (_subQuads[ii] != null)
            _subQuads[ii].buildFaces();
      }

      _childNeedsFaces = false;
      _needFaces = false;
   }

   boolean hasSubdividedNeighbors()
   {
      return (_grid._lists.findVertex (_iLeft, _iTop+_size/2) != null ||
              _grid._lists.findVertex (_iLeft+_size, _iTop+_size/2) != null ||
              _grid._lists.findVertex (_iLeft+_size/2, _iTop) != null ||
              _grid._lists.findVertex (_iLeft+_size/2, _iTop+_size) != null);
   }

   GroundVertex allocCenterVertex (int qx, int qz)
   {
      long xx = _xLeft + (_xRight - _xLeft) * qx / 4;
      long zz = _zTop + (_zBottom - _zTop) * qz / 4;
      return _grid._lists.allocVertex (xx - _grid._xMeshCenter,
                                       _grid._gm._heightMap.getTerrainHeight(xx,zz),
                                       zz - _grid._zMeshCenter);
   }

   void fanHorizontalEdge (GroundVertex centerVertex, int uCenter, int vCenter, Texture tx, int iy, int vy, int ix1, int ix2, int dd)
   {
      GroundVertex prevVertex = null;
      int prevU = 0;

      for (int ix = ix1; ; ix += dd)
      {
         GroundVertex nextVertex;
         if ((nextVertex = _grid._lists.findVertex (ix, iy)) != null)
         {
            int nextU = _u1 + (_u2-_u1) * (ix - _iLeft) / _size;

            if (prevVertex != null)
            {
               _faces[ _numFacesNow++ ] = _grid._lists.allocFace (
                                                centerVertex, uCenter, vCenter,
                                                prevVertex, prevU, vy,
                                                nextVertex, nextU, vy,
                                                tx);
            }

            prevVertex = nextVertex;
            prevU = nextU;
         }
         if (ix == ix2)
            break;
      }
   }

   void fanVerticalEdge (GroundVertex centerVertex, int uCenter, int vCenter, Texture tx, int ix, int ux, int iy1, int iy2, int dd)
   {
      GroundVertex prevVertex = null;
      int prevV = 0;

      for (int iy = iy1; ; iy += dd)
      {
         GroundVertex nextVertex;
         if ((nextVertex = _grid._lists.findVertex (ix, iy)) != null)
         {
            int nextV = _v1 + (_v2-_v1) * (iy - _iTop) / _size;

            if (prevVertex != null)
            {
               _faces[ _numFacesNow++ ] = _grid._lists.allocFace (
                                                centerVertex, uCenter, vCenter,
                                                prevVertex, ux, prevV,
                                                nextVertex, ux, nextV,
                                                tx);
            }

            prevVertex = nextVertex;
            prevV = nextV;
         }
         if (iy == iy2)
            break;
      }
   }

   void fanSquare (GroundVertex centerVertex, int uCenter, int vCenter, Texture tx, int ix1, int u1, int ix2, int u2, int iy1, int v1, int iy2, int v2)
   {
      fanHorizontalEdge (centerVertex, uCenter, vCenter, tx, iy1, v1, ix2, ix1, -1);
      fanVerticalEdge (centerVertex, uCenter, vCenter, tx, ix1, u2, iy1, iy2, 1);
      fanHorizontalEdge (centerVertex, uCenter, vCenter, tx, iy2, v2, ix1, ix2, 1);
      fanVerticalEdge (centerVertex, uCenter, vCenter, tx, ix2, u1, iy2, iy1, -1);
   }

   int _iLeft;
   int _iTop;
   int _size;
   long _xTarget;
   long _zTarget;
   long _xLeft;
   long _xRight;
   long _zTop;
   long _zBottom;
   int _u1;
   int _u2;
   int _v1;
   int _v2;

   int _numFacesNow;
   int _numSubQuadsNow;
   GroundMeshGrid _grid;
   GroundFace[] _faces;
   GroundQuad[] _subQuads;
   GroundVertex _centerVertexA;
   GroundVertex _centerVertexB;
   GroundVertex[] _cornerVertices;
   GroundQuad _nextInFreeList;
   boolean _needFaces;
   boolean _childNeedsFaces;
}

