package com.randomsoftware.common.terrain;
import com.randomsoftware.common.animation.*;
import com.randomsoftware.common.meshlib.*;

import java.util.*;
import danger.app.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class TerrainConfig
{
   // The ground is, at biggest, composed of a square 2D grid of "groups".
   // The major_group_count indicates how many groups along each side
   // compose that ground; so 4x4, 5x5 and so on.

   int MAJOR_GROUP_COUNT = 4;

   // Each major group represents how many worldspace units?  A scale of
   // one is the minimum size, which is never subdivided.  This setting
   // must be a power of two; the bigger it is, the further off to the
   // horizon we can reach.

   int SCALE_MULT = 16;

   // Texture coordinates are calculated based on the specified texture
   // size.  All ground textures that you provide must be this size.

   int TEXTURE_WIDTH = 64;
   int TEXTURE_HEIGHT = 64;

   // As the ground viewer moves within the world space, we don't want
   // to recalculate the ground on every frame--after all, most of the
   // time the previously-calculated ground mesh is still adequate for
   // drawing from the new point of view.  This constant indicates how
   // far we right-shift the previous and current coordinates before
   // testing to see if they're different; higher values mean we update
   // the mesh less frequently.

   long SHIFT_UPDATE_TEST = FixedMath.SHIFT-1;

   // The precision constant determines how close a group must be to
   // the viewer's target region in order to be subdivided.  Its units
   // are fairly arbitrary; higher values indicate a more precise
   // surface even at a distance, while lower values give better
   // performance and an ugly, spiky terrain.

   long PRECISION = FixedMath.PREC*3;


   // That's it.  All other constants are derived automatically from
   // the ones you configured above.

   void calculateDerivedValues()
   {
      MAX_VERTEX_PER_GRID_SIDE = SCALE_MULT * MAJOR_GROUP_COUNT + 1;
      MAX_VERTEX_PER_QUAD_SIDE = SCALE_MULT + 1;
      MAX_FACE_PER_QUAD_SIDE = SCALE_MULT;
      MAX_FACE_PER_QUAD = MAX_FACE_PER_QUAD_SIDE * 4;
      SCALE = SCALE_MULT * FixedMath.PREC;
   }

   long SCALE;
   int MAX_VERTEX_PER_GRID_SIDE;
   int MAX_VERTEX_PER_QUAD_SIDE;
   int MAX_FACE_PER_QUAD_SIDE;
   int MAX_FACE_PER_QUAD;
}

