package com.randomsoftware.common.webscore;

import java.util.*;
import danger.app.*;
import danger.net.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

class LocalScoreListListener extends Listener
{
   public LocalScoreListListener (LocalScoreList list)
   {
      _list = list;
   }

   public boolean receiveEvent (Event e)
   {
      if (WebScoreCompat.Event_getType(e) == Event.EVENT_DATASTORE_RESTORED)
         _list.reload();
      return false;
   }

   LocalScoreList _list;
};

public class LocalScoreList
{
   public LocalScoreList (int maxCount)
   {
      _maxCount = maxCount;
      _listener = new LocalScoreListListener (this);
      _ds = DataStore.createDataStore ("LocalScore", true);
      _ds.registerListenerForAutoSyncDone (_listener);
      _ds.setAutoSyncNotifyee (_listener);
      reload();
   }

   public void reload()
   {
      try {
         _scores = new Vector();
         for (int ii = 0; ii < _ds.getRecordCount(); ++ii)
            sortedInsert (new ScoreRecord (ii, _ds.getRecordData(ii)));
         // for (int ii = 0; ii < 20; ++ii)
         //   sortedInsert (new ScoreRecord ((ii+1)*1000));
         pruneExcessRecords();
      } catch (Exception e) {
         _scores = new Vector();
         _ds.removeAllRecords();
      }
   }

   public void add (ScoreRecord sr)
   {
      if (scoreBelongsInTable (sr))
      {
         if (size() < _maxCount)
         {
            sr.recnum = _ds.addRecord (sr.toData());
         }
         else
         {
            ScoreRecord kill = elementAt (size()-1);
            _scores.removeElementAt (size()-1);

            sr.recnum = kill.recnum;
            _ds.setRecordData (sr.recnum, sr.toData());
         }

         sortedInsert (sr);
      }
   }

   public int size()
   {
      return _scores.size();
   }

   public ScoreRecord elementAt (int ii)
   {
      return (ScoreRecord)(_scores.elementAt(ii));
   }


   boolean scoreBelongsInTable (ScoreRecord sr)
   {
      if (size() < _maxCount)
         return true;
      else
         return (sr.score > elementAt(size()-1).score);
   }

   void sortedInsert (ScoreRecord srNew)
   {
      for (int ii = 0; ii < size(); ++ii)
      {
         if (srNew.score > elementAt(ii).score)
         {
            _scores.insertElementAt (srNew, ii);
            return;
         }
      }
      _scores.insertElementAt (srNew, size());
   }

   void pruneExcessRecords()
   {
      while (size() > _maxCount)
      {
         ScoreRecord kill = elementAt (size()-1);
         _scores.removeElementAt (size()-1);
         _ds.removeRecord (kill.recnum);

         for (int ii = 0; ii < size(); ++ii)
         {
            if (elementAt(ii).recnum > kill.recnum)
               elementAt(ii).recnum --;
         }
      }
   }

   LocalScoreListListener _listener;
   DataStore _ds;
   Vector _scores;
   int _maxCount;
}

