package com.randomsoftware.common.xmllib;

import java.util.*;
import danger.app.*;
import danger.net.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class XmlParser
{
   public static XmlTag read (String input) throws XmlFormatException
   {
      XmlParser xpp = new XmlParser (input);
      xpp.crack();
      return xpp._topTag;
   }

   XmlParser (String input) throws XmlFormatException
   {
      _ts = new XmlTextStream (input);
      _topTag = new XmlTag ("internal.document");
      _parseStack = new Stack();
   }

   void crack() throws XmlFormatException
   {
      _parseStack.push (_topTag);

      XmlToken tok = new XmlToken();
      while ((tok.read (_ts, XmlToken.POPF_OUTSIDE_TAGDEF)) != XmlToken.EOF)
      {
         if ((tok.type == XmlToken.TAG_OPEN) || (tok.type == XmlToken.TAG_OPEN_COMMAND) || (tok.type == XmlToken.TAG_OPEN_PROCESS) || (tok.type == XmlToken.TAG_OPEN_SSI))
            crackTagOpen(tok);
         else if (tok.type == XmlToken.TAG_OPEN_CDATA)
            crackCData (tok);
         else if (tok.type == XmlToken.TAG_OPEN_END)
            crackTagClose (tok);
         else if (tok.type == XmlToken.TEXT)
            crackText(tok.text);
         else
            throw new XmlFormatException ("unexpected text '"+tok.text+"' found in xml stream");
      }
   }

   void crackTagOpen (XmlToken tok) throws XmlFormatException
   {
      int tokOpenType = tok.type;

      // We've popped a token that marks the beginning of a tag definition.
      // Grab its section name--that should be the next word.

      if (tok.read (_ts, XmlToken.POPF_INSIDE_TAGDEF) != XmlToken.TEXT)
         throw new XmlFormatException ("expected tag name after tag open, not '"+tok.text+"'");

      // This tag represents a new section; we'll need a place to store it.
      // Make sure the new section goes on the bottom of the hierarchy.

      XmlTag newTag = new XmlTag (tok.text);

      if (tokOpenType == XmlToken.TAG_OPEN_COMMAND)
         newTag.type = XmlTag.COMMAND;
      else if (tokOpenType == XmlToken.TAG_OPEN_PROCESS)
         newTag.type = XmlTag.PROCESS;
      else if (tokOpenType == XmlToken.TAG_OPEN_SSI)
         newTag.type = XmlTag.SSI;

      ((XmlTag)_parseStack.peek()).children.addElement (newTag);

      // Start popping off attributes until we hit a closure mark

      boolean haveLHS = false;
      for (;;)
      {
         // If we don't have an LHS yet, expect one (or a closure mark)

         if (!haveLHS)
         {
            tok.read (_ts, XmlToken.POPF_INSIDE_TAGDEF);
            if (tok.type == XmlToken.TAG_CLOSE_EMPTY)
               break;
            if (tok.type == XmlToken.TAG_CLOSE_PROCESS)
               break;
            if (tok.type == XmlToken.TAG_CLOSE)
               break;
            if (tok.type != XmlToken.TEXT)
               throw new XmlFormatException ("expected tag close or attribute lhs, not '"+tok.text+"'");
         }

         // This phrase represents a attribute LHS. Construct an attribute for it.

         XmlAttribute attr = new XmlAttribute (tok.text, "");
         newTag.attributes.addElement (attr);

         // Expect an equals sign, attribute LHS or closure mark

         tok.read (_ts, XmlToken.POPF_INSIDE_TAGDEF);
         if (tok.type == XmlToken.TAG_CLOSE_EMPTY)
            break;
         if (tok.type == XmlToken.TAG_CLOSE_PROCESS)
            break;
         if (tok.type == XmlToken.TAG_CLOSE)
            break;
         if ((haveLHS = (tok.type == XmlToken.TEXT)) == true)
            continue;
         if (tok.type != XmlToken.EQUALS)
            throw new XmlFormatException ("expected tag close, attribute lhs or equals sign, not '"+tok.text+"'");

         // We got an equals sign; expect an attribute RHS

         tok.read (_ts, XmlToken.POPF_INSIDE_TAGDEF);
         if (tok.type != XmlToken.TEXT)
            throw new XmlFormatException ("expected attribute rhs, not '"+tok.text+"'");

         attr.rhs = tok.text;
      }

      // We've got a closure mark. If it's not />, then this tag needs to be
      // shoved onto our parsing stack. Oh, another exception: process and
      // command and SSI tags don't have children--they just hold data.

      if ((newTag.type == XmlTag.DEFAULT) && (tok.type != XmlToken.TAG_CLOSE_EMPTY))
      {
         _parseStack.push (newTag);
      }
   }

   void crackCData (XmlToken tok) throws XmlFormatException
   {
      // We've just popped off a cdata start marker, so the next "token"
      // we pop off is going to be a big chunk of unprocessable data.

      tok.read (_ts, XmlToken.POPF_INSIDE_CDATA);

      if (tok.type == XmlToken.TAG_CLOSE_CDATA)
         crackText ("");
      else if (tok.type == XmlToken.TEXT)
         crackText (tok.text);
      else // (tok.type == XmlToken.EOF)
         throw new XmlFormatException ("expected cdata close marker before eof");

      // If we DID get some text, expect a closure marker next (or the
      // parser wouldn't have returned)

      if (tok.type != XmlToken.TAG_CLOSE_CDATA)
      {
         if (tok.read (_ts, XmlToken.POPF_INSIDE_CDATA) != XmlToken.TAG_CLOSE_CDATA)
            throw new XmlFormatException ("expected cdata close marker after data but before eof");
      }
   }

   void crackTagClose (XmlToken tok) throws XmlFormatException
   {
      // We've popped the token "</", so we know one of the tags that
      // we're currently studying is about to close. pop its name.

      if (tok.read (_ts, XmlToken.POPF_INSIDE_TAGDEF) != XmlToken.TEXT)
         throw new XmlFormatException ("expected tag name after tag close marker");

      // Check our parsing stack (starting at the bottom) to see if that
      // name represents any of our currently-open tags (it better!)

      boolean found = false;

      for (int ii = 0; ii < _parseStack.size(); ++ii)
      {
         XmlTag tag = (XmlTag)(_parseStack.elementAt(_parseStack.size() - ii - 1));

         if (tag.text.equals (tok.text))
         {
            // Found it! Everything under this tag is now officially closed.
            for (int popped = 0; popped <= ii; ++popped)
               _parseStack.pop();
            found = true;
            break;
         }
      }

      if (!found)
         throw new XmlFormatException ("tried to close missing tag "+tok.text);

      // We should now expect to pop ">" off the input stream.

      if (tok.read (_ts, XmlToken.POPF_INSIDE_TAGDEF) != XmlToken.TAG_CLOSE)
         throw new XmlFormatException ("missing end of tag closure marker");
   }

   void crackText (String text)
   {
      ((XmlTag)_parseStack.peek()).children.addElement (new String (text));
   }

   XmlTextStream _ts;
   XmlTag _topTag;
   Stack _parseStack;
}

