package com.randomsoftware.common.xmllib;

import java.util.*;
import danger.app.*;
import danger.net.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

public class XmlTag
{
   public String text;
   public Vector attributes;
   public Vector children;
   public int type;

   public static final int DEFAULT = 1;
   public static final int COMMAND = 2;
   public static final int PROCESS = 3;
   public static final int SSI = 4;

   public XmlTag (String name)
   {
      text = name;
      type = DEFAULT;
      attributes = new Vector();
      children = new Vector();
   }

   public String getText()
   {
      for (int ii = 0; ii < children.size(); ++ii)
      {
         Object o = children.elementAt(ii);
         if (o instanceof String)
            return (String)o;
      }
      return null;
   }

   public XmlTag getTag (String name)
   {
      for (int ii = 0; ii < children.size(); ++ii)
      {
         Object o = children.elementAt(ii);
         if ((o instanceof XmlTag) && (((XmlTag)o).text.equals(name)))
            return (XmlTag)o;
      }

      for (int ii = 0; ii < children.size(); ++ii)
      {
         Object o = children.elementAt(ii);
         if (o instanceof XmlTag)
         {
            XmlTag found;
            if ((found = ((XmlTag)o).getTag (name)) != null)
               return found;
         }
      }

      return null;
   }

   public String getAttribute (String lhs)
   {
      for (int ii = 0; ii < attributes.size(); ++ii)
      {
         XmlAttribute attr = (XmlAttribute)(attributes.elementAt(ii));
         if (attr.lhs.equals(lhs))
            return attr.rhs;
      }
      return null;
   }

   public String toString()
   {
      return toString("");
   }

   String toString (String indent)
   {
      String tagOpen = "<";
      String tagClose = ">";
      if (type == COMMAND)
         { tagOpen = "<!"; }
      else if (type == PROCESS)
         { tagOpen = "<?"; tagClose = "?>"; }
      else if (type == SSI)
         { tagOpen = "<!--#"; tagClose = "-->"; }
      if ((children.size()==0) && (tagOpen == "<"))
         { tagClose = "/>"; }

      String out = new String(indent + tagOpen + text);

      for (int ii = 0; ii < attributes.size(); ++ii)
      {
         XmlAttribute attr = (XmlAttribute)attributes.elementAt(ii);

         if (attr.rhs.length() == 0)
            out += " " + attr.lhs;
         else if ((attr.rhs.indexOf(' ')==-1) && (attr.rhs.indexOf('<')==-1) && (attr.rhs.indexOf('>')==-1) && (attr.rhs.indexOf('=')==-1))
            out += " " + attr.lhs + "=" + attr.rhs;
         else
            out += " " + attr.lhs + "=\"" + attr.rhs + "\"";
      }

      out += tagClose + "\n";

      if (children.size() != 0)
      {
         for (int ii = 0; ii < children.size(); ++ii)
         {
            Object o = children.elementAt(ii);
            if (o instanceof String)
               out += indent + "   " + (String)o + "\n";
            else
               out += ((XmlTag)o).toString (indent + "   ");
         }
         out += indent + "</" + text + ">\n";
      }

      return out;
   }
}

