package com.randomsoftware.common.xmllib;

import java.util.*;
import danger.app.*;
import danger.net.*;
import danger.ui.*;
import danger.util.*;
import danger.audio.*;

class XmlPredefToken
{
   int type;
   String text;
   int ignoreWhen;

   XmlPredefToken (int ty, String tx, int fl)
   {
      type = ty;
      text = tx;
      ignoreWhen = fl;
   }

};

public class XmlToken
{
   public static final int EOF = 0;                         // No more tokens
   public static final int TAG_OPEN_CDATA = 1;              // <![CDATA[
   public static final int TAG_OPEN_SSI = 2;                // <!--#
   public static final int TAG_OPEN_COMMENT = 3;            // <!--
   public static final int TAG_OPEN_COMMAND = 4;            // <!
   public static final int TAG_OPEN_END = 5;                // </
   public static final int TAG_OPEN_PROCESS = 6;            // <?
   public static final int TAG_OPEN = 7;                    // <
   public static final int TAG_CLOSE_CDATA = 8;             // ]]>
   public static final int TAG_CLOSE_COMMENT = 9;           // -->
   public static final int TAG_CLOSE_EMPTY = 10;            // />
   public static final int TAG_CLOSE_PROCESS = 11;          // ?>
   public static final int TAG_CLOSE = 12;                  // >
   public static final int EQUALS = 13;                     // =
   public static final int TEXT = 14;                       // text, "text", 'text'
   public static final int UNKNOWN = 15;                    // unexpected symbols

   static final int POPF_ACCEPT_ANY      = 0x0000;
   static final int POPF_INSIDE_TAGDEF   = 0x0001;
   static final int POPF_INSIDE_CDATA    = 0x0002;
   static final int POPF_INSIDE_COMMENT  = 0x0004;
   static final int POPF_OUTSIDE_TAGDEF  = 0x0008;

   /*
    * This array must be sorted such that longer tags hide their otherwise-
    * equivalent-but-shorter cousins: that is, "<!--" must appear before "<!".
    *
    */

   static final int NOT_PREDEF_TOKEN = -1;

   public static XmlPredefToken[] aPredefTokens = {
      new XmlPredefToken( TAG_OPEN_CDATA,        "<![CDATA[",  POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_OPEN_SSI,          "<!--#",      POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_OPEN_COMMENT,      "<!--",       POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_OPEN_COMMAND,      "<!",         POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_OPEN_END,          "</",         POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_OPEN_PROCESS,      "<?",         POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_OPEN,              "<",          POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_CLOSE_CDATA,       "]]>",        POPF_INSIDE_COMMENT ),
      new XmlPredefToken( TAG_CLOSE_COMMENT,     "-->",        POPF_OUTSIDE_TAGDEF | POPF_INSIDE_CDATA ),
      new XmlPredefToken( TAG_CLOSE_EMPTY,       "/>",         POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA | POPF_OUTSIDE_TAGDEF ),
      new XmlPredefToken( TAG_CLOSE_PROCESS,     "?>",         POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA | POPF_OUTSIDE_TAGDEF ),
      new XmlPredefToken( TAG_CLOSE,             ">",          POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA | POPF_OUTSIDE_TAGDEF ),
      new XmlPredefToken( EQUALS,                "=",          POPF_INSIDE_COMMENT | POPF_INSIDE_CDATA | POPF_OUTSIDE_TAGDEF ),
   };

   public static int cPredefTokens = aPredefTokens.length;


   public int type;
   public String text;

   public XmlToken()
   {
      type = UNKNOWN;
      text = null;
   }

   public XmlToken (int tt, String ss)
   {
      type = tt;
      text = ss;
   }

   public XmlToken (XmlTextStream ts, int popf)
   {
      read (ts, popf);
   }

   int read (XmlTextStream ts, int popf)
   {
      for (;;)
      {
         pop (ts, popf);
         if (type != TAG_OPEN_COMMENT)
            break;

         for (;;)
         {
            pop (ts, POPF_INSIDE_COMMENT);
            if (type == EOF || type == TAG_CLOSE_COMMENT)
               break;
         }
      }
      return type;
   }

   void pop (XmlTextStream ts, int popf)
   {
      type = UNKNOWN;
      text = new String();

      // Strip leading whitespace

      while (ts.iswhiteeol())
         ts.advance();

      // If this is the end of the stream, we're done. Otherwise, this next
      // character is the trigger that decides what this token will be.

      char ch;
      int iPredefToken;

      if ((ch = ts.peek()) == 0)
      {
         type = EOF;
      }
      else if (ch == '"')
      {
         ts.advance();
         type = TEXT;

         while ((ch = ts.read()) != 0)
         {
            if (ch == '"')
               break;
            text += ch;
         }
      }
      else if ((iPredefToken = checkForToken (ts, popf)) != NOT_PREDEF_TOKEN)
      {
         type = aPredefTokens[iPredefToken].type;
         text = aPredefTokens[iPredefToken].text;
         ts.advance (text.length());
      }
      else
      {
         boolean fBreakOnWhite = ((popf == POPF_ACCEPT_ANY) || ((popf & POPF_INSIDE_TAGDEF) != 0));
         boolean fSawWhite = false;
         type = TEXT;

         while ((ch = ts.peek()) != 0)
         {
            boolean fIsWhite = ts.iswhiteeol(ch);
            if (fBreakOnWhite && fIsWhite)
               break;
            if (checkForToken (ts, popf) != NOT_PREDEF_TOKEN)
               break;

            if (!fIsWhite)
            {
               if (fSawWhite)
                  text += ' ';
               text += ch;
            }
            fSawWhite = fIsWhite;

            ts.advance();
         }
      }
   }

   int checkForToken (XmlTextStream ts, int popf)
   {
      char ch;
      if ((ch = ts.peek()) != 0)
      {
         for (int ii = 0; ii < cPredefTokens; ++ii)
         {
            if ((aPredefTokens[ii].ignoreWhen & popf) != 0)
               continue;
            if (ts.matches (aPredefTokens[ii].text))
               return ii;
         }
      }
      return NOT_PREDEF_TOKEN;
   }
}

